@@ -14,6 +14,6 @@ ant release
 
 jarsigner -verbose -sigalg MD5withRSA -digestalg SHA1 bin/mlterm-release-unsigned.apk mlterm
 #"${ANDROID_SDK_PATH}/platform-tools/adb" connect localhost
-"${ANDROID_SDK_PATH}/platform-tools/adb" uninstall mlterm.native_activity
-"${ANDROID_SDK_PATH}/platform-tools/adb" install bin/mlterm-release-unsigned.apk
+#"${ANDROID_SDK_PATH}/platform-tools/adb" uninstall mlterm.native_activity
+"${ANDROID_SDK_PATH}/platform-tools/adb" install -r bin/mlterm-release-unsigned.apk
 "${ANDROID_SDK_PATH}/platform-tools/adb" logcat
