@@ -11,6 +11,7 @@
 #include  <kiklib/kik_debug.h>
 
 #include  "../x_window.h"
+#include  "../../common/c_animgif.c"
 
 
 #define  DISP_IS_INITED   (_disp.display)
@@ -20,9 +21,11 @@
 
 static x_display_t  _disp ;
 static Display  _display ;
+static int  rotate_display ;
 static int  locked ;
 
 static int  visible_frame_changed ;
+static int  new_yoffset ;
 static u_int  new_width ;
 static u_int  new_height ;
 
@@ -40,26 +43,23 @@ display_lock(void)
 	{
 		ANativeWindow_Buffer  buf ;
 
-		while( 1)
+		if( ANativeWindow_lock( _display.app->window , &buf , NULL) != 0)
 		{
-			if( ANativeWindow_lock( _display.app->window , &buf , NULL) != 0)
-			{
-				return  0 ;
-			}
+			return  0 ;
+		}
 
-			if( _display.buf.bits && _display.buf.bits != buf.bits)
+		if( _display.buf.bits != buf.bits)
+		{
+			if( _display.buf.bits &&
+			    _display.buf.height == buf.height &&
+			    _display.buf.stride == buf.stride)
 			{
 				memcpy( buf.bits , _display.buf.bits ,
-					_display.buf.stride * _display.buf.height *
+					_display.buf.height * _display.buf.stride *
 					_display.bytes_per_pixel) ;
-				ANativeWindow_unlockAndPost( _display.app->window) ;
 			}
-			else
-			{
-				_display.buf = buf ;
 
-				break ;
-			}
+			_display.buf = buf ;
 		}
 
 		locked = 1 ;
@@ -74,7 +74,8 @@ get_fb(
 	int  y
 	)
 {
-	return  _display.buf.bits + (y * _display.buf.stride + x) * _display.bytes_per_pixel ;
+	return  ((u_char*)_display.buf.bits) +
+		((_display.yoffset + y) * _display.buf.stride + x) * _display.bytes_per_pixel ;
 }
 
 static int
@@ -306,6 +307,52 @@ process_key_event(
 	return  1 ;
 }
 
+static void
+show_soft_input(
+	JavaVM *  vm
+	)
+{
+	JNIEnv *  env ;
+	jobject  this ;
+
+	(*vm)->AttachCurrentThread( vm , &env , NULL) ;
+
+	this = _display.app->activity->clazz ;
+	(*env)->CallVoidMethod( env , this ,
+		(*env)->GetMethodID( env , (*env)->GetObjectClass( env , this) ,
+			"showSoftInput" , "()V")) ;
+
+	(*vm)->DetachCurrentThread(vm) ;
+}
+
+/*
+ * _disp.roots[1] is ignored.
+ * x and y are rotated values.
+ */
+static inline x_window_t *
+get_window(
+	int  x ,	/* X in display */
+	int  y		/* Y in display */
+	)
+{
+	x_window_t *  win ;
+	u_int  count ;
+
+	for( count = 0 ; count < _disp.roots[0]->num_of_children ; count++)
+	{
+		if( ( win = _disp.roots[0]->children[count])->is_mapped)
+		{
+			if( win->x <= x && x < win->x + ACTUAL_WIDTH(win) &&
+			    win->y <= y && y < win->y + ACTUAL_HEIGHT(win))
+			{
+				return  win ;
+			}
+		}
+	}
+
+	return  _disp.roots[0] ;
+}
+
 static int
 process_mouse_event(
 	int  source ,
@@ -318,6 +365,8 @@ process_mouse_event(
 	if( source & AINPUT_SOURCE_MOUSE)
 	{
 		XButtonEvent  xev ;
+		static int  click_num ;
+		x_window_t *  win ;
 
 		switch( action & AMOTION_EVENT_ACTION_POINTER_INDEX_MASK)
 		{
@@ -363,10 +412,53 @@ process_mouse_event(
 		}
 
 		xev.time = time / 1000000 ;
-		xev.x = x ;
-		xev.y = y ;
+		if( rotate_display)
+		{
+			if( rotate_display > 0)
+			{
+				xev.x = y ;
+				xev.y = _disp.height - x - 1 ;
+			}
+			else
+			{
+				xev.x = _disp.width - y - 1 ;
+				xev.y = x ;
+			}
+		}
+		else
+		{
+			xev.x = x ;
+			xev.y = y ;
+		}
 		xev.state = _display.key_state ;
 
+		if( xev.type == ButtonPress)
+		{
+			if( xev.x + xev.y + 20 >= _disp.width + _disp.height)
+			{
+				if( click_num == 0)
+				{
+					click_num = 1 ;
+				}
+				else /* if( click_num == 1) */
+				{
+					click_num = 0 ;
+
+				#if  0
+					/* This doesn't work on Android 3.x and 4.x. */
+					ANativeActivity_showSoftInput( _display.app->activity ,
+						ANATIVEACTIVITY_SHOW_SOFT_INPUT_FORCED) ;
+				#else
+					show_soft_input( _display.app->activity->vm) ;
+				#endif
+				}
+			}
+			else
+			{
+				click_num = 0 ;
+			}
+		}
+
 	#if  0
 		kik_debug_printf( KIK_DEBUG_TAG
 			"Button is %s x %d y %d btn %d time %d\n" ,
@@ -375,7 +467,11 @@ process_mouse_event(
 			xev.x , xev.y , xev.button , xev.time) ;
 	#endif
 
-		x_window_receive_event( _disp.roots[0] , &xev) ;
+		win = get_window( xev.x , xev.y) ;
+		xev.x -= win->x ;
+		xev.y -= win->y ;
+
+		x_window_receive_event( win , &xev) ;
 	}
 
 	return  1 ;
@@ -391,16 +487,17 @@ on_input_event(
 	{
 	case  AINPUT_EVENT_TYPE_MOTION:
 	#if  0
-		kik_debug_printf( "MOTION %d %d %d x %d y %d\n" ,
+		kik_debug_printf( "MOTION %d %d x %d y %d\n" ,
 			AInputEvent_getSource( event) ,
-			AMotionEvent_getAction( event) , AMotionEvent_getEventTime( event) ,
-			AMotionEvent_getX( event , 0) , AMotionEvent_getY( event , 0)) ;
+			AMotionEvent_getAction( event) ,
+			AMotionEvent_getX( event , 0) ,
+			AMotionEvent_getY( event , 0) - _display.yoffset) ;
 	#endif
 		return  process_mouse_event( AInputEvent_getSource( event) ,
 				AMotionEvent_getAction( event) ,
 				AMotionEvent_getEventTime( event) ,
 				AMotionEvent_getX( event , 0) ,
-				AMotionEvent_getY( event , 0)) ;
+				AMotionEvent_getY( event , 0) - _display.yoffset) ;
 
 	case  AINPUT_EVENT_TYPE_KEY:
 	#if  0
@@ -450,44 +547,26 @@ init_window(
 	{
 		_disp.width = ANativeWindow_getWidth( window) ;
 		_disp.height = ANativeWindow_getHeight( window) ;
+
+		if( rotate_display)
+		{
+			u_int  tmp ;
+
+			tmp = _disp.width ;
+			_disp.width = _disp.height ;
+			_disp.height = tmp ;
+		}
 	}
 	else
 	{
 		/* Changed in visibleFrameChanged. */
 	}
 
-#if  0
-	switch( ANativeWindow_getFormat( window))
-	{
-	case  WINDOW_FORMAT_RGBA_8888:
-		_disp.depth = 32 ;
-		_display.bytes_per_pixel = 4 ;
-		_display.rgbinfo = rgbinfos[1] ;
-		break ;
-
-	case  WINDOW_FORMAT_RGBX_8888:
-		_disp.depth = 24 ;
-		_display.bytes_per_pixel = 4 ;
-		_display.rgbinfo = rgbinfos[1] ;
-		break ;
-
-	case  WINDOW_FORMAT_RGB_565:
-	default:
-		_disp.depth = 16 ;
-		_display.bytes_per_pixel = 2 ;
-		_display.rgbinfo = rgbinfos[0] ;
-	}
-
-	ANativeWindow_setBuffersGeometry( window ,
-		_disp.width , _disp.height ,
-		ANativeWindow_getFormat( window)) ;
-#else
 	_disp.depth = 16 ;
 	_display.bytes_per_pixel = 2 ;
 	_display.rgbinfo = rgbinfos[0] ;
 
 	ANativeWindow_setBuffersGeometry( window , 0 , 0 , WINDOW_FORMAT_RGB_565) ;
-#endif
 
 	if( _display.buf.bits)
 	{
@@ -504,6 +583,9 @@ init_window(
 		{
 			u_int  count ;
 
+			/* In case of locked is 1 here. */
+			locked = 0 ;
+
 			/* mlterm paused and restarted. */
 			for( count = 0 ; count < _disp.num_of_roots ; count++)
 			{
@@ -544,6 +626,8 @@ on_app_cmd(
 	#ifdef  DEBUG
 		kik_debug_printf( "TERM_WINDOW\n") ;
 	#endif
+		locked = -1 ;	/* Don't lock until APP_CMD_INIT_WINDOW after restart. */
+
 		break ;
 
 	case APP_CMD_WINDOW_REDRAW_NEEDED:
@@ -730,14 +814,14 @@ x_display_remove_root(
 
 void
 x_display_idling(
-	x_display_t *  disp
+	x_display_t *  disp	/* ignored */
 	)
 {
 	u_int  count ;
 
-	for( count = 0 ; count < disp->num_of_roots ; count ++)
+	for( count = 0 ; count < _disp.num_of_roots ; count ++)
 	{
-		x_window_idling( disp->roots[count]) ;
+		x_window_idling( _disp.roots[count]) ;
 	}
 }
 
@@ -804,11 +888,52 @@ x_display_set_use_ansi_colors(
 {
 }
 
+void
+x_display_rotate(
+	int  rotate
+	)
+{
+	if( rotate == rotate_display)
+	{
+		return ;
+	}
+
+	if( rotate_display + rotate != 0)
+	{
+		u_int  tmp ;
+
+		tmp = _disp.width ;
+		_disp.width = _disp.height ;
+		_disp.height = tmp ;
+
+		rotate_display = rotate ;
+
+		if( _disp.num_of_roots > 0)
+		{
+			x_window_resize_with_margin( _disp.roots[0] ,
+				_disp.width , _disp.height , NOTIFY_TO_MYSELF) ;
+		}
+	}
+	else
+	{
+		/* If rotate_display == -1 rotate == 1 or vice versa, don't swap. */
+
+		rotate_display = rotate ;
+
+		if( _disp.num_of_roots > 0)
+		{
+			x_window_update_all( _disp.roots[0]) ;
+		}
+	}
+}
+
+
 int
 x_display_init(
 	struct android_app *  app
 	)
 {
+	int  ret ;
 	int  ident ;
 	int  events ;
 	struct android_poll_source *  source ;
@@ -816,6 +941,9 @@ x_display_init(
 	/* Make sure glue isn't stripped. */
 	app_dummy() ;
 
+	ret = _display.app ? 0 : 1 ;
+	_display.app = app ;
+
 	app->onAppCmd = on_app_cmd ;
 
 	do
@@ -843,18 +971,7 @@ x_display_init(
 					_display.sensor_man , app->looper ,
 					LOOPER_ID_USER , NULL , NULL) ;
 
-	if( _display.app)
-	{
-		_display.app = app ;
-
-		return  0 ;
-	}
-	else
-	{
-		_display.app = app ;
-
-		return  1 ;
-	}
+	return  ret ;
 }
 
 void
@@ -862,7 +979,7 @@ x_display_final(void)
 {
 	if( locked >= 0)
 	{
-		locked = -1 ;	/* unlocked until APP_CMD_INIT_WINDOW after restart. */
+		locked = -1 ;	/* Don't lock until APP_CMD_INIT_WINDOW after restart. */
 		ANativeActivity_finish( _display.app->activity) ;
 	}
 }
@@ -876,8 +993,10 @@ x_display_process_event(
 	if( visible_frame_changed)
 	{
 		/* XXX should synchronize */
+		_display.yoffset = new_yoffset ;
 		_disp.width = new_width ;
 		_disp.height = new_height ;
+
 		visible_frame_changed = 0 ;
 
 		if( _disp.num_of_roots > 0)
@@ -903,10 +1022,12 @@ x_display_process_event(
 			while( ASensorEventQueue_getEvents(
 				_display.sensor_evqueue , &event , 1) > 0)
 			{
+			#if  0
 				kik_debug_printf( "Accelerometer: x=%f y=%f z=%f" ,
 					event.acceleration.x ,
 					event.acceleration.y ,
 					event.acceleration.z) ;
+			#endif
 			}
 		}
 	}
@@ -918,7 +1039,7 @@ x_display_process_event(
 		kik_debug_printf( KIK_DEBUG_TAG " destroy requested.\n") ;
 	#endif
 
-		locked = -1 ;
+		locked = -1 ;	/* Don't lock until APP_CMD_INIT_WINDOW after restart. */
 
 		return  0 ;
 	}
@@ -946,7 +1067,6 @@ x_display_get_str(
 	JavaVM *  vm ;
 	jobject  this ;
 	jstring  jstr_key ;
-	char *  key ;
 	size_t  len ;
 
 	vm = _display.app->activity->vm ;
@@ -960,6 +1080,8 @@ x_display_get_str(
 
 	if( jstr_key)
 	{
+		const char *  key ;
+
 		key = (*env)->GetStringUTFChars( env , jstr_key , NULL) ;
 
 		if( ( len = strlen(key)) > seq_len)
@@ -992,6 +1114,24 @@ x_display_get_pixel(
 {
 	u_char *  fb ;
 
+	if( rotate_display)
+	{
+		int  tmp ;
+
+		if( rotate_display > 0)
+		{
+			tmp = x ;
+			x = _disp.height - y - 1 ;
+			y = tmp ;
+		}
+		else
+		{
+			tmp = x ;
+			x = y ;
+			y = _disp.width - tmp - 1 ;
+		}
+	}
+
 	fb = get_fb( x , y) ;
 
 	if( _display.bytes_per_pixel == 4)
@@ -1015,7 +1155,54 @@ x_display_put_image(
 {
 	if( display_lock())
 	{
-		memcpy( get_fb( x , y) , image , size) ;
+		if( rotate_display)
+		{
+			/* Display is rotated. */
+
+			u_char *  fb ;
+			int  tmp ;
+			int  line_length ;
+			size_t  count ;
+
+			tmp = x ;
+			if( rotate_display > 0)
+			{
+				x = _disp.height - y - 1 ;
+				y = tmp ;
+				line_length = _display.buf.stride * _display.bytes_per_pixel ;
+			}
+			else
+			{
+				x = y ;
+				y = _disp.width - tmp - 1 ;
+				line_length = -(_display.buf.stride * _display.bytes_per_pixel) ;
+			}
+
+			fb = get_fb( x , y) ;
+
+			if( _display.bytes_per_pixel == 2)
+			{
+				size /= 2 ;
+				for( count = 0 ; count < size ; count++)
+				{
+					*((u_int16_t*)fb) = ((u_int16_t*)image)[count] ;
+					fb += line_length ;
+				}
+			}
+			else /* if( _display.bytes_per_pixel == 4) */
+			{
+				size /= 4 ;
+				for( count = 0 ; count < size ; count++)
+				{
+					*((u_int32_t*)fb) = ((u_int32_t*)image)[count] ;
+					fb += line_length ;
+				}
+			}
+		}
+		else
+		{
+			memcpy( get_fb( x , y) , image , size) ;
+		}
 	}
 }
 
@@ -1048,6 +1235,36 @@ x_display_copy_lines(
 		u_int  count ;
 		size_t  line_length ;
 
+		if( rotate_display)
+		{
+			int  tmp ;
+
+			if( rotate_display > 0)
+			{
+				tmp = src_x ;
+				src_x = _disp.height - src_y - height ;
+				src_y = tmp ;
+
+				tmp = dst_x ;
+				dst_x = _disp.height - dst_y - height ;
+				dst_y = tmp ;
+			}
+			else
+			{
+				tmp = src_x ;
+				src_x = src_y ;
+				src_y = _disp.width - tmp - width ;
+
+				tmp = dst_x ;
+				dst_x = dst_y ;
+				dst_y = _disp.width - tmp - width ;
+			}
+
+			tmp = height ;
+			height = width ;
+			width = tmp ;
+		}
+
 		copy_len = width * _display.bytes_per_pixel ;
 		line_length = _display.buf.stride * _display.bytes_per_pixel ;
 
@@ -1118,23 +1335,152 @@ x_cmap_get_pixel_rgb(
 	return  0 ;
 }
 
+u_char *
+x_display_get_bitmap(
+	char *  path ,
+	u_int *  width ,
+	u_int *  height
+	)
+{
+	JNIEnv *  env ;
+	JavaVM *  vm ;
+	jobject  this ;
+	jintArray  jarray ;
+	char *  image ;
+
+	vm = _display.app->activity->vm ;
+	(*vm)->AttachCurrentThread( vm , &env , NULL) ;
+
+	this = _display.app->activity->clazz ;
+
+	image = NULL ;
+	if( ( jarray = (*env)->CallObjectMethod( env , this ,
+			(*env)->GetMethodID( env ,
+				(*env)->GetObjectClass( env , this) ,
+				"getBitmap" , "(Ljava/lang/String;II)[I") ,
+			(*env)->NewStringUTF( env , path) ,
+			*width , *height)))
+	{
+		jint  len ;
+
+		len = (*env)->GetArrayLength( env , jarray) ;
+
+		if( ( image = malloc( (len - 2) * sizeof(u_int32_t))))
+		{
+			jint *  elems ;
+
+			elems = (*env)->GetIntArrayElements( env , jarray , NULL) ;
+
+			*width = elems[len - 2] ;
+			*height = elems[len - 1] ;
+			memcpy( image , elems , (len - 2) * sizeof(u_int32_t)) ;
+
+			(*env)->ReleaseIntArrayElements( env , jarray , elems , 0) ;
+		}
+	}
+
+	(*vm)->DetachCurrentThread(vm) ;
+
+	return  image ;
+}
+
 
 /* Called in the main thread (not in the native activity thread) */
 void
 Java_mlterm_native_1activity_MLActivity_visibleFrameChanged(
 	JNIEnv *  env ,
 	jobject  this ,
+	jint  yoffset ,
 	jint  width ,
 	jint  height
 	)
 {
 #ifdef  DEBUG
-	kik_debug_printf( "Visible frame changed w %d h %d => w %d h %d\n" ,
-		_disp.width , _disp.height , width , height) ;
+	kik_debug_printf( "Visible frame changed yoff %d w %d h %d => yoff %d w %d h %d\n" ,
+		_display.yoffset , _disp.width , _disp.height , yoffset , width , height) ;
 #endif
 
-	/* XXX should synchronize */
-	new_width = width ;
-	new_height = height ;
-	visible_frame_changed = 1 ;
+	if( width < 50 || height < 50)
+	{
+		/* Don't resize because it may be impossible to show any characters. */
+		return ;
+	}
+
+	if( _disp.roots)
+	{
+		/* XXX should synchronize */
+
+		new_yoffset = yoffset ;
+
+		if( rotate_display)
+		{
+			new_width = height ;
+			new_height = width ;
+		}
+		else
+		{
+			new_width = width ;
+			new_height = height ;
+		}
+
+		visible_frame_changed = 1 ;
+	}
+	else
+	{
+		_display.yoffset = yoffset ;
+
+		if( rotate_display)
+		{
+			_disp.width = height ;
+			_disp.height = width ;
+		}
+		else
+		{
+			_disp.width = width ;
+			_disp.height = height ;
+		}
+	}
+}
+
+jint
+Java_mlterm_native_1activity_MLActivity_hashPath(
+	JNIEnv *  env ,
+	jobject  this ,
+	jstring  jstr	/* must be original URL. (Don't specify /sdcard/.mlterm/anim*) */
+	)
+{
+	const char *  path ;
+	jint  hash ;
+
+	path = (*env)->GetStringUTFChars( env , jstr , NULL) ;
+	hash = hash_path( path) ;
+	(*env)->ReleaseStringUTFChars( env , jstr , path) ;
+
+	return  hash ;
+}
+
+void
+Java_mlterm_native_1activity_MLActivity_splitAnimationGif(
+	JNIEnv *  env ,
+	jobject  this ,
+	jstring  jstr	/* must be original URL. (Don't specify /sdcard/.mlterm/anim*) */
+	)
+{
+	const char *  str ;
+	const char *  path ;
+	char tmp[24 + 5 + 1] ;
+	int  hash ;
+
+	path = str = (*env)->GetStringUTFChars( env , jstr , NULL) ;
+	hash = hash_path( path) ;
+
+	if( strstr( path , "://"))
+	{
+		sprintf( tmp , "/sdcard/.mlterm/anim%d.gif" , hash) ;
+		path = tmp ;
+	}
+
+	split_animation_gif( path , "/sdcard/.mlterm/" , hash) ;
+
+	(*env)->ReleaseStringUTFChars( env , jstr , str) ;
 }
