@@ -13,6 +13,7 @@ echo "Prepare to build for android. (project: ${PROJECT_PATH})"
 echo "Press enter key to continue."
 read
 
+mkdir -p ${PROJECT_PATH}
 cp build.sh ${PROJECT_PATH}/
 cp *.xml ${PROJECT_PATH}/
 
@@ -36,6 +37,8 @@ cp ../main/*.[ch] ${PROJECT_PATH}/jni/main/
 
 mkdir -p ${PROJECT_PATH}/jni/common
 cp ../common/c_imagelib.c ${PROJECT_PATH}/jni/common/
+cp ../common/c_sixel.c ${PROJECT_PATH}/jni/common/
+cp ../common/c_animgif.c ${PROJECT_PATH}/jni/common/
 
 cp jni/Android.mk ${PROJECT_PATH}/jni/
 cp jni/x_event_source.c ${PROJECT_PATH}/jni/xwindow/
