@@ -8,7 +8,6 @@
 #include  <kiklib/kik_list.h>
 #include  <kiklib/kik_debug.h>
 #include  <kiklib/kik_mem.h>
-#include  <kiklib/kik_str.h>	/* kik_compare_str */
 
 #include  <x_im.h>
 
@@ -230,7 +229,7 @@ update_preedit_text(
 
 				if( (*syms->ml_char_combine)( p - 1 , mkf_char_to_int(&ch) ,
 					ch.cs , is_fullwidth , is_comb , fg_color , bg_color ,
-					0 , 0 , 1))
+					0 , 0 , 1 , 0 , 0))
 				{
 					continue ;
 				}
@@ -243,7 +242,7 @@ update_preedit_text(
 			(*syms->ml_char_set)( p , mkf_char_to_int(&ch) , ch.cs ,
 					      is_fullwidth , is_comb ,
 					      fg_color , bg_color ,
-					      0 , 0 , 1) ;
+					      0 , 0 , 1 , 0 , 0) ;
 
 			p ++ ;
 			ibus->im.preedit.filled_len ++ ;
@@ -567,12 +566,6 @@ delete(
 	ibus_proxy_destroy( (IBusProxy*)ibus->context) ;
 #endif
 
-	ref_count -- ;
-
-#ifdef  IM_IBUS_DEBUG
-	kik_debug_printf( KIK_DEBUG_TAG " An object was deleted. ref_count: %d\n", ref_count) ;
-#endif
-
 	kik_list_search_and_remove( im_ibus_t , ibus_list , ibus) ;
 
 	if( ibus->conv)
@@ -591,7 +584,7 @@ delete(
 
 	free( ibus) ;
 
-	if( ref_count == 0)
+	if( -- ref_count == 0)
 	{
 		int  fd ;
 
@@ -633,6 +626,10 @@ delete(
 		}
 	}
 
+#ifdef  IM_IBUS_DEBUG
+	kik_debug_printf( KIK_DEBUG_TAG " An object was deleted. ref_count: %d\n", ref_count) ;
+#endif
+
 	return  ref_count ;
 }
 
