@@ -521,7 +521,7 @@ preedit_changed(
 			if( (*syms->ml_char_combine)( p - 1 , mkf_char_to_int(&ch) ,
 						ch.cs , is_fullwidth , is_comb ,
 						fg_color , bg_color ,
-						0 , 0 , is_underline))
+						0 , 0 , is_underline , 0 , 0))
 			{
 				pos++ ;
 				continue ;
@@ -535,7 +535,7 @@ preedit_changed(
 		(*syms->ml_char_set)( p , mkf_char_to_int(&ch) , ch.cs ,
 				      is_fullwidth , is_comb ,
 				      fg_color , bg_color ,
-				      0 , 0 , is_underline) ;
+				      0 , 0 , is_underline , 0 , 0) ;
 
 		pos++ ;
 		p++ ;
