@@ -114,7 +114,7 @@ dialog_callback(
 	jclass  class ;
 	static jmethodID  mid ;
 
-	if( style != KIK_DIALOG_OKCANCEL)
+	if( style != KIK_DIALOG_OKCANCEL || ! env_for_dialog)
 	{
 		return  -1 ;
 	}
@@ -141,25 +141,6 @@ dialog_callback(
 #endif
 
 
-static int
-true_or_false(
-	char *  str
-	)
-{
-	if( strcmp( str , "true") == 0)
-	{
-		return  1 ;
-	}
-	else if( strcmp( str , "false") == 0)
-	{
-		return  0 ;
-	}
-	else
-	{
-		return  -1 ;
-	}
-}
-
 static void
 set_config(
 	void *  p ,
@@ -172,36 +153,19 @@ set_config(
 
 	nativeObj = p ;
 
-	if( strcmp( key , "encoding") == 0)
+	if( ml_term_set_config( nativeObj->term , key , value))
+	{
+		/* do nothing */
+	}
+	else if( strcmp( key , "encoding") == 0)
 	{
 		ml_char_encoding_t  encoding ;
 
 		if( ( encoding = ml_get_char_encoding( value)) != ML_UNKNOWN_ENCODING)
 		{
-			ml_term_set_auto_encoding( nativeObj->term ,
-				strcasecmp( value , "auto") == 0 ? 1 : 0) ;
-
 			ml_term_change_encoding( nativeObj->term , encoding) ;
 		}
 	}
-	else if( strcmp( key , "tabsize") == 0)
-	{
-		u_int  tab_size ;
-
-		if( kik_str_to_uint( &tab_size , value))
-		{
-			ml_term_set_tab_size( nativeObj->term , tab_size) ;
-		}
-	}
-	else if( strcmp( key , "use_combining") == 0)
-	{
-		int  flag ;
-
-		if( ( flag = true_or_false( value)) != -1)
-		{
-			ml_term_set_use_char_combining( nativeObj->term , flag) ;
-		}
-	}
 	else if( strcmp( key , "use_multi_column_char") == 0)
 	{
 		int  flag ;
@@ -211,33 +175,6 @@ set_config(
 			ml_term_set_use_multi_col_char( nativeObj->term , flag) ;
 		}
 	}
-	else if( strcmp( key , "col_size_of_width_a") == 0)
-	{
-		u_int  size ;
-
-		if( kik_str_to_uint( &size , value))
-		{
-			ml_term_set_col_size_of_width_a( nativeObj->term , size) ;
-		}
-	}
-	else if( strcmp( key , "logging_vt_seq") == 0)
-	{
-		int  flag ;
-
-		if( ( flag = true_or_false( value)) != -1)
-		{
-			ml_term_set_logging_vt_seq( nativeObj->term , flag) ;
-		}
-	}
-	else if( strcmp( key , "use_local_echo") == 0)
-	{
-		int  flag ;
-
-		if( ( flag = true_or_false( value)) != -1)
-		{
-			ml_term_set_use_local_echo( nativeObj->term , flag) ;
-		}
-	}
 }
 
 static void
@@ -249,21 +186,33 @@ get_config(
 	)
 {
 	native_obj_t *  nativeObj ;
+	ml_term_t *  term ;
 	char *  value ;
-	char  digit[DIGIT_STR_LEN(u_int) + 1] ;
-	char  cwd[PATH_MAX] ;
 
 	nativeObj = p ;
 
+	if( dev)
+	{
+		if( ! ( term = ml_get_term( dev)))
+		{
+			return ;
+		}
+	}
+	else
+	{
+		term = nativeObj->term ;
+	}
+
+	if( ml_term_get_config( term , nativeObj->term , key , to_menu , NULL))
+	{
+		return ;
+	}
+
 	value = NULL ;
 
-	if( strcmp( key , "encoding") == 0)
+	if( strcmp( key , "use_multi_column_char") == 0)
 	{
-		value = ml_get_char_encoding_name( ml_term_get_encoding( nativeObj->term)) ;
-	}
-	else if( strcmp( key , "is_auto_encoding") == 0)
-	{
-		if( ml_term_is_auto_encoding( nativeObj->term))
+		if( ml_term_is_using_multi_col_char( term))
 		{
 			value = "true" ;
 		}
@@ -272,110 +221,18 @@ get_config(
 			value = "false" ;
 		}
 	}
-	else if( strcmp( key , "tabsize") == 0)
+	else if( strcmp( key , "pty_list") == 0)
 	{
-		sprintf( digit , "%d" , ml_term_get_tab_size( nativeObj->term)) ;
-		value = digit ;
-	}
-	else if( strcmp( key , "use_combining") == 0)
-	{
-		if( ml_term_is_using_char_combining( nativeObj->term))
-		{
-			value = "true" ;
-		}
-		else
-		{
-			value = "false" ;
-		}
-	}
-	else if( strcmp( key , "use_multi_column_char") == 0)
-	{
-		if( ml_term_is_using_multi_col_char( nativeObj->term))
-		{
-			value = "true" ;
-		}
-		else
-		{
-			value = "false" ;
-		}
-	}
-	else if( strcmp( key , "col_size_of_width_a") == 0)
-	{
-		if( ml_term_get_col_size_of_width_a( nativeObj->term) == 2)
-		{
-			value = "2" ;
-		}
-		else
-		{
-			value = "1" ;
-		}
-	}
-	else if( strcmp( key , "locale") == 0)
-	{
-		value = kik_get_locale() ;
-	}
-	else if( strcmp( key , "pwd") == 0)
-	{
-		value = getcwd( cwd , sizeof(cwd)) ;
-	}
-	else if( strcmp( key , "rows") == 0)
-	{
-		sprintf( digit , "%d" , ml_term_get_logical_rows( nativeObj->term)) ;
-		value = digit ;
-	}
-	else if( strcmp( key , "cols") == 0)
-	{
-		sprintf( digit , "%d" , ml_term_get_logical_cols( nativeObj->term)) ;
-		value = digit ;
-	}
-	else if( strcmp( key , "pty_name") == 0)
-	{
-		if( dev)
-		{
-			if( ( value = ml_term_window_name( nativeObj->term)) == NULL)
-			{
-				value = dev ;
-			}
-		}
-		else
-		{
-			value = ml_term_get_slave_name( nativeObj->term) ;
-		}
-	}
-	else if( strcmp( key , "logging_vt_seq") == 0)
-	{
-		if( ml_term_is_logging_vt_seq( nativeObj->term))
-		{
-			value = "true" ;
-		}
-		else
-		{
-			value = "false" ;
-		}
-	}
-	else if( strcmp( key , "use_local_echo") == 0)
-	{
-		if( ml_term_is_using_local_echo( nativeObj->term))
-		{
-			value = "true" ;
-		}
-		else
-		{
-			value = "false" ;
-		}
+		value = ml_get_pty_list() ;
 	}
 
 	if( ! value)
 	{
-		ml_term_write( nativeObj->term , "#error\n" , 7 , to_menu) ;
+		ml_term_response_config( nativeObj->term , "error" , NULL , to_menu) ;
 	}
 	else
 	{
-		ml_term_write( nativeObj->term , "#" , 1 , to_menu) ;
-		ml_term_write( nativeObj->term , key , strlen(key) , to_menu) ;
-		ml_term_write( nativeObj->term , "=" , 1 , to_menu) ;
-		ml_term_write( nativeObj->term , value , strlen(value) , to_menu) ;
-		ml_term_write( nativeObj->term , "\n" , 1 , to_menu) ;
+		ml_term_response_config( nativeObj->term , key , value , to_menu) ;
 	}
 }
 
@@ -530,7 +387,8 @@ need_style(
 
 	if( ml_char_fg_color( ch) != ML_FG_COLOR ||
 	    ml_char_bg_color( ch) != ML_BG_COLOR ||
-	    ml_char_is_underlined( ch) ||
+	    ml_char_underline_style( ch) ||
+	    ml_char_is_crossed_out( ch) ||
 	    (ml_char_font( ch) & (FONT_BOLD|FONT_ITALIC)))
 	{
 		need_style = 2 ;
@@ -543,7 +401,8 @@ need_style(
 	if( prev_ch &&
 	    ml_char_fg_color( ch) == ml_char_fg_color( prev_ch) &&
 	    ml_char_bg_color( ch) == ml_char_bg_color( prev_ch) &&
-	    ml_char_is_underlined( ch) == ml_char_is_underlined( prev_ch) &&
+	    ml_char_underline_style( ch) == ml_char_underline_style( prev_ch) &&
+	    ml_char_is_crossed_out( ch) == ml_char_is_crossed_out( prev_ch) &&
 	    (ml_char_font( ch) & (FONT_BOLD|FONT_ITALIC)) ==
 	    (ml_char_font( prev_ch) & (FONT_BOLD|FONT_ITALIC)) )
 	{
@@ -708,6 +567,7 @@ Java_mlterm_MLTermPty_nativeOpen(
 	static int  use_login_shell ;
 	static int  logging_vt_seq ;
 	static int  use_local_echo ;
+	static int  use_auto_detect ;
 	static char *  public_key ;
 	static char *  private_key ;
 	static char **  default_argv ;
@@ -743,7 +603,7 @@ Java_mlterm_MLTermPty_nativeOpen(
 	{
 		kik_conf_t *  conf ;
 
-		kik_init_prog( "mlterm" , "3.3.2") ;
+		kik_init_prog( "mlterm" , "3.3.8") ;
 		kik_set_sys_conf_dir( CONFIG_PATH) ;
 		kik_locale_init( "") ;
 		kik_sig_child_init() ;
@@ -760,11 +620,9 @@ Java_mlterm_MLTermPty_nativeOpen(
 		utf16_conv = mkf_utf16le_conv_new() ;
 	#endif
 
-		logging_vt_seq = 0 ;
 		tab_size = 8 ;
 		encoding_default = ml_get_char_encoding( "auto") ;
 		is_auto_encoding_default = 1 ;
-		unicode_policy = 0 ;
 
 		if( strcmp( kik_get_lang() , "ja") == 0)
 		{
@@ -777,7 +635,6 @@ Java_mlterm_MLTermPty_nativeOpen(
 
 		use_char_combining = 1 ;
 		use_multi_col_char = 1 ;
-		use_login_shell = 0 ;
 	#ifdef  USE_LOCAL_ECHO_BY_DEFAULT
 		use_local_echo = 1 ;
 	#else
@@ -863,17 +720,6 @@ Java_mlterm_MLTermPty_nativeOpen(
 				}
 			}
 
-			if( ( value = kik_conf_get_value( conf , "use_unicode_property")))
-			{
-				if( strcmp( value , "true") == 0)
-				{
-					if( unicode_policy != ONLY_USE_UNICODE_FONT)
-					{
-						unicode_policy |= USE_UNICODE_PROPERTY ;
-					}
-				}
-			}
-
 			if( ( value = kik_conf_get_value( conf , "col_size_of_width_a")))
 			{
 				kik_str_to_uint( &col_size_a , value) ;
@@ -934,6 +780,19 @@ Java_mlterm_MLTermPty_nativeOpen(
 				}
 			}
 
+			if( ( value = kik_conf_get_value( conf , "auto_detect_encodings")))
+			{
+				ml_set_auto_detect_encodings( value) ;
+			}
+
+			if( ( value = kik_conf_get_value( conf , "use_auto_detect")))
+			{
+				if( strcmp( value , "true") == 0)
+				{
+					use_auto_detect = 1 ;
+				}
+			}
+
 		#ifdef  USE_LIBSSH2
 			if( ( value = kik_conf_get_value( conf , "ssh_public_key")))
 			{
@@ -1037,21 +896,18 @@ Java_mlterm_MLTermPty_nativeOpen(
 	}
 
 	if( ! ( nativeObj->term = ml_create_term( cols , rows , tab_size , 0 , encoding ,
-					is_auto_encoding , unicode_policy , col_size_a ,
+					is_auto_encoding , use_auto_detect , logging_vt_seq ,
+					unicode_policy , col_size_a ,
 					use_char_combining , use_multi_col_char ,
 					0 /* use_bidi */ , 0 /* bidi_mode */ ,
+					NULL /* bidi_separators */ ,
 					0 /* use_ind */ , 1 /* use_bce */ ,
 					0 /* use_dynamic_comb */ , BSM_STATIC ,
-					0 /* vertical_mode */ , use_local_echo , NULL , NULL)))
+					0 /* vertical_mode */ , use_local_echo , NULL , NULL , 0)))
 	{
 		goto  error ;
 	}
 
-	if( logging_vt_seq)
-	{
-		ml_term_set_logging_vt_seq( nativeObj->term , logging_vt_seq) ;
-	}
-
 	nativeObj->pty_listener.self = nativeObj ;
 	nativeObj->pty_listener.closed = pty_closed ;
 
@@ -1157,7 +1013,11 @@ Java_mlterm_MLTermPty_nativeOpen(
 #endif
 
 	ret = ml_term_open_pty( nativeObj->term , cmd_path , argv , envv ,
-			host , pass , public_key , private_key) ;
+			host , NULL , pass , public_key , private_key) ;
+
+#ifdef  USE_LIBSSH2
+	env_for_dialog = NULL ;
+#endif
 
 	if( jarray_argv)
 	{
@@ -1475,7 +1335,7 @@ Java_mlterm_MLTermPty_nativeWrite(
 	if( *str == '\0')
 	{
 		/* Control+space */
-		ml_term_write( ((native_obj_t*)nativeObj)->term , str , 1 , 0) ;
+		ml_term_write( ((native_obj_t*)nativeObj)->term , str , 1) ;
 	}
 	else
 	{
@@ -1485,7 +1345,7 @@ Java_mlterm_MLTermPty_nativeWrite(
 		       ( len = ml_term_convert_to( ((native_obj_t*)nativeObj)->term ,
 					buf , sizeof(buf) , utf8_parser)) > 0)
 		{
-			ml_term_write( ((native_obj_t*)nativeObj)->term , buf , len , 0) ;
+			ml_term_write( ((native_obj_t*)nativeObj)->term , buf , len) ;
 		}
 
 	#if  0
@@ -1549,6 +1409,7 @@ Java_mlterm_MLTermPty_nativeGetRedrawString(
 	static jfieldID  style_bg_color ;
 	static jfieldID  style_bg_pixel ;
 	static jfieldID  style_underline ;
+	static jfieldID  style_strikeout ;
 	static jfieldID  style_bold ;
 	static jfieldID  style_italic ;
 	jobjectArray  array ;
@@ -1662,6 +1523,8 @@ Java_mlterm_MLTermPty_nativeGetRedrawString(
 								"bg_pixel" , "I") ;
 					style_underline = (*env)->GetFieldID( env , style_class ,
 								"underline" , "Z") ;
+					style_strikeout = (*env)->GetFieldID( env , style_class ,
+								"strikeout" , "Z") ;
 					style_bold = (*env)->GetFieldID( env , style_class ,
 								"bold" , "Z") ;
 					style_italic = (*env)->GetFieldID( env , style_class ,
@@ -1696,7 +1559,12 @@ Java_mlterm_MLTermPty_nativeGetRedrawString(
 
 				(*env)->SetBooleanField( env , styles[num_of_styles - 1] ,
 					style_underline ,
-					ml_char_is_underlined( line->chars + mod_beg + count) ?
+					ml_char_underline_style( line->chars + mod_beg + count) ?
+						JNI_TRUE : JNI_FALSE) ;
+
+				(*env)->SetBooleanField( env , styles[num_of_styles - 1] ,
+					style_strikeout ,
+					ml_char_is_crossed_out( line->chars + mod_beg + count) ?
 						JNI_TRUE : JNI_FALSE) ;
 
 				(*env)->SetBooleanField( env , styles[num_of_styles - 1] ,
@@ -2099,7 +1967,7 @@ Java_mlterm_MLTermPty_nativeReportMouseTracking(
 			seq_len = strlen( seq) ;
 		}
 
-		ml_term_write( nativeObj->term , seq , seq_len , 0) ;
+		ml_term_write( nativeObj->term , seq , seq_len) ;
 
 	#ifdef  __DEBUG
 		kik_debug_printf( KIK_DEBUG_TAG " [reported cursor pos] %d %d\n" , col , row) ;
