@@ -226,6 +226,9 @@ public class MLTerm extends StyledText
 	{
 		if( getTextLimit() != -1 && getTextLimit() < getCharCount() + num)
 		{
+			/* Clear bufOffset, bufStr and bufReplaceLen before removing old text. */
+			replaceTextBuffering( 0 , 0 , null , null) ;
+
 			/* 500 is for buffer. */
 			getContent().replaceTextRange( 0 , getCharCount() + num + 500 - getTextLimit() , "") ;
 
@@ -485,6 +488,7 @@ public class MLTerm extends StyledText
 										colors[ region.styles[count].fg_color] ,
 										colors[ region.styles[count].bg_color]) ;
 					styles[count].underline = region.styles[count].underline ;
+					styles[count].strikeout = region.styles[count].strikeout ;
 					styles[count].fontStyle = (region.styles[count].bold ?
 												SWT.BOLD : SWT.NORMAL) |
 					                          (region.styles[count].italic ?
