@@ -9,13 +9,12 @@
 #include  <sys/socket.h>
 #include  <errno.h>
 #include  <fcntl.h>
-#include  <limits.h>	/* LINE_MAX */
 #include  <signal.h>
 #include  <stdio.h>
 #include  <string.h>	/* memcpy */
 #include  <unistd.h>
 
-#include  "kik_config.h"	/* HAVE_SETSID */
+#include  "kik_def.h"	/* HAVE_SETSID, LINE_MAX */
 #include  "kik_debug.h"
 #include  "kik_mem.h"	/* realloc/free */
 
@@ -386,9 +385,9 @@ start_pty_helper(void)
 	int  tmp[2] ;
 	int  tunnel ;
 	
-	if( access( LIBEXECDIR "/gnome-pty-helper" , X_OK) != 0)
+	if( access( LIBEXECDIR "/mlterm/gnome-pty-helper" , X_OK) != 0)
 	{
-		kik_error_printf( "Couldn't run %s" , LIBEXECDIR "/gnome-pty-helper");
+		kik_error_printf( "Couldn't run %s" , LIBEXECDIR "/mlterm/gnome-pty-helper");
 		
 		return  0 ;
 	}
@@ -439,7 +438,7 @@ start_pty_helper(void)
 		close( tunnel) ;
 		close( pty_helper_tunnel) ;
 		
-		execl(LIBEXECDIR "/gnome-pty-helper" , "gnome-pty-helper" , NULL) ;
+		execl(LIBEXECDIR "/mlterm/gnome-pty-helper" , "gnome-pty-helper" , NULL) ;
 		
 		exit( EXIT_SUCCESS) ;
 	}
