@@ -511,3 +511,117 @@ kik_str_replace(
 
 	return  new_str ;
 }
+
+#if  0
+char *
+kik_str_escape_backslash(
+	char *  str
+	)
+{
+	char *  escaped_str ;
+	char *  p ;
+
+	if( ! ( p = escaped_str = malloc( strlen(str) +
+					kik_count_char_in_str( str , '\\') + 1)))
+	{
+		return  str ;
+	}
+
+	while( 1)
+	{
+		*(p++) = *str ;
+
+		if( *str == '\0')
+		{
+			g_free( str) ;
+
+			return  escaped_str ;
+		}
+		else if( *str == '\\')
+		{
+			*(p++) = '\\' ;
+		}
+
+		str ++ ;
+	}
+}
+#endif
+
+char *
+kik_str_unescape(
+	const char *  str
+	)
+{
+	char *  new_str ;
+	char *  p ;
+
+	if( ( new_str = malloc( strlen( str) + 1)) == NULL)
+	{
+		return  NULL ;
+	}
+
+	for( p = new_str ; *str ; str++ , p++)
+	{
+		if( *str == '\\')
+		{
+			u_int  digit ;
+
+			if( *(++str) == '\0')
+			{
+				break ;
+			}
+			else if( sscanf( str , "x%2x" , &digit) == 1)
+			{
+				*p = (char)digit ;
+				str += 2 ;
+			}
+			else if( *str == 'n')
+			{
+				*p = '\n' ;
+			}
+			else if( *str == 'r')
+			{
+				*p = '\r' ;
+			}
+			else if( *str == 't')
+			{
+				*p = '\t' ;
+			}
+			else if( *str == 'e' || *str == 'E')
+			{
+				*p = '\033' ;
+			}
+			else
+			{
+				*p = *str ;
+			}
+		}
+		else if( *str == '^')
+		{
+			if( *(++str) == '\0')
+			{
+				break ;
+			}
+			else if( '@' <= *str && *str <= '_')
+			{
+				*p = *str - 'A' + 1 ;
+			}
+			else if( *str == '?')
+			{
+				*p = '\x7f' ;
+			}
+			else
+			{
+				*p = *str ;
+			}
+		}
+		else
+		{
+			*p = *str ;
+		}
+	}
+
+	*p = '\0' ;
+
+	return  new_str ;
+}
