@@ -10,8 +10,8 @@
 
 kik_utmp_t
 kik_utmp_new(
-	char *  tty ,
-	char *  host ,
+	const char *  tty ,
+	const char *  host ,
 	int  pty_fd
 	)
 {
