@@ -1,10 +1,20 @@
 #include "indian.h"
-#include <ctype.h>	/* isprint */
 
 char *binsearch(struct tabl *table, int sz, char *word) {
 	int result, index, lindex, hindex;
 
-	if (isprint(word[0])) return word;
+	if (word[1] == '\0') {
+		if (0xf1 <= ((unsigned char*)word)[0] && ((unsigned char*)word)[0] <= 0xfa) {
+			/* is digit */
+			word[0] -= 0xc1;
+			return word;
+		}
+		else if ( ((unsigned char*)word)[0] == 0xea) {
+			/* full stop */
+			word[0] = 0x2a;
+			return word;
+		}
+	}
 
 	lindex = 0 ;
 	hindex = sz ;
@@ -23,27 +33,8 @@ char *binsearch(struct tabl *table, int sz, char *word) {
 	}
 }
 
-char *illdefault(struct tabl *table, char *wrd, int sz) {
-	
-	int i;
-	char *ostr;
-	char iwrd[2];
-	char *p;
-	
-	ostr = (char *) calloc(1000 , sizeof(char));
-
-	for(i = 0; i < strlen(wrd); i++) {
-		iwrd[0]=wrd[i];
-		iwrd[1]='\0';
-		if((p=binsearch(table, sz, iwrd)))
-			strcat(ostr,p);
-	}
-
-	return ostr;
-}
-
 int iscii2font(struct tabl *table, char *input, char *output, int sz) {
-	bzero(output,strlen(output));
-	strcat(output , (char *) split(table, input, sz));
+	memset(output, 0, strlen(output));
+	strcat(output, (char *) split(table, input, sz));
 	return strlen(output); 
 }
