@@ -10,9 +10,10 @@ void  ml_line_set_use_bidi(void) ;
 void  ml_line_bidi_convert_logical_char_index_to_visual(void) ;
 void  ml_line_bidi_convert_visual_char_index_to_logical(void) ;
 void  ml_line_bidi_copy_logical_str(void) ;
+void  ml_line_bidi_is_rtl(void) ;
 void  ml_bidi_copy(void) ;
 void  ml_bidi_reset(void) ;
-void  ml_line_bidi_is_rtl(void) ;
+void  ml_line_bidi_need_shape(void) ;
 
 
 /* --- global variables --- */
@@ -30,5 +31,6 @@ void *  ml_ctl_bidi_func_table[MAX_CTL_BIDI_FUNCS] =
 	ml_is_arabic_combining ,
 	ml_bidi_copy ,
 	ml_bidi_reset ,
+	ml_line_bidi_need_shape ,
 
 } ;
