@@ -10,6 +10,7 @@ void  ml_line_set_use_iscii(void) ;
 void  ml_line_iscii_convert_logical_char_index_to_visual(void) ;
 void  ml_iscii_copy(void) ;
 void  ml_iscii_reset(void) ;
+void  ml_line_iscii_need_shape(void) ;
 
 
 /* --- global variables --- */
@@ -26,5 +27,6 @@ void *  ml_ctl_iscii_func_table[MAX_CTL_ISCII_FUNCS] =
 	ml_iscii_shape_new ,
 	ml_iscii_copy ,
 	ml_iscii_reset ,
+	ml_line_iscii_need_shape ,
 
 } ;
