@@ -29,6 +29,7 @@ typedef struct  bidi_logical_visual
 	int  cursor_logical_col ;
 	int  ltr_rtl_meet_pos ;
 	ml_bidi_mode_t  bidi_mode ;
+	const char *  separators ;
 
 } bidi_logical_visual_t ;
 
@@ -98,12 +99,41 @@ bidi_logical_rows(
 	return  logvis->model->num_of_rows ;
 }
 
+static void
+bidi_render_line(
+	ml_logical_visual_t *  logvis ,
+	ml_line_t *  line
+	)
+{
+	int  need_render ;
+
+	need_render = 0 ;
+
+	if( ! ml_line_is_using_bidi( line))
+	{
+		ml_line_set_use_bidi( line , 1) ;
+
+		need_render = 1 ;
+	}
+
+	if( ml_line_is_modified( line) || need_render)
+	{
+		if( ! ml_line_bidi_render( line ,
+			((bidi_logical_visual_t*)logvis)->bidi_mode ,
+			((bidi_logical_visual_t*)logvis)->separators))
+		{
+		#ifdef  DEBUG
+			kik_warn_printf( KIK_DEBUG_TAG " ml_line_bidi_render failed.\n") ;
+		#endif
+		}
+	}
+}
+
 static int
 bidi_render(
 	ml_logical_visual_t *  logvis
 	)
 {
-	ml_line_t *  line ;
 	int  row ;
 
 	if( logvis->is_visual)
@@ -116,29 +146,7 @@ bidi_render(
 	 */
 	for( row = 0 ; row < logvis->model->num_of_rows ; row ++)
 	{
-		int  need_render ;
-
-		line = ml_model_get_line( logvis->model , row) ;
-
-		need_render = 0 ;
-		
-		if( ! ml_line_is_using_bidi( line))
-		{
-			ml_line_set_use_bidi( line , 1) ;
-
-			need_render = 1 ;
-		}
-
-		if( ml_line_is_modified( line) || need_render)
-		{
-			if( ! ml_line_bidi_render( line ,
-						((bidi_logical_visual_t*)logvis)->bidi_mode))
-			{
-			#ifdef  DEBUG
-				kik_warn_printf( KIK_DEBUG_TAG " ml_line_bidi_render failed.\n") ;
-			#endif
-			}
-		}
+		bidi_render_line( logvis , ml_model_get_line( logvis->model , row)) ;
 	}
 
 	return  1 ;
@@ -220,7 +228,7 @@ bidi_logical(
 		if( ! ml_line_bidi_logical( ml_model_get_line( logvis->model , row)))
 		{
 		#ifdef  __DEBUG
-			kik_debug_printf( KIK_DEBUG_TAG " visualize row %d failed.\n" , row) ;
+			kik_debug_printf( KIK_DEBUG_TAG " logicalize row %d failed.\n" , row) ;
 		#endif
 		}
 	}
@@ -244,27 +252,8 @@ bidi_visual_line(
 	ml_line_t *  line
 	)
 {
-	int  need_render ;
+	bidi_render_line( logvis , line) ;
 
-	need_render = 0 ;
-	
-	if( ! ml_line_is_using_bidi( line))
-	{
-		ml_line_set_use_bidi( line , 1) ;
-
-		need_render = 1 ;
-	}
-
-	if( ml_line_is_modified( line) || need_render)
-	{
-		if( ! ml_line_bidi_render( line , ((bidi_logical_visual_t*)logvis)->bidi_mode))
-		{
-		#ifdef  DEBUG
-			kik_warn_printf( KIK_DEBUG_TAG " ml_line_bidi_render failed.\n") ;
-		#endif
-		}
-	}
-	
 	if( ! ml_line_bidi_visual( line))
 	{
 	#ifdef  DEBUG
@@ -280,7 +269,8 @@ bidi_visual_line(
 
 ml_logical_visual_t *
 ml_logvis_bidi_new(
-	ml_bidi_mode_t  bidi_mode
+	ml_bidi_mode_t  bidi_mode ,
+	const char *  separators
 	)
 {
 	bidi_logical_visual_t *  bidi_logvis ;
@@ -291,6 +281,7 @@ ml_logvis_bidi_new(
 	}
 	
 	bidi_logvis->bidi_mode = bidi_mode ;
+	bidi_logvis->separators = separators ;
 
 	bidi_logvis->logvis.delete = bidi_delete ;
 	bidi_logvis->logvis.init = bidi_init ;
