@@ -26,11 +26,6 @@ typedef struct  iscii_logical_visual
 {
 	ml_logical_visual_t  logvis ;
 	
-	ml_line_t *  logical_lines ;
-	
-	u_int  logical_num_of_cols ;
-	u_int  logical_num_of_rows ;
-
 	int  cursor_logical_char_index ;
 	int  cursor_logical_col ;
 
@@ -43,26 +38,6 @@ typedef struct  iscii_logical_visual
  * ISCII logical <=> visual methods
  */
 
-static int
-iscii_delete_cached_lines(
-	iscii_logical_visual_t *  iscii_logvis
-	)
-{
-	int  row ;
-
-	for( row = 0 ; row < iscii_logvis->logical_num_of_rows ; row ++)
-	{
-		ml_line_final( &iscii_logvis->logical_lines[row]) ;
-	}
-
-	free( iscii_logvis->logical_lines) ;
-	iscii_logvis->logical_lines = NULL ;
-	iscii_logvis->logical_num_of_rows = 0 ;
-	iscii_logvis->logical_num_of_cols = 0 ;
-
-	return  1 ;
-}
-
 static int
 iscii_delete(
 	ml_logical_visual_t *  logvis
@@ -70,8 +45,6 @@ iscii_delete(
 {
 	int  row ;
 
-	iscii_delete_cached_lines( (iscii_logical_visual_t*) logvis) ;
-	
 	if( logvis->model)
 	{
 		for( row = 0 ; row < logvis->model->num_of_rows ; row ++)
@@ -95,7 +68,7 @@ iscii_init(
 	logvis->model = model ;
 	logvis->cursor = cursor ;
 
-	return  iscii_delete_cached_lines( (iscii_logical_visual_t*) logvis) ;
+	return  1 ;
 }
 
 static u_int
@@ -114,62 +87,49 @@ iscii_logical_rows(
 	return  logvis->model->num_of_rows ;
 }
 
+static void
+iscii_render_line(
+	ml_logical_visual_t *  logvis ,
+	ml_line_t *  line
+	)
+{
+	int  need_render ;
+
+	if( ml_line_is_empty( line))
+	{
+		return ;
+	}
+
+	need_render = 0 ;
+
+	if( ! ml_line_is_using_iscii( line))
+	{
+		ml_line_set_use_iscii( line , 1) ;
+
+		need_render = 1 ;
+	}
+
+	if( ml_line_is_modified( line) || need_render)
+	{
+		if( ! ml_line_iscii_render( line))
+		{
+		#ifdef  DEBUG
+			kik_warn_printf( KIK_DEBUG_TAG " ml_line_iscii_render failed.\n") ;
+		#endif
+		}
+	}
+}
+
 static int
 iscii_render(
 	ml_logical_visual_t *  logvis
 	)
 {
-	ml_line_t *  line ;
 	int  row ;
 
 	for( row = 0 ; row < logvis->model->num_of_rows ; row ++)
 	{
-		int  need_render ;
-
-		line = ml_model_get_line( logvis->model , row) ;
-
-		if( ml_line_is_empty( line))
-		{
-			continue ;
-		}
-
-		need_render = 0 ;
-
-		if( ! ml_line_is_using_iscii( line))
-		{
-			ml_line_set_use_iscii( line , 1) ;
-
-			need_render = 1 ;
-		}
-
-		if( ml_line_is_modified( line) || need_render)
-		{
-			if( ! ml_line_iscii_render( line))
-			{
-			#ifdef  DEBUG
-				kik_warn_printf( KIK_DEBUG_TAG " ml_line_iscii_render failed.\n") ;
-			#endif
-			}
-		}
-	}
-
-	return  1 ;
-}
-
-static int
-copy_color_reversed_flag(
-	ml_line_t *  dst ,
-	ml_line_t *  src
-	)
-{
-	int  col ;
-	u_int  copy_len ;
-	
-	copy_len = K_MIN(src->num_of_filled_chars,dst->num_of_filled_chars) ;
-
-	for( col = 0 ; col < copy_len ; col ++)
-	{
-		ml_char_copy_color_reversed_flag( dst->chars + col , src->chars + col) ;
+		iscii_render_line( logvis , ml_model_get_line( logvis->model , row)) ;
 	}
 
 	return  1 ;
@@ -181,7 +141,6 @@ iscii_visual(
 	)
 {
 	iscii_logical_visual_t *  iscii_logvis ;
-	ml_line_t *  line ;
 	int  row ;
 
 	if( logvis->is_visual)
@@ -191,67 +150,19 @@ iscii_visual(
 
 	iscii_logvis = (iscii_logical_visual_t*) logvis ;
 
-	if( iscii_logvis->logical_num_of_rows != logvis->model->num_of_rows ||
-		iscii_logvis->logical_num_of_cols != logvis->model->num_of_cols)
-	{
-		/* ml_model_t resized */
-		
-		void *  p ;
-
-		if( iscii_logvis->logical_lines)
-		{
-			for( row = 0 ; row < iscii_logvis->logical_num_of_rows ; row ++)
-			{
-				ml_line_final( &iscii_logvis->logical_lines[row]) ;
-			}
-		}
-
-		if( iscii_logvis->logical_num_of_rows != logvis->model->num_of_rows)
-		{
-			if( ( p = realloc( iscii_logvis->logical_lines ,
-					sizeof( ml_line_t) * logvis->model->num_of_rows)) == NULL)
-			{
-				free( iscii_logvis->logical_lines) ;
-				iscii_logvis->logical_lines = NULL ;
-				
-				return  0 ;
-			}
-
-			iscii_logvis->logical_lines = p ;
-			
-			iscii_logvis->logical_num_of_rows = logvis->model->num_of_rows ;
-		}
-		
-		for( row = 0 ; row < iscii_logvis->logical_num_of_rows ; row ++)
-		{
-			ml_line_init( &iscii_logvis->logical_lines[row] ,
-					logvis->model->num_of_cols) ;
-		}
-
-		iscii_logvis->logical_num_of_cols = logvis->model->num_of_cols ;
-	}
-
 	for( row = 0 ; row < logvis->model->num_of_rows ; row ++)
 	{
-		line = ml_model_get_line( logvis->model , row) ;
-
-		if( ml_line_is_empty( line))
+		if( ! ml_line_iscii_visual( ml_model_get_line( logvis->model , row)))
 		{
-			continue ;
+		#ifdef  __DEBUG
+			kik_debug_printf( KIK_DEBUG_TAG " visualize row %d failed.\n" , row) ;
+		#endif
 		}
-
-		/* caching */
-		ml_line_copy_line( &iscii_logvis->logical_lines[row] , line) ;
-
-		ml_line_iscii_visual( line) ;
-
-		/* XXX Adhoc implementation for shrinking the amount of memory usage XXX */
-		copy_color_reversed_flag( line , &iscii_logvis->logical_lines[row]) ;
 	}
 
 	iscii_logvis->cursor_logical_char_index = logvis->cursor->char_index ;
 	iscii_logvis->cursor_logical_col = logvis->cursor->col ;
-	
+
 	logvis->cursor->char_index = ml_line_iscii_convert_logical_char_index_to_visual(
 					CURSOR_LINE(logvis) , logvis->cursor->char_index) ;
 	logvis->cursor->col = ml_convert_char_index_to_col( CURSOR_LINE(logvis) ,
@@ -284,39 +195,43 @@ iscii_logical(
 
 	for( row = 0 ; row < logvis->model->num_of_rows ; row ++)
 	{
-		ml_line_t *  vis_line ;
+		ml_line_t *  line ;
 
-		vis_line = ml_model_get_line( logvis->model , row) ;
+		line = ml_model_get_line( logvis->model , row) ;
 
-		/*
-		 * XXX Adhoc implementation for shrinking the amount of memory usage XXX
-		 *
-		 * If logical lines are changed by vt sequences,
-		 * reversed flag will be all cleared.
-		 */
-		copy_color_reversed_flag( &iscii_logvis->logical_lines[row] , vis_line) ;
-
-		/*
-		 * If line is drawin in visual mode, cache lines are also updated.
-		 */
-		if( ! ml_line_is_modified( vis_line))
+		if( ! ml_line_iscii_logical( line))
 		{
-			ml_line_set_updated( &iscii_logvis->logical_lines[row]) ;
+		#ifdef  __DEBUG
+			kik_debug_printf( KIK_DEBUG_TAG " logicalize row %d failed.\n" , row) ;
+		#endif
 		}
-		
-		ml_line_copy_line( vis_line , &iscii_logvis->logical_lines[row]) ;
+
+		/* XXX see ml_iscii_visual */
+	#if  1
+		if( line->num_of_chars > logvis->model->num_of_cols)
+		{
+			ml_str_final( line->chars + logvis->model->num_of_cols ,
+				line->num_of_chars - logvis->model->num_of_cols) ;
+			line->num_of_chars = logvis->model->num_of_cols ;
+
+			/*
+			 * line->num_of_filled_chars is equal or less than line->num_of_chars
+			 * because line is logicalized.
+			 */
+		}
+	#endif
 	}
 
 	logvis->cursor->char_index = iscii_logvis->cursor_logical_char_index ;
 	logvis->cursor->col = iscii_logvis->cursor_logical_col ;
-	
+
 #ifdef  __DEBUG
 	kik_debug_printf( KIK_DEBUG_TAG " [col %d index %d]\n" ,
 		logvis->cursor->col , logvis->cursor->char_index) ;
 #endif
 
 	logvis->is_visual = 0 ;
-	
+
 	return  1 ;
 }
 
@@ -326,8 +241,9 @@ iscii_visual_line(
 	ml_line_t *  line
 	)
 {
+	iscii_render_line( logvis , line) ;
 	ml_line_iscii_visual( line) ;
-	
+
 	return  1 ;
 }
 
