@@ -22,6 +22,9 @@
 #include  <fcntl.h>		/* open */
 #include  <unistd.h>		/* close/pipe */
 #include  <stdio.h>		/* sprintf */
+#ifdef  __CYGWIN__
+#include  <sys/wait.h>
+#endif
 
 
 #ifndef  USE_WIN32API
@@ -77,6 +80,9 @@ typedef struct ml_pty_ssh
 	ssh_session_t *  session ;
 	LIBSSH2_CHANNEL *  channel ;
 
+	char *  lo_buf ;
+	size_t  lo_size ;
+
 } ml_pty_ssh_t ;
 
 typedef struct scp
@@ -796,6 +802,13 @@ write_to_pty(
 {
 	ssize_t  ret ;
 
+#if  defined(USE_WIN32API) || defined(HAVE_PTHREAD)
+	if( ((ml_pty_ssh_t*)pty)->session->doing_scp)
+	{
+		return  0 ;
+	}
+#endif
+
 	ret = libssh2_channel_write( ((ml_pty_ssh_t*)pty)->channel , buf , len) ;
 
 	if( ret == LIBSSH2_ERROR_SOCKET_SEND ||
@@ -821,6 +834,30 @@ read_pty(
 {
 	ssize_t  ret ;
 
+	if( ((ml_pty_ssh_t*)pty)->lo_buf)
+	{
+		if( ((ml_pty_ssh_t*)pty)->lo_size < len)
+		{
+			len = ((ml_pty_ssh_t*)pty)->lo_size ;
+		}
+
+		memcpy( buf , ((ml_pty_ssh_t*)pty)->lo_buf , len) ;
+
+		/* XXX */
+		free( ((ml_pty_ssh_t*)pty)->lo_buf) ;
+		((ml_pty_ssh_t*)pty)->lo_buf = NULL ;
+		((ml_pty_ssh_t*)pty)->lo_size = 0 ;
+
+		return  len ;
+	}
+
+#if  defined(USE_WIN32API) || defined(HAVE_PTHREAD)
+	if( ((ml_pty_ssh_t*)pty)->session->doing_scp)
+	{
+		return  0 ;
+	}
+#endif
+
 	ret = libssh2_channel_read( ((ml_pty_ssh_t*)pty)->channel , buf , len) ;
 
 #ifdef  USE_WIN32API
@@ -846,7 +883,7 @@ scp_stop(
 	ml_pty_ssh_t *  pty_ssh
 	)
 {
-	pty_ssh->session->doing_scp = 0 ;
+	pty_ssh->session->doing_scp = -1 ;
 
 	return  1 ;
 }
@@ -946,6 +983,22 @@ error1:
 }
 #endif	/* USE_WIN32API */
 
+#ifdef  __CYGWIN__
+static void
+check_sig_child(
+	pid_t  pid
+	)
+{
+	/* SIGCHLD signal isn't delivered on cygwin even if mlconfig exits. */
+	int  status ;
+
+	if( pid > 0 && waitpid( pid , &status , WNOHANG) == pid)
+	{
+		kik_trigger_sig_child( pid) ;
+	}
+}
+#endif
+
 static ssize_t
 lo_read_pty(
 	ml_pty_t *  pty ,
@@ -953,6 +1006,10 @@ lo_read_pty(
 	size_t  len
 	)
 {
+#ifdef  __CYGWIN__
+	check_sig_child( pty->config_menu.pid) ;
+#endif
+
 	return  read( pty->master , buf , len) ;
 }
 
@@ -963,6 +1020,10 @@ lo_write_to_pty(
 	size_t  len
 	)
 {
+#ifdef  __CYGWIN__
+	check_sig_child( pty->config_menu.pid) ;
+#endif
+
 	if( len == 1 && buf[0] == '\x03')
 	{
 		/* ^C */
@@ -1045,11 +1106,29 @@ unuse_loopback(
 	ml_pty_t *  pty
 	)
 {
+	char  buf[128] ;
+	ssize_t  len ;
+
 	if( ! pty->stored || --(pty->stored->ref_count) > 0)
 	{
 		return  1 ;
 	}
 
+	while( ( len = (*pty->read)( pty , buf , sizeof(buf))) > 0)
+	{
+		char *  p ;
+
+		if( ! ( p = realloc( ((ml_pty_ssh_t*)pty)->lo_buf ,
+				((ml_pty_ssh_t*)pty)->lo_size + len)))
+		{
+			break ;
+		}
+
+		memcpy( p + ((ml_pty_ssh_t*)pty)->lo_size , buf , len) ;
+		((ml_pty_ssh_t*)pty)->lo_buf = p ;
+		((ml_pty_ssh_t*)pty)->lo_size += len ;
+	}
+
 #ifdef  USE_WIN32API
 	if( pty->read == lo_recv_pty)
 	{
@@ -1085,10 +1164,11 @@ scp_thread(
 {
 	scp_t *  scp ;
 	size_t  rd_len ;
-	char  buf[1024] ;
+	char  buf[8192] ;
 	int  progress ;
 	char  msg1[] = "\x1b[?25l\r\nTransferring data.\r\n|" ;
 	char  msg2[] = "**************************************************|\x1b[?25h\r\n" ;
+	char  err_msg[] = "\r\nInterrupted.\x1b[?25h\r\n" ;
 
 #if  ! defined(USE_WIN32API) && defined(HAVE_PTHREAD)
 	pthread_detach( pthread_self()) ;
@@ -1101,7 +1181,7 @@ scp_thread(
 
 	ml_write_to_pty( &scp->pty_ssh->pty , msg1 , sizeof(msg1) - 1) ;
 
-	while( rd_len < scp->src_size && scp->pty_ssh->session->doing_scp)
+	while( rd_len < scp->src_size && scp->pty_ssh->session->doing_scp > 0)
 	{
 		int  new_progress ;
 		ssize_t  len ;
@@ -1163,7 +1243,15 @@ scp_thread(
 		#endif
 		}
 	}
-	ml_write_to_pty( &scp->pty_ssh->pty , msg2 , sizeof(msg2) - 1) ;
+
+	if( scp->pty_ssh->session->doing_scp > 0)
+	{
+		ml_write_to_pty( &scp->pty_ssh->pty , msg2 , sizeof(msg2) - 1) ;
+	}
+	else
+	{
+		ml_write_to_pty( &scp->pty_ssh->pty , err_msg , sizeof(err_msg) - 1) ;
+	}
 
 #if  1
 	kik_usleep( 100000) ;	/* Expect to switch to main thread and call ml_read_pty(). */
@@ -1390,7 +1478,7 @@ x11_callback(
 	if( connect( display_sock , (struct sockaddr *)&addr , sizeof(addr)) < 0)
 	{
 	error:
-		kik_msg_printf( "Failed to connect X Server.\n") ;
+		kik_error_printf( "Failed to connect X Server.\n") ;
 		closesocket( display_sock) ;
 		display_sock = -1 ;
 
@@ -1977,6 +2065,7 @@ ml_pty_ssh_scp_intern(
 {
 	scp_t *  scp ;
 	struct stat  st ;
+	char *  msg ;
 
 	/* Note that session is non-block mode in this context. */
 
@@ -2066,6 +2155,14 @@ ml_pty_ssh_scp_intern(
 		goto  error ;
 	}
 
+	if( ( msg = alloca( 24 + strlen(src_path) + strlen(dst_path) + 1)))
+	{
+		sprintf( msg , "\r\nSCP: %s%s => %s%s" ,
+			src_is_remote ? "remote:" : "local:" , src_path ,
+			src_is_remote ? "local:" : "remote:" , dst_path) ;
+		ml_write_to_pty( pty , msg , strlen(msg)) ;
+	}
+
 #if  defined(USE_WIN32API)
 	{
 		HANDLE  thrd ;
@@ -2168,10 +2265,12 @@ ml_pty_ssh_poll(
 	{
 		u_int  idx ;
 
+	#if  defined(USE_WIN32API) || defined(HAVE_PTHREAD)
 		if( sessions[count]->doing_scp)
 		{
 			continue ;
 		}
+	#endif
 
 		for( idx = 0 ; idx < sessions[count]->num_of_ptys ; idx++)
 		{
@@ -2199,6 +2298,9 @@ ml_pty_ssh_poll(
 	return  num_of_fds ;
 }
 
+/*
+ * The returned fds can contain -1 which means the failure of x11_callback().
+ */
 u_int
 ml_pty_ssh_get_x11_fds(
 	int **  fds
@@ -2273,6 +2375,13 @@ ml_pty_ssh_send_recv_x11(
 
 	session = sessions[count] ;
 
+#if  defined(USE_WIN32API) || defined(HAVE_PTHREAD)
+	if( session->doing_scp)
+	{
+		return  0 ;
+	}
+#endif
+
 	if( session->x11_fds[idx] == -1 ||	/* Failed to connect X server */
 	    ! ( ( ! bidirection ||
 	          xserver_to_ssh( session->x11_channels[idx] , session->x11_fds[idx])) &&
