@@ -9,17 +9,31 @@
 #include  <kiklib/kik_util.h>	/* K_MIN */
 
 
-#define  MLFONT(attr)  (((attr) >> 5) & 0x7fff)
+#define  UNDERLINE_STYLE(attr)  (((attr) >> 21) & 0x3)
+
+#define  IS_ZEROWIDTH(attr)  ((attr) & (0x1 << 20))
+
+/* Combination of UNICODE_AREA, IS_ITALIC, IS_BOLD, IS_FULLWIDTH and CHARSET */
+#define  MLFONT(attr) \
+	((attr) & (0x1 << 17)) ?	/* is unicode area cs or not */ \
+	((((attr) >> 5) & 0xe00) | ISO10646_UCS4_1 | (((attr) << 7) & 0x1ff000)) : \
+	(((attr) >> 5) & 0xfff)
+
+#define  IS_VISIBLE(attr)  ((attr) & (0x1 << 19))
+#define  IS_BLINKING(attr)  ((attr) & (0x1 << 18))
 #define  IS_ITALIC(attr)  ((attr) & (0x1 << 16))
 #define  IS_BOLD(attr)  ((attr) & (0x1 << 15))
 #define  IS_FULLWIDTH(attr)  ((attr) & (0x1 << 14))
-#define  CHARSET(attr)  (((attr) >> 5) & 0x1ff)
+#define  CHARSET(attr) \
+	((attr) & (0x1 << 17)) ?	/* is unicode area cs or not */ \
+	ISO10646_UCS4_1 : \
+	(((attr) >> 5) & 0x1ff)
 
 #define  IS_REVERSED(attr)  ((attr) & (0x1 << 4))
 #define  REVERSE_COLOR(attr) ((attr) |= (0x1 << 4))
 #define  RESTORE_COLOR(attr) ((attr) &= ~(0x1 << 4))
 
-#define  IS_UNDERLINED(attr)  ((attr) & (0x1 << 3))
+#define  IS_CROSSED_OUT(attr)  ((attr) & (0x1 << 3))
 
 #define  IS_COMB(attr)  ((attr) & (0x1 << 2))
 
@@ -31,15 +45,15 @@
 #define  USE_MULTI_CH(attr)  ((attr) &= 0xfffffe)
 #define  UNUSE_MULTI_CH(attr)  ((attr) |= 0x1)
 
-#define  COMPOUND_ATTR(charset,is_fullwidth,is_bold,is_italic,unicode_area,is_underlined,is_comb) \
-	( ((unicode_area) << 17) | ((is_italic) << 16) | ((is_bold) << 15) | \
-	  ((is_fullwidth) << 14) | ((charset) << 5) | ( 0x0 << 4) |  \
-	  ((is_underlined) << 3) | ((is_comb) << 2) | ( 0x0 << 1) | 0x1)
+#define  COMPOUND_ATTR(charset,is_zerowidth,is_fullwidth,is_bold,is_italic,is_unicode_area_cs,underline_style,is_crossed_out,is_blinking,is_comb) \
+	( ((underline_style) << 21) | ((is_zerowidth) << 20) | (0x1 << 19) | \
+	  ((is_blinking) << 18) | ((is_unicode_area_cs) << 17) | ((is_italic) << 16) | \
+	  ((is_bold) << 15) | ((is_fullwidth) << 14) | ((charset) << 5) | \
+	  ((is_crossed_out) << 3) | ((is_comb) << 2) | 0x1)
 
 
 /* --- static variables --- */
 
-static int  use_char_combining = 1 ;
 static int  use_multi_col_char = 1 ;
 
 static struct
@@ -73,16 +87,6 @@ get_comb_size(
 
 /* --- global functions --- */
 
-int
-ml_set_use_char_combining(
-	int  use_it
-	)
-{
-	use_char_combining = use_it ;
-
-	return  1 ;
-}
-
 int
 ml_set_use_multi_col_char(
 	int  use_it
@@ -94,17 +98,29 @@ ml_set_use_multi_col_char(
 }
 
 ml_font_t
-ml_char_add_unicode_area_font(
+ml_char_get_unicode_area_font(
 	u_int32_t  min ,
 	u_int32_t  max
 	)
 {
+	u_int  idx ;
 	void *  p ;
 
-	if( num_of_unicode_areas >= 7 ||
+	for( idx = num_of_unicode_areas ; idx > 0 ; idx--)
+	{
+		if( min == unicode_areas[idx - 1].min &&
+		    max == unicode_areas[idx - 1].max)
+		{
+			return  ISO10646_UCS4_1 | (idx << 12) ;
+		}
+	}
+
+	if( num_of_unicode_areas == 511 /* Max is 2^9-1 */ ||
 	    ! ( p = realloc( unicode_areas ,
 			sizeof(*unicode_areas) * (num_of_unicode_areas + 1))))
 	{
+		kik_msg_printf( "No more unicode areas.\n") ;
+
 		return  UNKNOWN_CS ;
 	}
 
@@ -166,21 +182,18 @@ ml_char_set(
 	ml_color_t  bg_color ,
 	int  is_bold ,
 	int  is_italic ,
-	int  is_underlined
+	int  underline_style ,
+	int  is_crossed_out ,
+	int  is_blinking
 	)
 {
 	u_int  idx ;
+	int  is_zerowidth ;
+
 	ml_char_final( ch) ;
 
 	ch->u.ch.code = code ;
 
-#ifdef  DEBUG
-	if( cs >= MAX_CHARSET)
-	{
-		kik_warn_printf(KIK_DEBUG_TAG "charset(%x) is out of range\n", cs) ;
-	}
-#endif
-
 	if( unicode_areas && cs == ISO10646_UCS4_1)
 	{
 		for( idx = num_of_unicode_areas ; idx > 0 ; idx --)
@@ -188,6 +201,7 @@ ml_char_set(
 			if( unicode_areas[idx - 1].min <= code &&
 			    code <= unicode_areas[idx - 1].max)
 			{
+				cs = idx ;
 				break ;
 			}
 		}
@@ -197,8 +211,36 @@ ml_char_set(
 		idx = 0 ;
 	}
 
-	ch->u.ch.attr = COMPOUND_ATTR(cs,is_fullwidth!=0,is_bold!=0,is_italic!=0,idx,
-				is_underlined!=0,is_comb!=0) ;
+#if  1
+	/*
+	 * 0 should be returned for all zero-width characters of Unicode,
+	 * but 0 is returned for following characters alone for now.
+	 * 200C;ZERO WIDTH NON-JOINER
+	 * 200D;ZERO WIDTH JOINER
+	 * 200E;LEFT-TO-RIGHT MARK
+	 * 200F;RIGHT-TO-LEFT MARK
+	 * 202A;LEFT-TO-RIGHT EMBEDDING
+	 * 202B;RIGHT-TO-LEFT EMBEDDING
+	 * 202C;POP DIRECTIONAL FORMATTING
+	 * 202D;LEFT-TO-RIGHT OVERRIDE
+	 * 202E;RIGHT-TO-LEFT OVERRIDE
+	 *
+	 * see is_noconv_unicode() in ml_vt100_parser.c
+	 */
+	if( cs == ISO10646_UCS4_1 &&
+	    ( (0x200c <= code && code <= 0x200f) || (0x202a <= code && code <= 0x202e)))
+	{
+		is_zerowidth = 1 ;
+	}
+	else
+#endif
+	{
+		is_zerowidth = 0 ;
+	}
+
+	ch->u.ch.attr = COMPOUND_ATTR(cs,is_zerowidth,is_fullwidth!=0,is_bold!=0,
+				is_italic!=0,idx>0,underline_style,is_crossed_out!=0,
+				is_blinking!=0,is_comb!=0) ;
 	ch->u.ch.fg_color = fg_color ;
 	ch->u.ch.bg_color = bg_color ;
 
@@ -216,16 +258,13 @@ ml_char_combine(
 	ml_color_t  bg_color ,
 	int  is_bold ,
 	int  is_italic ,
-	int  is_underlined
+	int  underline_style ,
+	int  is_crossed_out ,
+	int  is_blinking
 	)
 {
 	ml_char_t *  multi_ch ;
 
-	if( ! use_char_combining)
-	{
-		return  0 ;
-	}
-
 	/*
 	 * This check should be excluded, because characters whose is_comb flag
 	 * (combining property of mkf) is NULL can be combined
@@ -238,15 +277,17 @@ ml_char_combine(
 	}
 #endif
 
-#ifdef  DEBUG
-	if( cs >= MAX_CHARSET)
-	{
-		kik_warn_printf(KIK_DEBUG_TAG "charset(%x) is out of range\n", cs) ;
-	}
-#endif
-
 	if( IS_SINGLE_CH(ch->u.ch.attr))
 	{
+		if( IS_ZEROWIDTH(ch->u.ch.attr))
+		{
+			/*
+			 * Zero width characters must not be combined to
+			 * show string like U+09b0 + U+200c + U+09cd + U+09af correctly.
+			 */
+			return  0 ;
+		}
+
 		if( ( multi_ch = malloc( sizeof( ml_char_t) * 2)) == NULL)
 		{
 		#ifdef  DEBUG
@@ -272,7 +313,8 @@ ml_char_combine(
 
 		ml_char_init( multi_ch + 1) ;
 		if( ml_char_set( multi_ch + 1 , code , cs , is_fullwidth , is_comb ,
-			fg_color , bg_color , is_bold , is_italic , is_underlined) == 0)
+			fg_color , bg_color , is_bold , is_italic , underline_style ,
+			is_crossed_out , is_blinking) == 0)
 		{
 			return  0 ;
 		}
@@ -280,7 +322,16 @@ ml_char_combine(
 	else
 	{
 		u_int  comb_size ;
-		
+
+		if( IS_ZEROWIDTH(ch->u.multi_ch->u.ch.attr))
+		{
+			/*
+			 * Zero width characters must not be combined to
+			 * show string like U+09b0 + U+200c + U+09cd + U+09af correctly.
+			 */
+			return  0 ;
+		}
+
 		if( ( comb_size = get_comb_size( ch->u.multi_ch)) >= MAX_COMB_SIZE)
 		{
 		#ifdef  DEBUG
@@ -311,7 +362,8 @@ ml_char_combine(
 		SET_COMB_TRAILING( multi_ch[comb_size].u.ch.attr) ;
 		ml_char_init( multi_ch + comb_size + 1) ;
 		if( ml_char_set( multi_ch + comb_size + 1 , code , cs , is_fullwidth ,
-			is_comb , fg_color , bg_color , is_bold , is_italic , is_underlined) == 0)
+			is_comb , fg_color , bg_color , is_bold , is_italic , underline_style ,
+			is_crossed_out , is_blinking) == 0)
 		{
 			return  0 ;
 		}
@@ -333,7 +385,8 @@ ml_char_combine_simple(
 			IS_FULLWIDTH(comb->u.ch.attr) , IS_COMB(comb->u.ch.attr) ,
 			comb->u.ch.fg_color , comb->u.ch.bg_color ,
 			IS_BOLD(comb->u.ch.attr) , IS_ITALIC(comb->u.ch.attr) ,
-			IS_UNDERLINED(comb->u.ch.attr)) ;
+			UNDERLINE_STYLE(comb->u.ch.attr) , IS_CROSSED_OUT(comb->u.ch.attr) ,
+			IS_BLINKING(comb->u.ch.attr)) ;
 }
 
 ml_char_t *
@@ -371,6 +424,25 @@ ml_get_combining_chars(
 	}
 }
 
+ml_char_t *
+ml_get_picture_char(
+	ml_char_t *  ch
+	)
+{
+	if( ! IS_SINGLE_CH(ch->u.ch.attr))
+	{
+		ch = ch->u.multi_ch ;
+
+		if( IS_COMB_TRAILING(ch->u.ch.attr) &&
+		    CHARSET(ch[1].u.ch.attr) == PICTURE_CHARSET)
+		{
+			return  ch + 1 ;
+		}
+	}
+
+	return  NULL ;
+}
+
 /*
  * Not used for now.
  */
@@ -521,38 +593,14 @@ ml_char_cols(
 
 	if( IS_SINGLE_CH(attr))
 	{
-		if( use_multi_col_char && IS_FULLWIDTH(attr))
+		if( IS_ZEROWIDTH(attr))
+		{
+			return  0 ;
+		}
+		else if( use_multi_col_char && IS_FULLWIDTH(attr))
 		{
 			return  2 ;
 		}
-	#if  1
-		else if( CHARSET(attr) == ISO10646_UCS4_1)
-		{
-			/*
-			 * 0 should be returned for all zero-width characters of Unicode,
-			 * but 0 is returned for following characters alone for now.
-			 * 200C;ZERO WIDTH NON-JOINER
-			 * 200D;ZERO WIDTH JOINER
-			 * 200E;LEFT-TO-RIGHT MARK
-			 * 200F;RIGHT-TO-LEFT MARK
-			 * 202A;LEFT-TO-RIGHT EMBEDDING
-			 * 202B;RIGHT-TO-LEFT EMBEDDING
-			 * 202C;POP DIRECTIONAL FORMATTING
-			 * 202D;LEFT-TO-RIGHT OVERRIDE
-			 * 202E;RIGHT-TO-LEFT OVERRIDE
-			*/
-			u_int32_t  code ;
-
-			code = ch->u.ch.code ;
-
-			/* see is_noconv_unicode() in ml_vt100_parser.c */
-			if( ( 0x200c <= code && code <= 0x200f) ||
-			    ( 0x202a <= code && code <= 0x202e))
-			{
-				return  0 ;
-			}
-		}
-	#endif
 
 		return  1 ;
 	}
@@ -565,7 +613,7 @@ ml_char_cols(
 /*
  * 'use_multi_col_char' not concerned.
  */
-u_int
+int
 ml_char_is_fullwidth(
 	ml_char_t *  ch
 	)
@@ -606,27 +654,14 @@ ml_char_fg_color(
 
 	if( IS_SINGLE_CH(attr))
 	{
-		ml_color_t  color ;
-
 		if( IS_REVERSED(attr))
 		{
-			color = ch->u.ch.bg_color ;
-
-			/*
-			 * xterm doesn't use hilighted color for bold background color.
-			 */
+			return  IS_VISIBLE(attr) ? ch->u.ch.bg_color : ch->u.ch.fg_color ;
 		}
 		else
 		{
-			color = ch->u.ch.fg_color ;
-
-			if( color < MAX_VTSYS_COLORS && IS_BOLD(attr))
-			{
-				color |= ML_BOLD_COLOR_MASK ;
-			}
+			return  IS_VISIBLE(attr) ? ch->u.ch.fg_color : ch->u.ch.bg_color ;
 		}
-
-		return  color ;
 	}
 	else
 	{
@@ -664,33 +699,9 @@ ml_char_bg_color(
 	ml_char_t *  ch
 	)
 {
-	u_int  attr ;
-
-	attr = ch->u.ch.attr ;
-
-	if( IS_SINGLE_CH(attr))
+	if( IS_SINGLE_CH(ch->u.ch.attr))
 	{
-		ml_color_t  color ;
-
-		if( IS_REVERSED(attr))
-		{
-			color = ch->u.ch.fg_color ;
-
-			if( color < MAX_VTSYS_COLORS && IS_BOLD(attr))
-			{
-				color |= ML_BOLD_COLOR_MASK ;
-			}
-		}
-		else
-		{
-			color = ch->u.ch.bg_color ;
-
-			/*
-			 * xterm doesn't use hilighted color for bold background color.
-			 */
-		}
-
-		return  color ;
+		return  IS_REVERSED(ch->u.ch.attr) ? ch->u.ch.fg_color : ch->u.ch.bg_color ;
 	}
 	else
 	{
@@ -724,17 +735,87 @@ ml_char_set_bg_color(
 }
 
 int
-ml_char_is_underlined(
+ml_char_underline_style(
 	ml_char_t *  ch
 	)
 {
 	if( IS_SINGLE_CH(ch->u.ch.attr))
 	{
-		return  IS_UNDERLINED(ch->u.ch.attr) ;
+		return  UNDERLINE_STYLE(ch->u.ch.attr) ;
 	}
 	else
 	{
-		return  ml_char_is_underlined( ch->u.multi_ch) ;
+		return  ml_char_underline_style( ch->u.multi_ch) ;
+	}
+}
+
+int
+ml_char_is_crossed_out(
+	ml_char_t *  ch
+	)
+{
+	if( IS_SINGLE_CH(ch->u.ch.attr))
+	{
+		return  IS_CROSSED_OUT(ch->u.ch.attr) ;
+	}
+	else
+	{
+		return  ml_char_is_crossed_out( ch->u.multi_ch) ;
+	}
+}
+
+int
+ml_char_is_blinking(
+	ml_char_t *  ch
+	)
+{
+	if( IS_SINGLE_CH(ch->u.ch.attr))
+	{
+		return  IS_BLINKING(ch->u.ch.attr) ;
+	}
+	else
+	{
+		return  ml_char_is_blinking( ch->u.multi_ch) ;
+	}
+}
+
+int
+ml_char_set_visible(
+	ml_char_t *  ch ,
+	int  visible
+	)
+{
+	if( IS_SINGLE_CH(ch->u.ch.attr))
+	{
+		if( ! visible)
+		{
+			ch->u.ch.attr &= ~(0x1 << 19) ;
+		}
+		else
+		{
+			ch->u.ch.attr |= (0x1 << 19) ;
+		}
+
+		return  1 ;
+	}
+	else
+	{
+		return  ml_char_set_visible( ch->u.multi_ch , visible) ;
+	}
+}
+
+int
+ml_char_is_visible(
+	ml_char_t *  ch
+	)
+{
+	if( IS_SINGLE_CH(ch->u.ch.attr))
+	{
+		return  IS_VISIBLE(ch->u.ch.attr) ;
+	}
+	else
+	{
+		return  ml_char_is_visible( ch->u.multi_ch) ;
 	}
 }
 
@@ -800,38 +881,6 @@ ml_char_restore_color(
 	}
 }
 
-/* XXX only used in iscii_logical() */
-int
-ml_char_copy_color_reversed_flag(
-	ml_char_t *  dst ,
-	ml_char_t *  src
-	)
-{
-	if( IS_SINGLE_CH(src->u.ch.attr))
-	{
-		if( IS_REVERSED(src->u.ch.attr))
-		{
-			return  ml_char_reverse_color( dst) ;
-		}
-		else
-		{
-			return  ml_char_restore_color( dst) ;
-		}
-	}
-	else
-	{
-		/* See the first character only */
-		if( IS_REVERSED(src->u.multi_ch->u.ch.attr))
-		{
-			return  ml_char_reverse_color( dst) ;
-		}
-		else
-		{
-			return  ml_char_restore_color( dst) ;
-		}
-	}
-}
-
 int
 ml_char_is_null(
 	ml_char_t *  ch
@@ -871,7 +920,12 @@ ml_char_code_is(
 {
 	if( IS_SINGLE_CH(ch->u.ch.attr))
 	{
-		if( CHARSET(ch->u.ch.attr) == cs && ch->u.ch.code == code)
+		/*
+		 * XXX
+		 * gcc 4.8.2 output codes to cause unexpected result without
+		 * () before and after &&.
+		 */
+		if( ( CHARSET(ch->u.ch.attr) == cs) && (ch->u.ch.code == code))
 		{
 			return  1 ;
 		}
@@ -931,7 +985,7 @@ ml_sp_ch(void)
 	{
 		ml_char_init( &sp_ch) ;
 		ml_char_set( &sp_ch , ' ' , US_ASCII , 0 , 0 ,
-			ML_FG_COLOR , ML_BG_COLOR , 0 , 0 , 0) ;
+			ML_FG_COLOR , ML_BG_COLOR , 0 , 0 , 0 , 0 , 0) ;
 	}
 
 	return  &sp_ch ;
@@ -946,7 +1000,7 @@ ml_nl_ch(void)
 	{
 		ml_char_init( &nl_ch) ;
 		ml_char_set( &nl_ch , '\n' , US_ASCII , 0 , 0 ,
-			ML_FG_COLOR , ML_BG_COLOR , 0 , 0 , 0) ;
+			ML_FG_COLOR , ML_BG_COLOR , 0 , 0 , 0 , 0 , 0) ;
 	}
 
 	return  &nl_ch ;
