@@ -32,8 +32,17 @@
 
 /* For inline pictures (see x_picture.c) */
 #define  PICTURE_CHARSET  0x1ff
+#define  PICTURE_ID_BITS   9	/* fg or bg color */
+#define  PICTURE_POS_BITS  23	/* code */
 
 
+enum
+{
+	UNDERLINE_NONE ,
+	UNDERLINE_NORMAL ,
+	UNDERLINE_DOUBLE ,
+} ;
+
 /*
  * This object size should be kept as small as possible.
  * (ILP32: 64bit) (LP64: 64bit)
@@ -54,14 +63,17 @@ typedef struct ml_char
 			/*
 			 * attr member contents.
 			 * Total 23 bit
-			 * 3 bit : not used
-			 * 3 bit : unicode area(0x0 - 0x7)
+			 * 2 bit : underline_style(0 or 1 or 2)
+			 * 1 bit : is_zerowidth(0 or 1)
+			 * 1 bit : is_visible(0 or 1)
+			 * 1 bit : is_blinking(0 or 1)
+			 * 1 bit : is unicode area cs(0 or 1)
 			 * 1 bit : is_italic(0 or 1)
 			 * 1 bit : is_bold(0 or 1)
 			 * 1 bit : is_fullwidth(0 or 1)
 			 * 9 bit : charset(0x0 - 0x1ff)
 			 * 1 bit : is_reversed(0 or 1)	... used for X Selection
-			 * 1 bit : is_underlined(0 or 1)
+			 * 1 bit : is_crossed_out
 			 * 1 bit : is_comb(0 or 1)
 			 * 1 bit : is_comb_trailing(0 or 1)
 			 * ---
@@ -91,11 +103,9 @@ typedef struct ml_char
 } ml_char_t ;
 
 
-int  ml_set_use_char_combining( int  use_it) ;
-
 int  ml_set_use_multi_col_char( int  use_it) ;
 
-ml_font_t  ml_char_add_unicode_area_font( u_int32_t  min , u_int32_t  max) ;
+ml_font_t  ml_char_get_unicode_area_font( u_int32_t  min , u_int32_t  max) ;
 
 int  ml_char_init( ml_char_t *  ch) ;
 
@@ -103,11 +113,17 @@ int  ml_char_final( ml_char_t *  ch) ;
 
 int  ml_char_set( ml_char_t *  ch , u_int32_t  code , mkf_charset_t  cs ,
 	int  is_fullwidth , int  is_comb , ml_color_t  fg_color , ml_color_t  bg_color ,
-	int  is_bold , int  is_italic , int  is_underlined) ;
+	int  is_bold , int  is_italic , int  underline_style , int  is_crossed_out ,
+	int  is_blinking) ;
 
 int  ml_char_combine( ml_char_t *  ch , u_int32_t  code , mkf_charset_t  cs ,
 	int  is_fullwidth , int  is_comb , ml_color_t  fg_color , ml_color_t  bg_color ,
-	int  is_bold , int  is_italic , int  is_underlined) ;
+	int  is_bold , int  is_italic , int  underline_style , int  is_crossed_out ,
+	int  is_blinking) ;
+
+/* set both fg and bg colors for reversing. */
+#define  ml_char_combine_picture( ch , id , pos) \
+	ml_char_combine( ch , pos , PICTURE_CHARSET , 0 , 0 ,  id , id , 0 , 0 , 0 , 0 , 0)
 
 int  ml_char_combine_simple( ml_char_t *  ch , ml_char_t *  comb) ;
 
@@ -115,6 +131,8 @@ ml_char_t *  ml_get_base_char( ml_char_t *  ch) ;
 
 ml_char_t *  ml_get_combining_chars( ml_char_t *  ch , u_int *  size) ;
 
+ml_char_t *  ml_get_picture_char( ml_char_t *  ch) ;
+
 #if  0
 /*
  * Not used for now.
@@ -136,24 +154,33 @@ ml_font_t  ml_char_font( ml_char_t *  ch) ;
 
 u_int  ml_char_cols( ml_char_t *  ch) ;
 
-u_int  ml_char_is_fullwidth( ml_char_t *  ch) ;
+int  ml_char_is_fullwidth( ml_char_t *  ch) ;
 
 ml_color_t  ml_char_fg_color( ml_char_t *  ch) ;
 
 int  ml_char_set_fg_color( ml_char_t *  ch , ml_color_t  color) ;
 
+#define  ml_char_picture_id( ch)  ml_char_fg_color( ch)
+#define  ml_char_set_picture_id( ch , idx)  ml_char_set_fg_color( ch , idx)
+
 ml_color_t  ml_char_bg_color( ml_char_t *  ch) ;
 
 int  ml_char_set_bg_color( ml_char_t *  ch , ml_color_t  color) ;
 
-int  ml_char_is_underlined( ml_char_t *  ch) ;
+int  ml_char_underline_style( ml_char_t *  ch) ;
+
+int  ml_char_is_crossed_out( ml_char_t *  ch) ;
+
+int  ml_char_is_blinking( ml_char_t *  ch) ;
+
+int  ml_char_set_visible( ml_char_t *  ch , int  visible) ;
+
+int  ml_char_is_visible( ml_char_t *  ch) ;
 
 int  ml_char_reverse_color( ml_char_t *  ch) ;
 
 int  ml_char_restore_color( ml_char_t *  ch) ;
 
-int  ml_char_copy_color_reversed_flag( ml_char_t *  dst , ml_char_t *  src) ;
-
 int  ml_char_is_null( ml_char_t *  ch) ;
 
 int  ml_char_equal( ml_char_t *  ch1 , ml_char_t *  ch2) ;
