@@ -4,6 +4,8 @@
 
 #include  "ml_char_encoding.h"
 
+#include  <stdio.h>			/* sscanf */
+
 #include  <kiklib/kik_str.h>		/* kik_str_alloca_dup */
 #include  <kiklib/kik_debug.h>
 #include  <kiklib/kik_mem.h>		/* alloca */
@@ -41,6 +43,8 @@
 
 #include  <mkf/mkf_iso2022_conv.h>	/* mkf_iso2022_illegal_char */
 
+#include  "ml_drcs.h"
+
 
 typedef struct  encoding_table
 {
@@ -248,6 +252,23 @@ ovrd_iso2022kr_parser_init(
 	(*conv->delete)( conv) ;
 }
 
+static size_t
+iso2022_illegal_char(
+	mkf_conv_t *  conv ,
+	u_char *  dst ,
+	size_t  dst_size ,
+	int *  is_full ,
+	mkf_char_t *  ch
+	)
+{
+	if( ch->cs == ISO10646_UCS4_1)
+	{
+		ml_convert_unicode_pua_to_drcs( ch) ;
+	}
+
+	return  mkf_iso2022_illegal_char( conv , dst , dst_size , is_full , ch) ;
+}
+
 static size_t
 non_iso2022_illegal_char(
 	mkf_conv_t *  conv ,
@@ -430,7 +451,7 @@ ml_conv_new(
 
 	if( IS_ENCODING_BASED_ON_ISO2022(encoding))
 	{
-		conv->illegal_char = mkf_iso2022_illegal_char ;
+		conv->illegal_char = iso2022_illegal_char ;
 
 		if( encoding == ML_ISO2022KR)
 		{
@@ -522,3 +543,22 @@ ml_char_encoding_convert_with_parser(
 
 	return  filled_len ;
 }
+
+int
+ml_parse_unicode_area(
+	const char *  str ,
+	u_int *  min ,
+	u_int *  max
+	)
+{
+	if( sscanf( str , "U+%x-%x" , min , max) != 2)
+	{
+		kik_msg_printf( "Illegal unicode area format: %s\n" , str) ;
+
+		return  0 ;
+	}
+	else
+	{
+		return  1 ;
+	}
+}
