@@ -432,7 +432,12 @@ error2:
 
 			exit(1) ;
 		}
-		
+
+	#if  (defined(__CYGWIN__) || defined(__MSYS__)) && ! defined(DEBUG)
+		/* Suppress error message */
+		close( STDERR_FILENO) ;
+	#endif
+
 	       	execv( cmd_path , args) ;
 
 		/* failed */
@@ -453,7 +458,7 @@ error2:
 			}
 		}
 
-		kik_msg_printf( "Failed to exec %s.\n" , cmd_path) ;
+		kik_error_printf( "Failed to exec %s.\n" , cmd_path) ;
 
 		exit(1) ;
 	}
