@@ -22,6 +22,10 @@
 #define  CURSOR_DEBUG
 #endif
 
+#if  0
+#define  COMPAT_XTERM
+#endif
+
 /*
  * ml_edit_t::tab_stops
  * e.g.)
@@ -76,7 +80,16 @@ insert_chars(
 
 	buf_len = edit->model.num_of_cols ;
 
-	num_of_cols = edit->margin_end + 1 ;
+#ifndef  COMPAT_XTERM
+	if( edit->cursor.col > edit->margin_end)
+	{
+		num_of_cols = edit->model.num_of_cols ;
+	}
+	else
+#endif
+	{
+		num_of_cols = edit->margin_end + 1 ;
+	}
 
 	if( ( buffer = ml_str_alloca( buf_len)) == NULL)
 	{
@@ -141,7 +154,15 @@ insert_chars(
 		filled_cols += cols ;
 	}
 
-	last_index = filled_len ;
+	if( edit->cursor.char_index + filled_len == num_of_cols)
+	{
+		/* cursor position doesn't proceed. */
+		last_index = filled_len - 1 ;
+	}
+	else
+	{
+		last_index = filled_len ;
+	}
 
 	/*
 	 * cursor char
@@ -647,7 +668,7 @@ ml_edit_clone(
 			dst_edit->wraparound_ready_line = dst_line ;
 		}
 
-		ml_line_copy_line( dst_line , src_line) ;
+		ml_line_copy( dst_line , src_line) ;
 	}
 
 	return  1 ;
@@ -749,14 +770,16 @@ ml_edit_insert_chars(
 {
 	reset_wraparound_checker( edit) ;
 
-	if( CURSOR_IS_INSIDE_MARGIN(edit))
-	{
-		return  insert_chars( edit , ins_chars , num_of_ins_chars , 1) ;
-	}
-	else
+#ifdef  COMPAT_XTERM
+	if( ! CURSOR_IS_INSIDE_MARGIN(edit))
 	{
 		return  ml_edit_overwrite_chars( edit , ins_chars , num_of_ins_chars) ;
 	}
+	else
+#endif
+	{
+		return  insert_chars( edit , ins_chars , num_of_ins_chars , 1) ;
+	}
 }
 
 int
@@ -832,13 +855,13 @@ ml_edit_overwrite_chars(
 
 	buf_len = num_of_ow_chars + edit->model.num_of_cols ;
 
-	if( CURSOR_IS_INSIDE_MARGIN(edit))
+	if( edit->cursor.col > edit->margin_end)
 	{
-		num_of_cols = edit->margin_end + 1 ;
+		num_of_cols = edit->model.num_of_cols ;
 	}
 	else
 	{
-		num_of_cols = edit->model.num_of_cols ;
+		num_of_cols = edit->margin_end + 1 ;
 	}
 
 	if( ( buffer = ml_str_alloca( buf_len)) == NULL)
@@ -1344,22 +1367,6 @@ ml_edit_clear_above(
 	}
 }
 
-int
-ml_edit_fill_all(
-	ml_edit_t *  edit ,
-	ml_char_t *  ch
-	)
-{
-	int  row ;
-
-	for( row = 0 ; row < edit->model.num_of_rows ; row ++)
-	{
-		ml_line_clear_with( ml_model_get_line( &edit->model , row) , 0 , ch) ;
-	}
-
-	return  1 ;
-}
-
 int
 ml_edit_set_scroll_region(
 	ml_edit_t *  edit ,
@@ -1419,6 +1426,48 @@ ml_edit_set_scroll_region(
 	return  1 ;
 }
 
+int
+ml_edit_scroll_leftward(
+	ml_edit_t *  edit ,
+	u_int  size
+	)
+{
+	int  orig_mode ;
+
+	orig_mode = edit->is_relative_origin ;
+	edit->is_relative_origin = 1 ;	/* Compatible with RLogin */
+
+	ml_edit_copy_area( edit , size , 0 ,
+		ml_edit_get_cols( edit) - size , ml_edit_get_rows( edit) , 0 , 0) ;
+	ml_edit_erase_area( edit , ml_edit_get_cols( edit) - size , 0 ,
+		size , ml_edit_get_rows( edit)) ;
+
+	edit->is_relative_origin = orig_mode ;
+
+	return  1 ;
+}
+
+int
+ml_edit_scroll_rightward(
+	ml_edit_t *  edit ,
+	u_int  size
+	)
+{
+	int  orig_mode ;
+
+	orig_mode = edit->is_relative_origin ;
+	edit->is_relative_origin = 1 ;	/* Compatible with RLogin */
+
+	ml_edit_copy_area( edit , 0 , 0 ,
+		ml_edit_get_cols( edit) - size , ml_edit_get_rows( edit) , size , 0) ;
+	ml_edit_erase_area( edit , 0 , 0 ,
+		size , ml_edit_get_rows( edit)) ;
+
+	edit->is_relative_origin = orig_mode ;
+
+	return  1 ;
+}
+
 int
 ml_edit_scroll_upward(
 	ml_edit_t *  edit ,
@@ -2000,6 +2049,38 @@ ml_edit_restore_cursor(
 	}
 }
 
+int
+ml_edit_fill_area(
+	ml_edit_t *  edit ,
+	ml_char_t *  ch ,
+	int  col ,
+	int  row ,
+	u_int  num_of_cols ,
+	u_int  num_of_rows
+	)
+{
+	int  char_index ;
+	u_int  cols_rest ;
+	ml_line_t *  line ;
+
+	for( ; num_of_rows > 0 ; num_of_rows --)
+	{
+		line = ml_model_get_line( &edit->model , row++) ;
+
+		char_index = ml_convert_col_to_char_index( line , &cols_rest , col ,
+				BREAK_BOUNDARY) ;
+		if( cols_rest > 0)
+		{
+			ml_line_fill( line , ml_sp_ch() , char_index , cols_rest) ;
+			char_index += cols_rest ;
+		}
+
+		ml_line_fill( line , ch , char_index , num_of_cols) ;
+	}
+
+	return  1 ;
+}
+
 int
 ml_edit_copy_area(
 	ml_edit_t *  edit ,
