@@ -136,6 +136,21 @@ ml_bidi_reset(
 	return  (*func)( state) ;
 }
 
+static int
+ml_line_bidi_need_shape(
+	ml_line_t *  line
+	)
+{
+	int (*func)( ml_line_t *) ;
+
+	if( ! ( func = ml_load_ctl_bidi_func( ML_LINE_BIDI_NEED_SHAPE)))
+	{
+		return  0 ;
+	}
+
+	return  (*func)( line) ;
+}
+
 
 static int
 ml_line_set_use_iscii(
@@ -184,13 +199,29 @@ ml_iscii_reset(
 	return  (*func)( state) ;
 }
 
-#else
+static int
+ml_line_iscii_need_shape(
+	ml_line_t *  line
+	)
+{
+	int (*func)( ml_line_t *) ;
+
+	if( ! ( func = ml_load_ctl_iscii_func( ML_LINE_ISCII_NEED_SHAPE)))
+	{
+		return  0 ;
+	}
+
+	return  (*func)( line) ;
+}
+
+#else	/* NO_DYNAMIC_LOAD_CTL */
 
 #ifndef  USE_FRIBIDI
 #define  ml_line_set_use_bidi( line , flag)  (0)
 #define  ml_line_bidi_convert_visual_char_index_to_logical( line , char_index)  (char_index)
 #define  ml_line_bidi_copy_logical_str( line , dst , beg , len)  (0)
 #define  ml_line_bidi_is_rtl( line)  (0)
+#define  ml_line_bidi_need_shape( line)  (0)
 #define  ml_bidi_copy( dst , src)  (0)
 #define  ml_bidi_reset( state)  (0)
 #else
@@ -199,24 +230,27 @@ int  ml_line_set_use_bidi( ml_line_t *  line , int  flag) ;
 int  ml_line_bidi_convert_visual_char_index_to_logical( ml_line_t *  line , int  char_index) ;
 int  ml_line_bidi_copy_logical_str( ml_line_t *  line , ml_char_t *  dst , int  beg , u_int  len) ;
 int  ml_line_bidi_is_rtl( ml_line_t *  line) ;
+int  ml_line_bidi_need_shape( ml_line_t *  line) ;
 int  ml_bidi_copy( ml_bidi_state_t  dst , ml_bidi_state_t  src) ;
 int  ml_bidi_reset( ml_bidi_state_t  state) ;
 int  ml_line_bidi_convert_logical_char_index_to_visual( ml_line_t *  line , int  char_index ,
 	int *  ltr_rtl_meet_pos) ;
-#endif
+#endif	/* USE_FRIBIDI */
 
 #ifndef  USE_IND
 #define  ml_line_set_use_iscii( line , flag)  (0)
 #define  ml_iscii_copy( dst , src)  (0)
 #define  ml_iscii_reset( state)  (0)
+#define  ml_line_iscii_need_shape( line)  (0)
 #else
 /* Link functions in libctl/ml_*iscii.c */
 int  ml_line_set_use_iscii( ml_line_t *  line , int  flag) ;
+int  ml_line_iscii_need_shape( ml_line_t *  line) ;
 int  ml_iscii_copy( ml_iscii_state_t  dst , ml_iscii_state_t  src) ;
 int  ml_iscii_reset( ml_iscii_state_t  state) ;
 int  ml_line_iscii_convert_logical_char_index_to_visual( ml_line_t *  line ,
 	int  logical_char_index) ;
-#endif
+#endif	/* USE_IND */
 
 #endif	/* NO_DYNAMIC_LOAD_CTL */
 
@@ -253,7 +287,7 @@ ml_line_clone(
 	)
 {
 	ml_line_init( clone , num_of_chars) ;
-	ml_line_copy_line( clone , orig) ;
+	ml_line_copy( clone , orig) ;
 
 	return  1 ;
 }
@@ -1026,11 +1060,6 @@ ml_convert_char_index_to_col(
 	int  count ;
 	int  col ;
 
-	if( IS_EMPTY(line))
-	{
-		return  0 ;
-	}
-
 	if( char_index >= line->num_of_chars)
 	{
 	#ifdef  __DEBUG
@@ -1135,9 +1164,10 @@ ml_line_reverse_color(
 		return  0 ;
 	}
 
-	ml_char_reverse_color( line->chars + char_index) ;
-
-	ml_line_set_modified( line , char_index , char_index) ;
+	if( ml_char_reverse_color( line->chars + char_index))
+	{
+		ml_line_set_modified( line , char_index , char_index) ;
+	}
 
 	return  1 ;
 }
@@ -1153,9 +1183,10 @@ ml_line_restore_color(
 		return  0 ;
 	}
 
-	ml_char_restore_color( line->chars + char_index) ;
-
-	ml_line_set_modified( line , char_index , char_index) ;
+	if( ml_char_restore_color( line->chars + char_index))
+	{
+		ml_line_set_modified( line , char_index , char_index) ;
+	}
 
 	return  1 ;
 }
@@ -1165,7 +1196,7 @@ ml_line_restore_color(
  * But bidi parameters are also copyed as it is.
  */
 int
-ml_line_copy_line(
+ml_line_copy(
 	ml_line_t *  dst ,	/* should be initialized ahead */
 	ml_line_t *  src
 	)
@@ -1218,6 +1249,21 @@ ml_line_copy_line(
 	return  1 ;
 }
 
+int
+ml_line_swap(
+	ml_line_t *  line1 ,
+	ml_line_t *  line2
+	)
+{
+	ml_line_t  tmp ;
+
+	tmp = *line1 ;
+	*line1 = *line2 ;
+	*line2 = tmp ;
+
+	return  1 ;
+}
+
 int
 ml_line_share(
 	ml_line_t *  dst ,
@@ -1410,7 +1456,7 @@ ml_line_convert_logical_char_index_to_visual(
 	}
 
 	if( ml_line_is_using_iscii( line) &&
-	    ( iscii_func = ml_load_ctl_bidi_func(
+	    ( iscii_func = ml_load_ctl_iscii_func(
 				ML_LINE_ISCII_CONVERT_LOGICAL_CHAR_INDEX_TO_VISUAL)))
 	{
 		char_index = (*iscii_func)( line , char_index) ;
@@ -1430,6 +1476,12 @@ ml_line_shape(
 	ml_line_t *  orig ;
 	ml_char_t *  shaped ;
 
+	if( ( ! ml_line_is_using_bidi( line) || ! ml_line_bidi_need_shape( line)) &&
+	    ( ! ml_line_is_using_iscii( line) || ! ml_line_iscii_need_shape( line)))
+	{
+		return  NULL ;
+	}
+
 	if( ( orig = malloc( sizeof( ml_line_t))) == NULL)
 	{
 		return  NULL ;
