@@ -51,6 +51,7 @@ ml_pty_new(
 	char **  cmd_argv ,		/* can be NULL(only if cmd_path is NULL) */
 	char **  env ,			/* can be NULL */
 	const char *  host ,		/* DISPLAY env or remote host */
+	const char *  work_dir ,	/* can be NULL */
 	const char *  pass ,		/* can be NULL */
 	const char *  pubkey ,		/* can be NULL */
 	const char *  privkey ,		/* can be NULL */
@@ -63,7 +64,8 @@ ml_pty_new(
 #ifndef  USE_WIN32API
 	if( ! pass)
 	{
-		pty = ml_pty_unix_new( cmd_path , cmd_argv , env , host , cols , rows) ;
+		pty = ml_pty_unix_new( cmd_path , cmd_argv , env , host ,
+				work_dir , cols , rows) ;
 	}
 	else
 #endif
@@ -78,6 +80,11 @@ ml_pty_new(
 	#endif
 	}
 
+	if( pty)
+	{
+		ml_config_menu_init( &pty->config_menu) ;
+	}
+
 	return  pty ;
 }
 
@@ -104,6 +111,8 @@ ml_pty_new_with(
 		pty = NULL ;
 	}
 
+	ml_config_menu_init( &pty->config_menu) ;
+
 	return  pty ;
 }
 
@@ -129,6 +138,7 @@ ml_pty_delete(
 #endif
 
 	free( pty->buf) ;
+	ml_config_menu_final( &pty->config_menu) ;
 
 	(*pty->final)( pty) ;
 
@@ -338,6 +348,50 @@ ml_read_pty(
 	}
 }
 
+void
+ml_response_config(
+	ml_pty_t *  pty ,
+	char *  key ,
+	char *  value ,
+	int  to_menu
+	)
+{
+	char *  res ;
+	char *  fmt ;
+	size_t  res_len ;
+
+	res_len = 1 + strlen(key) + 1 ;
+	if( value)
+	{
+		res_len += (1 + strlen(value)) ;
+		fmt = "#%s=%s\n" ;
+	}
+	else
+	{
+		fmt = "#%s\n" ;
+	}
+
+	if( ! ( res = alloca( res_len + 1)))
+	{
+		res = "#error\n" ;
+	}
+
+	sprintf( res , fmt , key , value) ;
+
+#ifdef  __DEBUG
+	kik_debug_printf( KIK_DEBUG_TAG " %s\n" , res) ;
+#endif
+
+	if( to_menu)
+	{
+		ml_config_menu_write( &pty->config_menu , res , res_len) ;
+	}
+	else
+	{
+		ml_write_to_pty( pty , res , res_len) ;
+	}
+}
+
 pid_t
 ml_pty_get_pid(
   	ml_pty_t *  pty
@@ -383,7 +437,25 @@ ml_pty_get_slave_name(
 #endif
 
 	/* Virtual pty name */
+#ifdef  USE_LIBSSH2
+	sprintf( virt_name , "/dev/vpty%d" ,
+		((pty->child_pid >> 1) & 0xfff)) ;	/* child_pid == channel */
+#else
 	sprintf( virt_name , "/dev/vpty%d" , pty->master) ;
+#endif
 
 	return  virt_name ;
 }
+
+int
+ml_start_config_menu(
+	ml_pty_t *  pty ,
+	char *  cmd_path ,
+	int  x ,
+	int  y ,
+	char *  display
+	)
+{
+	return  ml_config_menu_start( &pty->config_menu , cmd_path ,
+				x , y , display , pty) ;
+}
