@@ -6,6 +6,7 @@
 #define  __ML_PTY_INTERN_H__
 
 #include  "ml_pty.h"
+#include  "ml_config_menu.h"
 
 
 typedef struct  ml_pty
@@ -26,6 +27,8 @@ typedef struct  ml_pty
 
 	ml_pty_event_listener_t *  pty_listener ;
 
+	ml_config_menu_t  config_menu ;
+
 	struct
 	{
 		int  master ;
@@ -41,7 +44,7 @@ typedef struct  ml_pty
 
 
 ml_pty_t *  ml_pty_unix_new( const char *  cmd_path , char **  cmd_argv , char **  env ,
-	const char *  host , u_int  cols , u_int  rows) ;
+	const char *  host , const char *  work_dir , u_int  cols , u_int  rows) ;
 
 ml_pty_t *  ml_pty_unix_new_with( int  master , int  slave , pid_t  child_pid ,
 	const char *  host , u_int  cols , u_int  rows) ;
