@@ -118,6 +118,23 @@ receive_scrolled_out_line(
 	
 	ml_log_add( &screen->logs , line) ;
 
+	/* XXX see ml_line_iscii_visual() */
+#if  1
+	if( line->num_of_chars > ml_screen_get_logical_cols( screen))
+	{
+		/*
+		 * line->num_of_filled_chars can be more than line->num_of_chars
+		 * without ml_line_reset() here because line is visualized.
+		 */
+		ml_line_reset( line) ;
+		ml_line_set_updated( line) ;
+
+		ml_str_final( line->chars + ml_screen_get_logical_cols( screen) ,
+			line->num_of_chars - ml_screen_get_logical_cols( screen)) ;
+		line->num_of_chars = ml_screen_get_logical_cols( screen) ;
+	}
+#endif
+
 	if( ml_screen_is_backscrolling( screen) == BSM_STATIC)
 	{
 		screen->backscroll_rows ++ ;
@@ -205,9 +222,138 @@ window_scroll_downward_region(
 			screen->screen_listener->self , beg_row , end_row , size) ;
 }
 
+static int
+modify_region(
+	ml_screen_t *  screen ,		/* visual */
+	int *  end_char_index ,
+	int *  end_row
+	)
+{
+	int  row ;
+	ml_line_t *  line ;
+
+	/* Removing empty lines of the end. */
+
+	row = *end_row ;
+
+	while( ( line = ml_screen_get_line( screen , row)) == NULL || ml_line_is_empty( line))
+	{
+		if( -- row < 0 && abs(row) > ml_get_num_of_logged_lines( &screen->logs))
+		{
+			return  0 ;
+		}
+	}
+
+	if( row < *end_row)
+	{
+		if( ml_line_is_rtl( line))
+		{
+			*end_char_index = ml_line_beg_char_index_regarding_rtl( line) ;
+		}
+		else
+		{
+			if( ( *end_char_index =
+				ml_line_get_num_of_filled_chars_except_spaces( line)) > 0)
+			{
+				(*end_char_index) -- ;
+			}
+		}
+
+		*end_row = row ;
+	}
+
+	return  1 ;
+}
+
+static void
+convert_col_to_char_index(
+	ml_screen_t *  screen ,		/* visual */
+	ml_line_t *  line ,
+	int *  beg_char_index ,
+	int *  end_char_index ,	/* end + 1 */
+	int  beg_col ,
+	int  end_col
+	)
+{
+	int  padding ;
+	int  beg ;
+	int  end ;
+	u_int  rest ;
+
+	if( ml_line_is_rtl( line) &&
+	    ( padding = ml_screen_get_cols( screen) -
+	                ml_line_get_num_of_filled_cols( line)) > 0)
+	{
+		beg_col -= padding ;
+		end_col -= padding ;
+	}
+
+	*beg_char_index = ml_convert_col_to_char_index( line , &rest , beg_col , 0) ;
+
+	if( ( end = ml_line_get_num_of_filled_chars_except_spaces( line)) <= *beg_char_index ||
+	         ( end == *beg_char_index + 1 && rest > 0))
+	{
+		*beg_char_index = end ;
+	}
+	else if( ( beg = ml_line_beg_char_index_regarding_rtl( line)) > *beg_char_index)
+	{
+		*beg_char_index = beg ;
+	}
+
+	*end_char_index = ml_convert_col_to_char_index( line , NULL , end_col , 0) + 1 ;
+	if( end < *end_char_index)
+	{
+		*end_char_index = end ;
+	}
+}
+
+static int
+reverse_or_restore_color_rect(
+	ml_screen_t *  screen ,		/* visual */
+	int  beg_col ,
+	int  beg_row ,
+	int  end_col ,
+	int  end_row ,
+	int (*func)( ml_line_t * , int)
+	)
+{
+	int  row ;
+	int  beg_char_index ;
+	int  end_char_index ;
+	ml_line_t *  line ;
+
+	if( beg_col > end_col)
+	{
+		int  col ;
+
+		col = beg_col ;
+		beg_col = end_col ;
+		end_col = col ;
+	}
+
+	for( row = beg_row ; row <= end_row ; row++)
+	{
+		if( ( line = ml_screen_get_line( screen , row)))
+		{
+			int  char_index ;
+
+			convert_col_to_char_index( screen , line ,
+				&beg_char_index , &end_char_index , beg_col , end_col) ;
+
+			for( char_index = beg_char_index ; char_index < end_char_index ;
+			     char_index ++)
+			{
+				(*func)( line , char_index) ;
+			}
+		}
+	}
+
+	return  1 ;
+}
+
 static int
 reverse_or_restore_color(
-	ml_screen_t *  screen ,
+	ml_screen_t *  screen ,		/* visual */
 	int  beg_char_index ,
 	int  beg_row ,
 	int  end_char_index ,
@@ -230,49 +376,20 @@ reverse_or_restore_color(
 	u_int  size_except_spaces ;
 	int  beg_regarding_rtl ;
 
-	row = end_row ;
-
-	/* Removing empty lines of the end. */
-	while( ( line = ml_screen_get_line( screen , row)) == NULL || ml_line_is_empty( line))
+	if( ! modify_region( screen , &end_char_index , &end_row))
 	{
-		if( -- row < 0 && abs(row) > ml_get_num_of_logged_lines( &screen->logs))
-		{
-			return  0 ;
-		}
+		return  0 ;
 	}
 
-	if( row < end_row)
-	{
-		/* Empty lines of the end are removed. */
-
-		if( ml_line_is_rtl( line))
-		{
-			end_char_index = ml_line_beg_char_index_regarding_rtl( line) ;
-		}
-		else
-		{
-			if( ( end_char_index =
-				ml_line_get_num_of_filled_chars_except_spaces( line)) > 0)
-			{
-				end_char_index -- ;
-			}
-		}
-		
-		end_row = row ;
-	}
+	/* Removing empty lines of the beginning. */
 
 	row = beg_row ;
 
-	/* Removing empty lines of the beginning. */
 	while( 1)
 	{
-		while( ( line = ml_screen_get_line( screen , row)) == NULL ||
-			ml_line_is_empty( line))
+		if( ! ( line = ml_screen_get_line( screen , row)) || ml_line_is_empty( line))
 		{
-			if( ++ row > end_row)
-			{
-				return  0 ;
-			}
+			goto  next_line ;
 		}
 
 		size_except_spaces = ml_line_get_num_of_filled_chars_except_spaces( line) ;
@@ -309,7 +426,7 @@ reverse_or_restore_color(
 			return  0 ;
 		}
 	}
-	
+
 	if( row < end_row)
 	{
 		if( ml_line_is_rtl( line))
@@ -392,6 +509,68 @@ reverse_or_restore_color(
 	return  1 ;
 }
 
+static u_int
+check_or_copy_region_rect(
+	ml_screen_t *  screen ,		/* visual */
+	ml_char_t *  chars ,		/* Behavior is undefined if chars is insufficient. */
+	u_int  num_of_chars ,
+        int  beg_col ,
+	int  beg_row ,
+	int  end_col ,
+	int  end_row
+	)
+{
+	int  row ;
+	ml_line_t *  line ;
+	u_int  region_size ;
+	int  beg_char_index ;
+	int  end_char_index ;
+
+	if( beg_col > end_col)
+	{
+		int  col ;
+
+		col = beg_col ;
+		beg_col = end_col ;
+		end_col = col ;
+	}
+
+	region_size = 0 ;
+
+	for( row = beg_row ; row <= end_row ; row++)
+	{
+		if( ( line = ml_screen_get_line( screen , row)))
+		{
+			u_int  size ;
+
+			convert_col_to_char_index( screen , line ,
+				&beg_char_index , &end_char_index , beg_col , end_col) ;
+
+			size = end_char_index - beg_char_index ;
+
+			if( chars && num_of_chars >= region_size + size)
+			{
+				ml_line_copy_logical_str( line , chars + region_size ,
+						beg_char_index , size) ;
+			}
+
+			region_size += size ;
+
+			if( row < end_row)
+			{
+				if( chars && num_of_chars >= region_size + 1)
+				{
+					ml_char_copy( chars + region_size , ml_nl_ch()) ;
+				}
+
+				region_size ++ ;
+			}
+		}
+	}
+
+	return  region_size ;
+}
+
 static u_int
 check_or_copy_region(
 	ml_screen_t *  screen ,		/* visual */
@@ -424,35 +603,9 @@ check_or_copy_region(
 		beg_char_index , beg_row , end_char_index , end_row) ;
 #endif
 
-	row = end_row ;
-
-	/* Removing empty lines of the end. */
-	while( ( line = ml_screen_get_line( screen , row)) == NULL || ml_line_is_empty( line))
+	if( ! modify_region( screen , &end_char_index , &end_row))
 	{
-		if( -- row < 0 && abs(row) > ml_get_num_of_logged_lines( &screen->logs))
-		{
-			return  0 ;
-		}
-	}
-
-	if( row < end_row)
-	{
-		/* Empty lines of the end are removed. */
-
-		if( ml_line_is_rtl( line))
-		{
-			end_char_index = ml_line_beg_char_index_regarding_rtl( line) ;
-		}
-		else
-		{
-			if( ( end_char_index =
-				ml_line_get_num_of_filled_chars_except_spaces( line)) > 0)
-			{
-				end_char_index -- ;
-			}
-		}
-		
-		end_row = row ;
+		return  0 ;
 	}
 
 	row = beg_row ;
@@ -460,13 +613,9 @@ check_or_copy_region(
 	/* Removing empty lines of the beginning. */
 	while( 1)
 	{
-		while( ( line = ml_screen_get_line( screen , row)) == NULL ||
-			ml_line_is_empty( line))
+		if( ! ( line = ml_screen_get_line( screen , row)) || ml_line_is_empty( line))
 		{
-			if( ++ row > end_row)
-			{
-				return  0 ;
-			}
+			goto  next_line ;
 		}
 
 		size_except_spaces = ml_line_get_num_of_filled_chars_except_spaces( line) ;
@@ -666,7 +815,7 @@ ml_set_word_separators(
 		default_word_separators = word_separators ;
 	}
 
-	word_separators = strdup( seps) ;
+	word_separators = kik_str_unescape( seps) ;
 
 	return  1 ;
 }
@@ -1274,11 +1423,20 @@ ml_screen_reverse_color(
 	int  beg_char_index ,
 	int  beg_row ,
 	int  end_char_index ,
-	int  end_row
+	int  end_row ,
+	int  is_rect
 	)
 {
-	return  reverse_or_restore_color( screen , beg_char_index , beg_row ,
-			end_char_index , end_row , ml_line_reverse_color) ;
+	if( is_rect)
+	{
+		return  reverse_or_restore_color_rect( screen , beg_char_index , beg_row ,
+				end_char_index , end_row , ml_line_reverse_color) ;
+	}
+	else
+	{
+		return  reverse_or_restore_color( screen , beg_char_index , beg_row ,
+				end_char_index , end_row , ml_line_reverse_color) ;
+	}
 }
 
 int
@@ -1287,11 +1445,20 @@ ml_screen_restore_color(
 	int  beg_char_index ,
 	int  beg_row ,
 	int  end_char_index ,
-	int  end_row
+	int  end_row ,
+	int  is_rect
 	)
 {
-	return  reverse_or_restore_color( screen , beg_char_index , beg_row ,
-			end_char_index , end_row , ml_line_restore_color) ;
+	if( is_rect)
+	{
+		return  reverse_or_restore_color_rect( screen , beg_char_index , beg_row ,
+				end_char_index , end_row , ml_line_restore_color) ;
+	}
+	else
+	{
+		return  reverse_or_restore_color( screen , beg_char_index , beg_row ,
+				end_char_index , end_row , ml_line_restore_color) ;
+	}
 }
 
 u_int
@@ -1302,11 +1469,20 @@ ml_screen_copy_region(
         int  beg_char_index ,
 	int  beg_row ,
 	int  end_char_index ,
-	int  end_row
+	int  end_row ,
+	int  is_rect
 	)
 {
-	return  check_or_copy_region( screen , chars , num_of_chars , beg_char_index , beg_row ,
-			end_char_index , end_row) ;
+	if( is_rect)
+	{
+		return  check_or_copy_region_rect( screen , chars , num_of_chars ,
+				beg_char_index , beg_row , end_char_index , end_row) ;
+	}
+	else
+	{
+		return  check_or_copy_region( screen , chars , num_of_chars ,
+				beg_char_index , beg_row , end_char_index , end_row) ;
+	}
 }
 
 u_int
@@ -1315,11 +1491,20 @@ ml_screen_get_region_size(
 	int  beg_char_index ,
 	int  beg_row ,
 	int  end_char_index ,
-	int  end_row
+	int  end_row ,
+	int  is_rect
 	)
 {
-	return  check_or_copy_region( screen , NULL , 0 , beg_char_index , beg_row ,
-			end_char_index , end_row) ;
+	if( is_rect)
+	{
+		return  check_or_copy_region_rect( screen , NULL , 0 , beg_char_index ,
+				beg_row , end_char_index , end_row) ;
+	}
+	else
+	{
+		return  check_or_copy_region( screen , NULL , 0 , beg_char_index ,
+				beg_row , end_char_index , end_row) ;
+	}
 }
 
 int
@@ -1811,6 +1996,52 @@ ml_screen_search_find(
 	return  res ;
 }
 
+int
+ml_screen_blink(
+	ml_screen_t *  screen ,
+	int  visible
+	)
+{
+	int  has_blinking_char ;
+
+	has_blinking_char = 0 ;
+
+	if( screen->has_blinking_char)
+	{
+		int  char_index ;
+		int  row ;
+		ml_line_t *  line ;
+
+		for( row = 0 ;
+		     row + screen->backscroll_rows < ml_edit_get_rows( screen->edit) ;
+		     row ++)
+		{
+			if( ( line = ml_screen_get_line( screen , row)) == NULL)
+			{
+				continue ;
+			}
+
+			for( char_index = 0 ; char_index < line->num_of_filled_chars ; char_index++)
+			{
+				if( ml_char_is_blinking( line->chars + char_index))
+				{
+					ml_char_set_visible( line->chars + char_index , visible) ;
+					ml_line_set_modified( line , char_index , char_index) ;
+
+					has_blinking_char = 1 ;
+				}
+			}
+		}
+
+		if( screen->backscroll_rows == 0)
+		{
+			screen->has_blinking_char = has_blinking_char ;
+		}
+	}
+
+	return  has_blinking_char ;
+}
+
 
 /*
  * VT100 commands
@@ -1859,7 +2090,9 @@ ml_screen_combine_with_prev_char(
 	ml_color_t  bg_color ,
 	int  is_bold ,
 	int  is_italic ,
-	int  is_underlined
+	int  is_underlined ,
+	int  is_crossed_out ,
+	int  is_blinking
 	)
 {
 	int  char_index ;
@@ -1883,7 +2116,8 @@ ml_screen_combine_with_prev_char(
 	}
 	
 	if( ! ml_char_combine( ch , code , cs , is_fullwidth , is_comb ,
-		fg_color , bg_color , is_bold , is_italic , is_underlined))
+		fg_color , bg_color , is_bold , is_italic , is_underlined ,
+		is_crossed_out , is_blinking))
 	{
 		return  0 ;
 	}
@@ -2126,16 +2360,20 @@ ml_screen_use_normal_edit(
 	ml_screen_t *  screen
 	)
 {
-	screen->edit = &screen->normal_edit ;
-
-	if( screen->logvis)
+	if( screen->edit != &screen->normal_edit)
 	{
-		(*screen->logvis->init)( screen->logvis ,
-			&screen->edit->model , &screen->edit->cursor) ;
+		screen->normal_edit.bce_ch = screen->alt_edit.bce_ch ;
+		screen->edit = &screen->normal_edit ;
+
+		if( screen->logvis)
+		{
+			(*screen->logvis->init)( screen->logvis ,
+				&screen->edit->model , &screen->edit->cursor) ;
+		}
+
+		ml_edit_set_modified_all( screen->edit) ;
 	}
 
-	ml_edit_set_modified_all( screen->edit) ;
-	
 	return  1 ;
 }
 
@@ -2144,19 +2382,23 @@ ml_screen_use_alternative_edit(
 	ml_screen_t *  screen
 	)
 {
-	screen->edit = &screen->alt_edit ;
-
-	ml_screen_goto_home( screen) ;
-	ml_screen_clear_below( screen) ;
-
-	if( screen->logvis)
+	if( screen->edit != &screen->alt_edit)
 	{
-		(*screen->logvis->init)( screen->logvis ,
-			&screen->edit->model , &screen->edit->cursor) ;
+		screen->alt_edit.bce_ch = screen->normal_edit.bce_ch ;
+		screen->edit = &screen->alt_edit ;
+
+		ml_screen_goto_home( screen) ;
+		ml_screen_clear_below( screen) ;
+
+		if( screen->logvis)
+		{
+			(*screen->logvis->init)( screen->logvis ,
+				&screen->edit->model , &screen->edit->cursor) ;
+		}
+
+		ml_edit_set_modified_all( screen->edit) ;
 	}
 
-	ml_edit_set_modified_all( screen->edit) ;
-	
 	return  1 ;
 }
 
@@ -2232,7 +2474,8 @@ ml_screen_disable_local_echo(
 {
 	u_int  row ;
 	u_int  num_of_rows ;
-	ml_line_t *  line ;
+	ml_line_t *  old_line ;
+	ml_line_t *  new_line ;
 
 	if( ! screen->stored_edits)
 	{
@@ -2241,17 +2484,26 @@ ml_screen_disable_local_echo(
 
 	num_of_rows = ml_edit_get_rows( screen->edit) ;
 
-	/* Modified lines are inherited to stored_edits. */
+	/*
+	 * Set modified flag to the difference between the current edit and the stored one
+	 * to restore the screen before starting local echo.
+	 */
 	for( row = 0 ; row < num_of_rows ; row++)
 	{
-		if( ( line = ml_edit_get_line( screen->edit , row)) &&
-		    ml_line_is_modified( line) &&
-		    ( line = ml_edit_get_line(
-				( screen->edit == &screen->normal_edit ?
-					&screen->stored_edits->normal_edit :
-					&screen->stored_edits->alt_edit) , row)))
+		if( ( new_line = ml_edit_get_line(
+					( screen->edit == &screen->normal_edit ?
+					  &screen->stored_edits->normal_edit :
+					  &screen->stored_edits->alt_edit) , row)) &&
+		    ( old_line = ml_edit_get_line( screen->edit , row)) &&
+		    ( ml_line_is_modified( old_line) ||
+		      old_line->num_of_filled_chars != new_line->num_of_filled_chars
+		#if  0
+		      || ml_str_bytes_equal( old_line->chars , new_line->chars ,
+				new_line->num_of_filled_chars)
+		#endif
+		      ))
 		{
-			ml_line_set_modified_all( line) ;
+			ml_line_set_modified_all( new_line) ;
 		}
 	}
 
@@ -2268,18 +2520,34 @@ ml_screen_disable_local_echo(
 }
 
 int
-ml_screen_fill_all_with_e(
-	ml_screen_t *  screen
+ml_screen_fill_area(
+	ml_screen_t *  screen ,
+	int  code ,	/* Unicode */
+	int  col ,
+	int  row ,
+	u_int  num_of_cols ,
+	u_int  num_of_rows
 	)
 {
-	ml_char_t  e_ch ;
+	ml_char_t  ch ;
 
-	ml_char_init( &e_ch) ;
-	ml_char_set( &e_ch , 'E' , US_ASCII , 0 , 0 , ML_FG_COLOR , ML_BG_COLOR , 0 , 0 , 0) ;
+	ml_char_init( &ch) ;
 
-	ml_edit_fill_all( screen->edit , &e_ch) ;
+	ml_char_set( &ch , code ,
+		code <= 0x7f ? US_ASCII : ISO10646_UCS4_1 ,	/* XXX biwidth is not supported. */
+		0 , 0 , ML_FG_COLOR , ML_BG_COLOR , 0 , 0 , 0 , 0 , 0) ;
 
-	ml_char_final( &e_ch) ;
+	ml_edit_fill_area( screen->edit , &ch , col , row , num_of_cols , num_of_rows) ;
+
+	ml_char_final( &ch) ;
 
 	return  1 ;
 }
+
+void
+ml_screen_enable_blinking(
+	ml_screen_t *  screen
+	)
+{
+	screen->has_blinking_char = 1 ;
+}
