@@ -81,6 +81,7 @@ typedef struct  ml_screen
 
 	int8_t  use_dynamic_comb ;	/* public */
 	int8_t  is_cursor_visible ;
+	int8_t  has_blinking_char ;
 
 } ml_screen_t ;
 
@@ -180,16 +181,17 @@ int  ml_screen_backscroll_downward( ml_screen_t *  screen , u_int  size) ;
 		ml_edit_set_tab_size( (screen)->edit , tab_size)
 
 int  ml_screen_restore_color( ml_screen_t *  screen , int  beg_char_index , int  beg_row ,
-	int  end_char_index , int  end_row) ;
+	int  end_char_index , int  end_row , int  is_rect) ;
 
 int  ml_screen_reverse_color( ml_screen_t *  screen , int  beg_char_index , int  beg_row ,
-	int  end_char_index , int  end_row) ;
+	int  end_char_index , int  end_row , int  is_rect) ;
 
 u_int  ml_screen_copy_region( ml_screen_t *  screen , ml_char_t *  chars ,
-	u_int  num_of_chars , int  beg_char_index , int  beg_row , int  end_char_index , int  end_row) ;
+	u_int  num_of_chars , int  beg_char_index , int  beg_row ,
+	int  end_char_index , int  end_row , int  is_rect) ;
 
 u_int  ml_screen_get_region_size( ml_screen_t *  screen , int  beg_char_index ,
-	int  beg_row , int  end_char_index , int  end_row) ;
+	int  beg_row , int  end_char_index , int  end_row , int  is_rect) ;
 
 int  ml_screen_get_line_region( ml_screen_t *  screen , int *  beg_row ,
 	int *  end_char_index , int *  end_row , int  base_row) ;
@@ -207,6 +209,8 @@ int  ml_screen_search_reset_position( ml_screen_t *  screen) ;
 int  ml_screen_search_find( ml_screen_t *  screen , int *  beg_char_index , int *  beg_row ,
 	int *  end_char_index , int *  end_row , void *  regex , int  backward) ;
 
+int  ml_screen_blink( ml_screen_t *  screen , int  visible) ;
+
 
 /*
  * VT100 commands (called in logical context)
@@ -217,7 +221,7 @@ ml_char_t *  ml_screen_get_n_prev_char( ml_screen_t *  screen , int  n) ;
 int  ml_screen_combine_with_prev_char( ml_screen_t *  screen , u_int32_t  code ,
 	mkf_charset_t  cs , int  is_fullwidth , int  is_comb ,
 	ml_color_t  fg_color , ml_color_t  bg_color , int  is_bold ,
-	int  is_italic , int  is_underlined) ;
+	int  is_italic , int  is_underlined , int  is_crossed_out , int  is_blinking) ;
 
 int  ml_screen_insert_chars( ml_screen_t *  screen , ml_char_t *  chars , u_int  len) ;
 
@@ -273,6 +277,10 @@ int  ml_screen_delete_lines( ml_screen_t *  screen , u_int  size) ;
 
 #define  ml_screen_scroll_downward( screen , size)  ml_edit_scroll_downward( (screen)->edit , size)
 
+#define  ml_screen_scroll_leftward( screen , size)  ml_edit_scroll_leftward( (screen)->edit , size)
+
+#define  ml_screen_scroll_rightward( screen , size)  ml_edit_scroll_rightward( (screen)->edit , size)
+
 int  ml_screen_go_forward( ml_screen_t *  screen , u_int  size) ;
 
 int  ml_screen_go_back( ml_screen_t *  screen , u_int  size) ;
@@ -319,7 +327,8 @@ int  ml_screen_local_echo_wait( ml_screen_t *  screen , int  msec) ;
 
 int  ml_screen_disable_local_echo( ml_screen_t *  screen) ;
 
-int  ml_screen_fill_all_with_e( ml_screen_t *  screen) ;
+int  ml_screen_fill_area( ml_screen_t *  screen , int  code ,
+	int  col , int  beg , u_int  num_of_cols , u_int  num_of_rows) ;
 
 #define  ml_screen_copy_area( screen , src_col , src_row , \
 		num_of_cols , num_of_rows , dst_col , dst_row) \
@@ -329,5 +338,7 @@ int  ml_screen_fill_all_with_e( ml_screen_t *  screen) ;
 #define  ml_screen_erase_area( screen , col , row , num_of_cols , num_of_rows) \
 		ml_edit_erase_area( (screen)->edit , col , row , num_of_cols , num_of_rows)
 
+void  ml_screen_enable_blinking( ml_screen_t *  screen) ;
+
 
 #endif
