@@ -9,6 +9,7 @@
 #include  <kiklib/kik_debug.h>
 
 #include  "ml_char_encoding.h"		/* ml_is_msb_set */
+#include  "ml_drcs.h"
 
 
 typedef struct ml_str_parser
@@ -109,15 +110,21 @@ next_char(
 
 	ch->cs = ml_char_cs( ml_ch) ;
 	ch->size = CS_SIZE(ch->cs) ;
+
 	mkf_int_to_bytes( ch->ch , ch->size , ml_char_code( ml_ch)) ;
 
-	if( ml_is_msb_set( ch->cs))
+#if  1
+	if( ! ml_convert_drcs_to_unicode_pua( ch))
+#endif
 	{
-		UNSET_MSB(ch->ch[0]) ;
-	}
+		/* XXX */
+		ch->property = 0 ;
 
-	/* XXX */
-	ch->property = 0 ;
+		if( ml_is_msb_set( ch->cs))
+		{
+			UNSET_MSB(ch->ch[0]) ;
+		}
+	}
 
 	if( ml_str_parser->left == 0)
 	{
