@@ -7,6 +7,7 @@
 #include  <kiklib/kik_mem.h>	/* malloc/free */
 #include  <kiklib/kik_debug.h>
 #include  <kiklib/kik_str.h>	/* strdup */
+#include  <kiklib/kik_sig_child.h>
 
 #include  "ml_pty.h"
 #include  "ml_vt100_parser.h"
@@ -20,7 +21,6 @@
 #else
 #include  <pthread.h>
 #endif
-#include  <kiklib/kik_sig_child.h>
 
 
 typedef struct
@@ -30,6 +30,7 @@ typedef struct
 	char **  argv ;
 	char **  env ;
 	char *  host ;
+	char *  work_dir ;
 	char *  pass ;
 	char *  pubkey ;
 	char *  privkey ;
@@ -60,6 +61,7 @@ pty_args_delete(
 
 	free( args->cmd_path) ;
 	free( args->host) ;
+	free( args->work_dir) ;
 	free( args->pass) ;
 	free( args->pubkey) ;
 	free( args->privkey) ;
@@ -92,6 +94,7 @@ pty_args_new(
 	char **  argv ,
 	char **  env ,
 	const char *  host ,
+	const char *  work_dir ,
 	const char *  pass ,
 	const char *  pubkey ,
 	const char *  privkey
@@ -118,6 +121,11 @@ pty_args_new(
 		args->host = strdup( host) ;
 	}
 
+	if( work_dir)
+	{
+		args->work_dir = strdup( work_dir) ;
+	}
+
 	if( pass)
 	{
 		args->pass = strdup( pass) ;
@@ -202,7 +210,7 @@ open_pty(
 	args = p ;
 
 	pty = ml_pty_new( args->cmd_path , args->argv , args->env , args->host ,
-			args->pass , args->pubkey , args->privkey ,
+			args->work_dir , args->pass , args->pubkey , args->privkey ,
 			ml_screen_get_logical_cols( args->term->screen) ,
 			ml_screen_get_logical_rows( args->term->screen)) ;
 
@@ -235,6 +243,21 @@ open_pty(
 
 #endif
 
+/* Must be called in visual context. */
+static void
+set_use_local_echo(
+	ml_term_t *  term ,
+	int  flag
+	)
+{
+	if( term->use_local_echo != flag && ! ( term->use_local_echo = flag))
+	{
+		ml_screen_logical( term->screen) ;
+		ml_screen_disable_local_echo( term->screen) ;
+		ml_screen_visual( term->screen) ;
+	}
+}
+
 
 /* --- global functions --- */
 
@@ -246,20 +269,24 @@ ml_term_new(
 	u_int  log_size ,
 	ml_char_encoding_t  encoding ,
 	int  is_auto_encoding ,
+	int  use_auto_detect ,
+	int  logging_vt_seq ,
 	ml_unicode_policy_t  policy ,
 	u_int  col_size_a ,
 	int  use_char_combining ,
 	int  use_multi_col_char ,
 	int  use_bidi ,
 	ml_bidi_mode_t  bidi_mode ,
+	const char *  bidi_separators ,
 	int  use_ind ,
 	int  use_bce ,
 	int  use_dynamic_comb ,
 	ml_bs_mode_t  bs_mode ,
 	ml_vertical_mode_t  vertical_mode ,
 	int  use_local_echo ,
-	char *  win_name ,
-	char *  icon_name
+	const char *  win_name ,
+	const char *  icon_name ,
+	ml_alt_color_mode_t  alt_color_mode
 	)
 {
 	ml_term_t *  term ;
@@ -283,8 +310,10 @@ ml_term_new(
 		goto  error ;
 	}
 
-	if( ! ( term->parser = ml_vt100_parser_new( term->screen , encoding , policy , col_size_a ,
-				use_char_combining , use_multi_col_char , win_name , icon_name)))
+	if( ! ( term->parser = ml_vt100_parser_new( term->screen , encoding , is_auto_encoding ,
+				use_auto_detect , logging_vt_seq ,
+				policy , col_size_a , use_char_combining , use_multi_col_char ,
+				win_name , icon_name , alt_color_mode)))
 	{
 	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG " ml_vt100_parser_new failed.\n") ;
@@ -295,13 +324,9 @@ ml_term_new(
 
 	ml_vt100_parser_set_use_bidi( term->parser , use_bidi) ;
 
-	if( ! ml_config_menu_init( &term->config_menu))
+	if( bidi_separators)
 	{
-	#ifdef  DEBUG
-		kik_warn_printf( KIK_DEBUG_TAG " ml_config_menu_init failed.\n") ;
-	#endif
-	
-		goto  error ;
+		term->bidi_separators = kik_str_unescape( bidi_separators) ;
 	}
 
 	term->vertical_mode = vertical_mode ;
@@ -309,9 +334,6 @@ ml_term_new(
 	term->use_bidi = use_bidi ;
 	term->use_ind = use_ind ;
 	term->use_dynamic_comb = use_dynamic_comb ;
-
-	term->is_auto_encoding = is_auto_encoding ;
-
 	term->use_local_echo = use_local_echo ;
 
 	return  term ;
@@ -344,6 +366,8 @@ ml_term_delete(
 	}
 #endif
 
+	free( term->user_data) ;
+
 	if( term->pty)
 	{
 		ml_pty_delete( term->pty) ;
@@ -359,14 +383,12 @@ ml_term_delete(
 	}
 
 	free( term->uri) ;
-
 	free( term->icon_path) ;
+	free( term->bidi_separators) ;
 
 	ml_screen_delete( term->screen) ;
 	ml_vt100_parser_delete( term->parser) ;
 
-	ml_config_menu_final( &term->config_menu) ;
-
 	free( term) ;
 
 	return  1 ;
@@ -405,6 +427,7 @@ ml_term_open_pty(
 	char **  argv ,
 	char **  env ,
 	const char *  host ,
+	const char *  work_dir ,
 	const char *  pass ,
 	const char *  pubkey ,
 	const char *  privkey
@@ -415,8 +438,8 @@ ml_term_open_pty(
 	#ifdef  OPEN_PTY_ASYNC
 		pty_args_t *  args ;
 
-		if( ! ( args = pty_args_new( term , cmd_path , argv , env , host , pass ,
-						pubkey , privkey)))
+		if( ! ( args = pty_args_new( term , cmd_path , argv , env , host ,
+					work_dir , pass , pubkey , privkey)))
 		{
 			return  0 ;
 		}
@@ -452,7 +475,8 @@ ml_term_open_pty(
 	#else	/* OPEN_PTY_ASYNC */
 		ml_pty_ptr_t  pty ;
 
-		if( ! ( pty = ml_pty_new( cmd_path , argv , env , host , pass , pubkey , privkey ,
+		if( ! ( pty = ml_pty_new( cmd_path , argv , env , host , work_dir ,
+				pass , pubkey , privkey ,
 				ml_screen_get_logical_cols( term->screen) ,
 				ml_screen_get_logical_rows( term->screen))))
 		{
@@ -559,41 +583,33 @@ ml_term_detach(
 	return  1 ;
 }
 
-int
-ml_term_set_auto_encoding(
-	ml_term_t *  term ,
-	int  is_auto_encoding
-	)
-{
-	term->is_auto_encoding = is_auto_encoding ;
-
-	return  1 ;
-}
-
 int
 ml_term_set_use_bidi(
 	ml_term_t *  term ,
 	int  flag
 	)
 {
-	term->use_bidi = flag ;
+	if( ( term->use_bidi = flag))
+	{
+		/* disable iscii */
+		term->use_ind = 0 ;
 
-	ml_vt100_parser_set_use_bidi( term->parser , flag) ;
+		ml_vt100_parser_set_use_bidi( term->parser , flag) ;
+	}
 
 	return  1 ;
 }
 
 int
-ml_term_set_use_local_echo(
+ml_term_set_use_ind(
 	ml_term_t *  term ,
 	int  flag
 	)
 {
-	if( term->use_local_echo != flag && ! ( term->use_local_echo = flag))
+	if( ( term->use_ind = flag))
 	{
-		ml_screen_logical( term->screen) ;
-		ml_screen_disable_local_echo( term->screen) ;
-		ml_screen_visual( term->screen) ;
+		/* disable bidi */
+		ml_term_set_use_bidi( term , 0) ;
 	}
 
 	return  1 ;
@@ -663,28 +679,20 @@ size_t
 ml_term_write(
 	ml_term_t *  term ,
 	u_char *  buf ,
-	size_t  len ,
-	int  to_menu
+	size_t  len
 	)
 {
-	if( to_menu)
+	if( term->pty == NULL)
 	{
-		return  ml_config_menu_write( &term->config_menu , buf , len) ;
+		return  0 ;
 	}
-	else
+
+	if( term->use_local_echo)
 	{
-		if( term->pty == NULL)
-		{
-			return  0 ;
-		}
-
-		if( term->use_local_echo)
-		{
-			ml_vt100_parser_local_echo( term->parser , buf , len) ;
-		}
-
-		return  ml_write_to_pty( term->pty , buf , len) ;
+		ml_vt100_parser_local_echo( term->parser , buf , len) ;
 	}
+
+	return  ml_write_to_pty( term->pty , buf , len) ;
 }
 
 int
@@ -780,19 +788,20 @@ ml_term_set_modified_region_in_screen(
 {
 	int  row ;
 	ml_line_t *  line ;
+	int  revert_to_visual ;
 
-#ifdef  DEBUG
-	if( term->screen->logvis && ! term->screen->logvis->is_visual)
+	/*
+	 * This function is usually called in visual context, and sometimes
+	 * called in logical context. (see flush_scroll_cache() in x_screen.c)
+	 */
+	if( ! ml_screen_logical_visual_is_reversible( term->screen) &&
+	    ml_screen_logical( term->screen))
 	{
-		kik_debug_printf( KIK_DEBUG_TAG
-			" ml_term_set_modified_lines() should be called in visual context but"
-			" is called in logical context.\n") ;
+		revert_to_visual = 1 ;
 	}
-#endif
-
-	if( ! ml_screen_logical_visual_is_reversible( term->screen))
+	else
 	{
-		ml_screen_logical( term->screen) ;
+		revert_to_visual = 0 ;
 	}
 
 	for( row = beg_row ; row < beg_row + nrows ; row ++)
@@ -803,7 +812,7 @@ ml_term_set_modified_region_in_screen(
 		}
 	}
 
-	if( ! ml_screen_logical_visual_is_reversible( term->screen))
+	if( revert_to_visual)
 	{
 		/* ml_screen_render( term->screen) ; */
 		ml_screen_visual( term->screen) ;
@@ -822,19 +831,20 @@ ml_term_set_modified_lines(
 {
 	int  row ;
 	ml_line_t *  line ;
+	int  revert_to_visual ;
 
-#ifdef  DEBUG
-	if( term->screen->logvis && ! term->screen->logvis->is_visual)
+	/*
+	 * This function is usually called in visual context, and sometimes
+	 * called in logical context. (see flush_scroll_cache() in x_screen.c)
+	 */
+	if( ! ml_screen_logical_visual_is_reversible( term->screen) &&
+	    ml_screen_logical( term->screen))
 	{
-		kik_debug_printf( KIK_DEBUG_TAG
-			" ml_term_set_modified_lines() should be called in visual context but"
-			" is called in logical context.\n") ;
+		revert_to_visual = 1 ;
 	}
-#endif
-
-	if( ! ml_screen_logical_visual_is_reversible( term->screen))
+	else
 	{
-		ml_screen_logical( term->screen) ;
+		revert_to_visual = 0 ;
 	}
 
 	for( row = beg ; row <= end ; row ++)
@@ -845,7 +855,7 @@ ml_term_set_modified_lines(
 		}
 	}
 
-	if( ! ml_screen_logical_visual_is_reversible( term->screen))
+	if( revert_to_visual)
 	{
 		/* ml_screen_render( term->screen) ; */
 		ml_screen_visual( term->screen) ;
@@ -863,19 +873,20 @@ ml_term_set_modified_lines_in_screen(
 {
 	int  row ;
 	ml_line_t *  line ;
+	int  revert_to_visual ;
 
-#ifdef  DEBUG
-	if( term->screen->logvis && ! term->screen->logvis->is_visual)
+	/*
+	 * This function is usually called in visual context, and sometimes
+	 * called in logical context. (see flush_scroll_cache() in x_screen.c)
+	 */
+	if( ! ml_screen_logical_visual_is_reversible( term->screen) &&
+	    ml_screen_logical( term->screen))
 	{
-		kik_debug_printf( KIK_DEBUG_TAG
-			" ml_term_set_modified_lines_in_screen() should be called in visual "
-			" context but is called in logical context.\n") ;
+		revert_to_visual = 1 ;
 	}
-#endif
-
-	if( ! ml_screen_logical_visual_is_reversible( term->screen))
+	else
 	{
-		ml_screen_logical( term->screen) ;
+		revert_to_visual = 0 ;
 	}
 
 	for( row = beg ; row <= end ; row ++)
@@ -886,7 +897,7 @@ ml_term_set_modified_lines_in_screen(
 		}
 	}
 
-	if( ! ml_screen_logical_visual_is_reversible( term->screen))
+	if( revert_to_visual)
 	{
 		/* ml_screen_render( term->screen) ; */
 		ml_screen_visual( term->screen) ;
@@ -900,23 +911,25 @@ ml_term_set_modified_all_lines_in_screen(
 	ml_term_t *  term
 	)
 {
-#ifdef  DEBUG
-	if( term->screen->logvis && ! term->screen->logvis->is_visual)
-	{
-		kik_debug_printf( KIK_DEBUG_TAG
-			" ml_term_set_modified_all_lines_in_screen() should be called in "
-			" visual context but is called in logical context.\n") ;
-	}
-#endif
+	int  revert_to_visual ;
 
-	if( ! ml_screen_logical_visual_is_reversible( term->screen))
+	/*
+	 * This function is usually called in visual context, and sometimes
+	 * called in logical context. (see flush_scroll_cache() in x_screen.c)
+	 */
+	if( ! ml_screen_logical_visual_is_reversible( term->screen) &&
+	    ml_screen_logical( term->screen))
 	{
-		ml_screen_logical( term->screen) ;
+		revert_to_visual = 1 ;
+	}
+	else
+	{
+		revert_to_visual = 0 ;
 	}
 
 	ml_screen_set_modified_all( term->screen) ;
 
-	if( ! ml_screen_logical_visual_is_reversible( term->screen))
+	if( revert_to_visual)
 	{
 		/* ml_screen_render( term->screen) ; */
 		ml_screen_visual( term->screen) ;
@@ -1047,12 +1060,11 @@ ml_term_update_special_visual(
 	else if( term->use_bidi && ml_term_get_encoding( term) == ML_UTF8)
 	{
 		if( ( term->shape = ml_arabic_shape_new()) &&
-		    ( logvis = ml_logvis_bidi_new( term->bidi_mode)))
+		    ( logvis = ml_logvis_bidi_new( term->bidi_mode , term->bidi_separators)))
 		{
 			if( ml_screen_add_logical_visual( term->screen , logvis))
 			{
 				has_logvis = 1 ;
-				need_comb = 1 ;
 			}
 			else
 			{
@@ -1082,7 +1094,6 @@ ml_term_update_special_visual(
 			if( ml_screen_add_logical_visual( term->screen , logvis))
 			{
 				has_logvis = 1 ;
-				need_comb = 1 ;
 			}
 			else
 			{
@@ -1103,10 +1114,10 @@ ml_term_update_special_visual(
 	#endif
 	}
 
-	if( need_comb && ! ml_term_is_using_char_combining( term))
+	if( need_comb && ! ml_vt100_parser_is_using_char_combining( term->parser))
 	{
 		kik_msg_printf( "Set use_combining=true forcibly.\n") ;
-		ml_term_set_use_char_combining( term , 1) ;
+		ml_vt100_parser_set_use_char_combining( term->parser , 1) ;
 	}
 
 	if( ! has_logvis)
@@ -1153,24 +1164,188 @@ ml_term_enter_backscroll_mode(
 int
 ml_term_set_icon_path(
 	ml_term_t *  term ,
-	char *  path
+	const char *  path
 	)
 {
 	free( term->icon_path) ;
-	term->icon_path = strdup( path) ;
+
+	if( path && *path)
+	{
+		term->icon_path = strdup( path) ;
+	}
+	else
+	{
+		term->icon_path = NULL ;
+	}
 
 	return 1 ;
 }
 
-int
-ml_term_start_config_menu(
+void
+ml_term_set_bidi_separators(
 	ml_term_t *  term ,
-	char *  cmd_path ,
-	int  x ,
-	int  y ,
-	char *  display
+	const char *  bidi_separators
 	)
 {
-	return  ml_config_menu_start( &term->config_menu , cmd_path ,
-				x , y , display , term->pty) ;
+	free( term->bidi_separators) ;
+
+	if( bidi_separators && *bidi_separators)
+	{
+		term->bidi_separators = kik_str_unescape( bidi_separators) ;
+	}
+	else
+	{
+		term->bidi_separators = NULL ;
+	}
+}
+
+int
+ml_term_get_config(
+	ml_term_t *  term ,
+	ml_term_t *  output ,
+	char *  key ,
+	int  to_menu ,
+	int *  flag
+	)
+{
+	char *  value ;
+
+	if( ml_vt100_parser_get_config( term->parser , output->pty , key , to_menu , flag))
+	{
+		return  1 ;
+	}
+
+	if( strcmp( key , "vertical_mode") == 0)
+	{
+		value = ml_get_vertical_mode_name( term->vertical_mode) ;
+	}
+	else if( strcmp( key , "use_dynamic_comb") == 0)
+	{
+		if( term->use_dynamic_comb)
+		{
+			value = "true" ;
+		}
+		else
+		{
+			value = "false" ;
+		}
+	}
+	else if( strcmp( key , "use_bidi") == 0)
+	{
+		if( term->use_bidi)
+		{
+			value = "true" ;
+		}
+		else
+		{
+			value = "false" ;
+		}
+	}
+	else if( strcmp( key , "use_ind") == 0)
+	{
+		if( term->use_ind)
+		{
+			value = "true" ;
+		}
+		else
+		{
+			value = "false" ;
+		}
+	}
+	else if( strcmp( key , "bidi_mode") == 0)
+	{
+		value = ml_get_bidi_mode_name( term->bidi_mode) ;
+	}
+	else if( strcmp( key , "bidi_separators") == 0)
+	{
+		if( ( value = term->bidi_separators) == NULL)
+		{
+			value = "" ;
+		}
+	}
+	else if( strcmp( key , "pty_name") == 0)
+	{
+		if( term != output)
+		{
+			if( ( value = ml_get_window_name( term->parser)) == NULL)
+			{
+				value = "" ;
+			}
+		}
+		else
+		{
+			value = ml_term_get_slave_name( term) ;
+		}
+	}
+	else if( strcmp( key , "icon_path") == 0)
+	{
+		if( ( value = term->icon_path) == NULL)
+		{
+			value = "" ;
+		}
+	}
+	else if( strcmp( key , "use_local_echo") == 0)
+	{
+		if( term->use_local_echo)
+		{
+			value = "true" ;
+		}
+		else
+		{
+			value = "false" ;
+		}
+	}
+	else
+	{
+		/* Continue to process it in x_screen.c */
+		return  0 ;
+	}
+
+	/* value is never set NULL above. */
+#if  0
+	if( ! value)
+	{
+		ml_response_config( term->pty , "error" , NULL , to_menu) ;
+	}
+#endif
+
+	if( flag)
+	{
+		*flag = value ? true_or_false( value) : -1 ;
+	}
+	else
+	{
+		ml_response_config( term->pty , key , value , to_menu) ;
+	}
+
+	return  1 ;
+}
+
+int
+ml_term_set_config(
+	ml_term_t *  term ,
+	char *  key ,
+	char *  value
+	)
+{
+	if( ml_vt100_parser_set_config( term->parser , key , value))
+	{
+		/* do nothing */
+	}
+	else if( strcmp( key , "use_local_echo") == 0)
+	{
+		int  flag ;
+
+		if( ( flag = true_or_false( value)) != -1)
+		{
+			set_use_local_echo( term , flag) ;
+		}
+	}
+	else
+	{
+		/* Continue to process it in x_screen.c */
+		return  0 ;
+	}
+
+	return  1 ;
 }
