@@ -278,20 +278,24 @@ ml_create_term(
 	u_int  log_size ,
 	ml_char_encoding_t  encoding ,
 	int  is_auto_encoding ,
+	int  use_auto_detect ,
+	int  logging_vt_seq ,
 	ml_unicode_policy_t  policy ,
 	int  col_size_a ,
 	int  use_char_combining ,
 	int  use_multi_col_char ,
 	int  use_bidi ,
 	ml_bidi_mode_t  bidi_mode ,
+	const char *  bidi_separators ,
 	int  use_ind ,
 	int  use_bce ,
 	int  use_dynamic_comb ,
 	ml_bs_mode_t  bs_mode ,
 	ml_vertical_mode_t  vertical_mode ,
 	int  use_local_echo ,
-	char *  win_name ,
-	char *  icon_name
+	const char *  win_name ,
+	const char *  icon_name ,
+	ml_alt_color_mode_t  alt_color_mode
 	)
 {
 #if  ! defined(USE_WIN32API) && ! defined(DEBUG)
@@ -325,11 +329,13 @@ ml_create_term(
 				{
 					if( ( terms[num_of_terms] = ml_term_new( cols , rows ,
 						tab_size , log_size , encoding , is_auto_encoding ,
+						use_auto_detect , logging_vt_seq ,
 						policy , col_size_a , use_char_combining ,
 						use_multi_col_char , use_bidi , bidi_mode ,
-						use_ind , use_bce , use_dynamic_comb ,
-						bs_mode , vertical_mode , use_local_echo ,
-						win_name , icon_name)))
+						bidi_separators , use_ind , use_bce ,
+						use_dynamic_comb , bs_mode , vertical_mode ,
+						use_local_echo , win_name , icon_name ,
+						alt_color_mode)))
 					{
 						ml_term_plug_pty( terms[num_of_terms++] , pty) ;
 						ml_set_pty_winsize( pty , cols , rows) ;
@@ -367,10 +373,11 @@ ml_create_term(
 	 */
 
 	if( ! ( terms[num_of_terms] = ml_term_new( cols , rows , tab_size , log_size , encoding ,
-				is_auto_encoding ,
-				policy , col_size_a , use_char_combining , use_multi_col_char ,
-				use_bidi , bidi_mode , use_ind , use_bce , use_dynamic_comb ,
-				bs_mode , vertical_mode , use_local_echo , win_name , icon_name)))
+				is_auto_encoding , use_auto_detect , logging_vt_seq , policy ,
+				col_size_a , use_char_combining ,
+				use_multi_col_char , use_bidi , bidi_mode , bidi_separators ,
+				use_ind , use_bce , use_dynamic_comb , bs_mode , vertical_mode ,
+				use_local_echo , win_name , icon_name , alt_color_mode)))
 	{
 		return  NULL ;
 	}
@@ -412,7 +419,7 @@ ml_destroy_term(
 
 ml_term_t *
 ml_get_term(
-	char *  dev
+	const char *  dev
 	)
 {
 	int  count ;
@@ -431,7 +438,7 @@ ml_get_term(
 
 ml_term_t *
 ml_get_detached_term(
-	char *  dev
+	const char *  dev
 	)
 {
 	int  count ;
