@@ -13,6 +13,7 @@
 #include  "ml_pty.h"
 #include  "ml_screen.h"
 #include  "ml_char_encoding.h"
+#include  "ml_drcs.h"
 
 
 #define  PTY_WR_BUFFER_SIZE  100
@@ -29,10 +30,10 @@ typedef enum  ml_unicode_policy
 {
 	NO_UNICODE_POLICY = 0x0 ,
 	NOT_USE_UNICODE_FONT = 0x1 ,
-	USE_UNICODE_PROPERTY = 0x2 ,
-	ONLY_USE_UNICODE_FONT = 0x4 ,
-	NOT_USE_UNICODE_BOXDRAW_FONT = 0x8 ,
-	ONLY_USE_UNICODE_BOXDRAW_FONT = 0x10 ,
+	ONLY_USE_UNICODE_FONT = 0x2 ,
+	NOT_USE_UNICODE_BOXDRAW_FONT = 0x4 ,
+	ONLY_USE_UNICODE_BOXDRAW_FONT = 0x8 ,
+	USE_UNICODE_DRCS = 0x10 ,
 
 	UNICODE_POLICY_MAX
 
@@ -58,6 +59,16 @@ typedef enum  ml_extended_mouse_report_mode
 
 } ml_extended_mouse_report_mode_t ;
 
+typedef enum  ml_alt_color_mode
+{
+	ALT_COLOR_BOLD = 0x1 ,
+	ALT_COLOR_ITALIC = 0x2 ,
+	ALT_COLOR_UNDERLINE = 0x4 ,
+	ALT_COLOR_BLINKING = 0x8 ,
+	ALT_COLOR_CROSSED_OUT = 0x10 ,
+
+} ml_alt_color_mode_t ;
+
 typedef struct  ml_write_buffer
 {
 	ml_char_t  chars[PTY_WR_BUFFER_SIZE] ;
@@ -76,6 +87,7 @@ typedef struct  ml_read_buffer
 	size_t  len ;
 	size_t  filled_len ;
 	size_t  left ;
+	size_t  new_len ;
 
 } ml_read_buffer_t ;
 
@@ -85,7 +97,8 @@ typedef struct  ml_xterm_event_listener
 
 	void (*start)( void *) ;	/* called in *visual* context. (Note that not logical) */
 	void (*stop)( void *) ;		/* called in visual context. */
-	
+	void (*interrupt)( void *) ;	/* called in visual context. */
+
 	void (*resize)( void * , u_int , u_int) ;	/* called in visual context. */
 	void (*reverse_video)( void * , int) ;		/* called in visual context. */
 	void (*set_mouse_report)( void * , ml_mouse_report_mode_t) ;/* called in visual context. */
@@ -94,11 +107,12 @@ typedef struct  ml_xterm_event_listener
 	void (*bel)( void *) ;				/* called in visual context. */
 	int (*im_is_active)( void *) ;			/* called in logical context. */
 	void (*switch_im_mode)( void *) ;		/* called in logical context. */
-	void (*set_selection)( void * , ml_char_t * , u_int) ;	/* called in logical context. */
+	void (*set_selection)( void * , ml_char_t * , u_int , u_char *) ;	/* called in logical context. */
 	int (*get_window_size)( void * , u_int * , u_int *) ;	/* called in logical context. */
 	int (*get_rgb)( void * , u_int8_t * , u_int8_t * ,
 			u_int8_t * , ml_color_t) ;		/* called in logical context. */
 	ml_char_t *  (*get_picture_data)( void * , char * , int * , int *) ; /* called in logical context. */
+	void (*add_frame_to_animation)( void * , char * , int * , int *) ; /* called in logical context. */
 
 } ml_xterm_event_listener_t ;
 
@@ -112,7 +126,7 @@ typedef struct  ml_config_event_listener
 
 	/* Assume that exec, set and get affect each window. */
 	int (*exec)( void * , char *) ;
-	void (*set)( void * , char * , char * , char *) ;
+	int (*set)( void * , char * , char * , char *) ;
 	void (*get)( void * , char * , char * , int) ;
 
 	/* Assume that saved, set_font and set_color affect all window. */
@@ -124,27 +138,39 @@ typedef struct  ml_config_event_listener
 
 } ml_config_event_listener_t ;
 
+typedef struct  ml_vt100_parser *  ml_vt100_parser_ptr_t ;
+
 typedef struct  ml_vt100_storable_states
 {
 	int8_t  is_saved ;
 	
 	int8_t  is_bold ;
 	int8_t  is_italic ;
-	int8_t  is_underlined ;
+	int8_t  underline_style ;
 	int8_t  is_reversed ;
+	int8_t  is_crossed_out ;
+	int8_t  is_blinking ;
 	ml_color_t  fg_color ;
 	ml_color_t  bg_color ;
 	mkf_charset_t  cs ;
 
 } ml_vt100_storable_states_t ;
 
-typedef struct ml_vt100_saved_names
+typedef struct  ml_vt100_saved_names
 {
 	char **  names ;
 	u_int  num ;
 
 } ml_vt100_saved_names_t ;
 
+typedef struct  ml_macro
+{
+	u_char *  str ;
+	int8_t  is_sixel ;
+	u_int8_t  sixel_num ;
+
+} ml_macro_t ;
+
 typedef struct  ml_vt100_parser
 {
 	ml_read_buffer_t  r_buf ;
@@ -184,8 +210,12 @@ typedef struct  ml_vt100_parser
 
 	int8_t  is_bold ;
 	int8_t  is_italic ;
-	int8_t  is_underlined ;
+	int8_t  underline_style ;
 	int8_t  is_reversed ;
+	int8_t  is_crossed_out ;
+	int8_t  is_blinking ;
+
+	int8_t  alt_color_mode ;
 
 	u_int8_t  col_size_of_width_a ;	/* 1 or 2 */
 
@@ -215,9 +245,9 @@ typedef struct  ml_vt100_parser
 #endif
 
 	int8_t  sixel_scrolling ;
-
 	int8_t  yield ;
 
+	int8_t  is_auto_encoding ;
 	int8_t  use_auto_detect ;
 
 	/* for save/restore cursor */
@@ -227,6 +257,11 @@ typedef struct  ml_vt100_parser
 	ml_vt100_saved_names_t  saved_win_names ;
 	ml_vt100_saved_names_t  saved_icon_names ;
 
+	ml_drcs_t *  drcs ;
+
+	ml_macro_t *  macros ;
+	u_int  num_of_macros ;
+
 } ml_vt100_parser_t ;
 
 
@@ -234,23 +269,32 @@ void  ml_set_use_alt_buffer( int  use) ;
 
 void  ml_set_use_ansi_colors( int  use) ;
 
-void  ml_set_use_scp( int  use) ;
+void  ml_set_unicode_noconv_areas( char *  areas) ;
 
-int  ml_set_unicode_noconv_areas( char *  areas) ;
+void  ml_set_full_width_areas( char *  areas) ;
+
+void  ml_set_use_ttyrec_format( int  use) ;
+
+#ifdef  USE_LIBSSH2
+void  ml_set_use_scp_full( int  use) ;
+#else
+#define  ml_set_use_scp_full(use)  (0)
+#endif
 
 ml_vt100_parser_t *  ml_vt100_parser_new( ml_screen_t *  screen , ml_char_encoding_t  encoding ,
+	int  is_auto_encoding , int  use_auto_detect , int  logging_vt_seq ,
 	ml_unicode_policy_t  policy , u_int  col_size_a ,
 	int  use_char_combining , int  use_multi_col_char ,
-	char *  win_name , char *  icon_name) ;
+	const char *  win_name , const char *  icon_name , ml_alt_color_mode_t  alt_color_mode) ;
 
 int  ml_vt100_parser_delete( ml_vt100_parser_t *  vt100_parser) ;
 
-int  ml_vt100_parser_set_pty( ml_vt100_parser_t *  vt100_parser , ml_pty_ptr_t  pty) ;
+void  ml_vt100_parser_set_pty( ml_vt100_parser_t *  vt100_parser , ml_pty_ptr_t  pty) ;
 
-int  ml_vt100_parser_set_xterm_listener( ml_vt100_parser_t *  vt100_parser ,
+void  ml_vt100_parser_set_xterm_listener( ml_vt100_parser_t *  vt100_parser ,
 	ml_xterm_event_listener_t *  xterm_listener) ;
 
-int  ml_vt100_parser_set_config_listener( ml_vt100_parser_t *  vt100_parser ,
+void  ml_vt100_parser_set_config_listener( ml_vt100_parser_t *  vt100_parser ,
 	ml_config_event_listener_t *  config_listener) ;
 
 int  ml_parse_vt100_sequence( ml_vt100_parser_t *  vt100_parser) ;
@@ -259,15 +303,21 @@ void  ml_reset_pending_vt100_sequence( ml_vt100_parser_t *  vt100_parser) ;
 
 /* Must be called in visual context. */
 int  ml_vt100_parser_write_loopback( ml_vt100_parser_t *  vt100_parser ,
-	u_char *  buf , size_t  len) ;
+	const u_char *  buf , size_t  len) ;
+
+#ifdef  __ANDROID__
+int  ml_vt100_parser_preedit( ml_vt100_parser_t *  vt100_parser ,
+	const u_char *  buf , size_t  len) ;
+#endif
 
 /* Must be called in visual context. */
-int  ml_vt100_parser_local_echo( ml_vt100_parser_t *  vt100_parser , u_char *  buf , size_t  len) ;
+int  ml_vt100_parser_local_echo( ml_vt100_parser_t *  vt100_parser ,
+	const u_char *  buf , size_t  len) ;
 
 int  ml_vt100_parser_change_encoding( ml_vt100_parser_t *  vt100_parser ,
 	ml_char_encoding_t  encoding) ;
 
-ml_char_encoding_t  ml_vt100_parser_get_encoding( ml_vt100_parser_t *  vt100_parser) ;
+#define  ml_vt100_parser_get_encoding( vt100_parser)  ((vt100_parser)->encoding)
 
 size_t  ml_vt100_parser_convert_to( ml_vt100_parser_t *  vt100_parser ,
 	u_char *  dst , size_t  len , mkf_parser_t *  parser) ;
@@ -280,12 +330,6 @@ int  ml_init_encoding_conv( ml_vt100_parser_t *  vt100_parser) ;
 
 #define  ml_get_icon_name( vt100_parser)  ((vt100_parser)->icon_name)
 
-int  ml_vt100_parser_set_col_size_of_width_a( ml_vt100_parser_t *  vt100_parser ,
-	u_int  col_size_a) ;
-
-#define  ml_vt100_parser_get_col_size_of_width_a( vt100_parser) \
-		((vt100_parser)->col_size_of_width_a)
-
 #define  ml_vt100_parser_set_use_char_combining( vt100_parser , use) \
 		((vt100_parser)->use_char_combining = (use))
 
@@ -298,11 +342,6 @@ int  ml_vt100_parser_set_col_size_of_width_a( ml_vt100_parser_t *  vt100_parser
 #define  ml_vt100_parser_is_using_multi_col_char( vt100_parser) \
 		((vt100_parser)->use_multi_col_char)
 
-#define  ml_vt100_parser_set_logging_vt_seq( vt100_parser , flag) \
-		((vt100_parser)->logging_vt_seq = (flag))
-
-#define  ml_vt100_parser_is_logging_vt_seq( vt100_parser)  ((vt100_parser)->logging_vt_seq)
-
 #define  ml_vt100_parser_get_mouse_report_mode( vt100_parser)  ((vt100_parser)->mouse_mode)
 
 #define  ml_vt100_parser_get_extended_mouse_report_mode( vt100_parser) \
@@ -330,13 +369,24 @@ int  ml_vt100_parser_set_col_size_of_width_a( ml_vt100_parser_t *  vt100_parser
 
 int  ml_set_auto_detect_encodings( char *  encodings) ;
 
-#define  ml_vt100_parser_set_use_auto_detect( vt100_parser , use) \
-		((vt100_parser)->use_auto_detect = (use))
-
-#define  ml_vt100_parser_is_using_auto_detect( vt100_parser) ((vt100_parser)->use_auto_detect)
-
 int  ml_convert_to_internal_ch( mkf_char_t *  ch , ml_unicode_policy_t  unicode_policy ,
 		mkf_charset_t  gl) ;
 
+#define  ml_vt100_parser_select_drcs( vt100_parser)  ml_drcs_select( (vt100_parser)->drcs)
+
+void  ml_vt100_parser_set_alt_color_mode( ml_vt100_parser_t *  vt100_parser ,
+	ml_alt_color_mode_t  mode) ;
+
+#define  ml_vt100_parser_get_alt_color_mode( vt100_parser)  ((vt100_parser)->alt_color_mode)
+
+int  true_or_false( const char *  str) ;
+
+int  ml_vt100_parser_get_config( ml_vt100_parser_t *  vt100_parser , ml_pty_ptr_t  output ,
+	char *  key , int  to_menu , int *  flag) ;
+
+int  ml_vt100_parser_set_config( ml_vt100_parser_t *  vt100_parser , char *  key , char *  val) ;
+
+int  ml_vt100_parser_exec_cmd( ml_vt100_parser_t *  vt100_parser , char *  cmd) ;
+
 
 #endif
