@@ -11,6 +11,7 @@
 #include  <c_intl.h>
 
 #include  "mc_char_encoding.h"
+#include  "mc_auto_detect.h"
 #include  "mc_color.h"
 #include  "mc_bgtype.h"
 #include  "mc_alpha.h"
@@ -28,7 +29,6 @@
 #include  "mc_bel.h"
 #include  "mc_sb.h"
 #include  "mc_im.h"
-#include  "mc_check.h"
 #include  "mc_sb_view.h"
 #include  "mc_io.h"
 #include  "mc_pty.h"
@@ -45,19 +45,15 @@
 
 /* --- static functions --- */
 
-static gint
+static void
 end_application(
 	GtkWidget *  widget ,
-	GdkEvent *  event ,
 	gpointer  data
 	)
 {
 	gtk_main_quit() ;
-
-	return  FALSE ;
 }
 
-
 static gint
 bidi_flag_checked(
 	GtkWidget *  widget ,
@@ -84,6 +80,7 @@ update(
 	)
 {
     mc_update_char_encoding() ;
+    mc_update_auto_detect() ;
     mc_update_color(MC_COLOR_FG) ;
     /*
      * alpha must be updated before bgtype because transparent or wall picture
@@ -118,6 +115,7 @@ update(
     mc_update_flag_mode(MC_FLAG_IND);
     mc_update_flag_mode(MC_FLAG_AWIDTH);
     mc_update_flag_mode(MC_FLAG_CLIPBOARD);
+    mc_update_flag_mode(MC_FLAG_LOCALECHO);
 
     mc_flush(io) ;
 
@@ -532,7 +530,7 @@ show(void)
 	GtkWidget *  separator ;
 	
 	window = gtk_window_new( GTK_WINDOW_TOPLEVEL) ;
-	g_signal_connect( window , "delete_event" , G_CALLBACK(end_application) , NULL) ;
+	g_signal_connect( window , "destroy" , G_CALLBACK(end_application) , NULL) ;
 	gtk_window_set_title( GTK_WINDOW(window) , _("mlterm configuration")) ;
 	gtk_container_set_border_width( GTK_CONTAINER(window) , 0) ;
 
@@ -585,6 +583,9 @@ show(void)
 	gtk_widget_show(config_widget);
 	gtk_box_pack_start(GTK_BOX(vbox), config_widget, FALSE, FALSE, 0);
 
+	if (!(config_widget = mc_auto_detect_config_widget_new())) return 0;
+	gtk_widget_show(config_widget);
+	gtk_box_pack_start(GTK_BOX(vbox), config_widget, FALSE, FALSE, 0);
 
 	if (!(config_widget = mc_im_config_widget_new())) return 0;
 	gtk_widget_show(config_widget);
@@ -802,10 +803,20 @@ show(void)
 	gtk_widget_show(config_widget);
 	gtk_box_pack_start(GTK_BOX(vbox), config_widget, FALSE, FALSE, 0);
 
+
+	hbox = gtk_hbox_new(TRUE, 5);
+	gtk_widget_show(hbox);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+
 	if (!(config_widget = mc_flag_config_widget_new(MC_FLAG_CLIPBOARD)))
 	    return 0;
 	gtk_widget_show(config_widget);
-	gtk_box_pack_start(GTK_BOX(vbox), config_widget, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), config_widget, TRUE, TRUE, 0);
+
+	if (!(config_widget = mc_flag_config_widget_new(MC_FLAG_LOCALECHO)))
+	    return 0;
+	gtk_widget_show(config_widget);
+	gtk_box_pack_start(GTK_BOX(hbox), config_widget, TRUE, TRUE, 0);
 
 
 	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_MOUSE);
@@ -847,6 +858,8 @@ main(
 	}
 #endif
 
+	kik_set_msg_log_file_name( "mlterm/msg.log") ;
+
 	gtk_init( &argc , &argv) ;
 
  	if( show() == 0)
