@@ -63,7 +63,17 @@ static char *  encodings[] =
 	"CP1256 (Arabic)" ,
 	"CP1257 (Baltic)" ,
 	"CP1258 (Vietnamese)" ,
-	"ISCII (Indics)" ,
+	"ISCII-ASSAMESE (Indics)" ,
+	"ISCII-BENGALI (Indics)" ,
+	"ISCII-GUJARATI (Indics)" ,
+	"ISCII-HINDI (Indics)" ,
+	"ISCII-KANNAdA (Indics)" ,
+	"ISCII-MALAYALAM (Indics)" ,
+	"ISCII-ORIYA (Indics)" ,
+	"ISCII-PUNJABI (Indics)" ,
+	"ISCII-ROMAN (Indics)" ,
+	"ISCII-TAMIL (Indics)" ,
+	"ISCII-TELUGU (Indics)" ,
 
 	N_("--- Japanese ---"),
 	"EUC-JP" ,
