@@ -9,6 +9,8 @@
 #include  <kiklib/kik_unistd.h>		/* STDIN_FILENO */
 #include  <kiklib/kik_debug.h>
 #include  <kiklib/kik_str.h>
+#include  <kiklib/kik_def.h>		/* USE_WIN32API */
+
 
 /* --- static variables --- */
 
@@ -18,41 +20,10 @@ static int  gui_is_win32 ;
 
 /* --- static functions --- */
 
-static char *
-load_challenge(void)
-{
-	FILE *  fp ;
-	char *  homedir ;
-	char  path[256] ;
-	static char  challenge[64] ;
-
-	if( ! ( homedir = getenv("HOME")))
-	{
-		return  "" ;
-	}
-
-	snprintf( path , sizeof(path) , "%s/.mlterm/challenge" , homedir) ;
-
-	if( ( fp = fopen( path , "r")))
-	{
-		size_t  len ;
-
-		if( ( len = fread( challenge , 1 , sizeof(challenge) - 2 , fp)) > 0)
-		{
-			challenge[len++] = ';' ;
-			challenge[len] = '\0' ;
-		}
-
-		fclose( fp) ;
-	}
-
-	return  challenge ;
-}
-
 static int
 append_value(
-	char *  key ,
-	char *  value
+	const char *  key ,
+	const char *  value
 	)
 {
 	if( message == NULL)
@@ -86,7 +57,7 @@ append_value(
 
 static char *
 get_value(
-	char *  key ,
+	const char *  key ,
 	mc_io_t  io
 	)
 {
@@ -141,7 +112,7 @@ get_value(
 
 int
 mc_exec(
-	char *  cmd
+	const char *  cmd
 	)
 {
 #ifdef __DEBUG
@@ -156,8 +127,8 @@ mc_exec(
 
 int
 mc_set_str_value(
-	char *  key ,
-	char *  value
+	const char *  key ,
+	const char *  value
 	)
 {
 	if (value == NULL) return 0;
@@ -171,7 +142,7 @@ mc_set_str_value(
 
 int
 mc_set_flag_value(
-	char *  key ,
+	const char *  key ,
 	int  flag_val
 	)
 {
@@ -185,19 +156,22 @@ mc_set_flag_value(
 int
 mc_flush(mc_io_t  io)
 {
-	char *  chal = "" ;
+	char *  chal ;
 
 	if( message == NULL)
 	{
 		return  1 ;
 	}
 
-	if( io == mc_io_set_save)
+	if( io == mc_io_set_save && ( chal = get_value( "challenge" , mc_io_get)))
 	{
-		chal = load_challenge() ;
+		printf( "\x1b]%d;%s;%s\x07" , io , chal , message);
+	}
+	else
+	{
+		printf( "\x1b]%d;%s\x07" , io , message);
 	}
 
-	printf( "\x1b]%d;%s%s\x07" , io , chal , message);
 	fflush( stdout) ;
 
 #if  0
@@ -212,7 +186,7 @@ mc_flush(mc_io_t  io)
 
 char *
 mc_get_str_value(
-	char *  key
+	const char *  key
 	)
 {
 	char *  value ;
@@ -229,7 +203,7 @@ mc_get_str_value(
 
 int
 mc_get_flag_value(
-	char *  key
+	const char *  key
 	)
 {
 	char *  value ;
@@ -280,30 +254,35 @@ mc_gui_is_win32(void)
 int
 mc_set_font_name(
 	mc_io_t  io ,
-	char *  file ,
-	char *  font_size ,
-	char *  cs ,
-	char *  font_name
+	const char *  file ,
+	const char *  font_size ,
+	const char *  cs ,
+	const char *  font_name
 	)
 {
-	char *  chal = "" ;
+	char *  chal ;
 
-	if( io == mc_io_set_save_font)
+	if( io == mc_io_set_save_font && ( chal = get_value( "challenge" , mc_io_get)))
 	{
-		chal = load_challenge() ;
+		printf( "\x1b]%d;%s;%s:%s=%s,%s\x07" ,
+			io , chal , file , cs , font_size , font_name) ;
+	}
+	else
+	{
+		printf( "\x1b]%d;%s:%s=%s,%s\x07" ,
+			io , file , cs , font_size , font_name) ;
 	}
 
-	printf( "\x1b]%d;%s%s:%s=%s,%s\x07" , io , chal , file , cs , font_size , font_name) ;
-	fflush( NULL) ;
+	fflush( stdout) ;
 
 	return  1 ;
 }
 
 char *
 mc_get_font_name(
-	char *  file ,
-	char *  font_size ,
-	char *  cs
+	const char *  file ,
+	const char *  font_size ,
+	const char *  cs
 	)
 {
 	size_t  len ;
