@@ -5,13 +5,16 @@
 #include  <stdio.h>
 #include  <unistd.h>	/* STDOUT_FILENO */
 #include  <stdlib.h>	/* mbstowcs_s */
+#include  <wchar.h>	/* wcslen */
 
 extern "C" {
 #include  <kiklib/kik_debug.h>
+#include  <kiklib/kik_conf_io.h>	/* kik_get_user_rc_path */
 }
 
 #include  <kiklib/kik_types.h>	/* u_int32_t/u_int16_t */
 #include  <kiklib/kik_def.h>	/* SSIZE_MAX, USE_WIN32API */
+#include  <kiklib/kik_mem.h>
 #if  defined(__CYGWIN__) || defined(__MSYS__)
 #include  <kiklib/kik_path.h>	/* cygwin_conv_to_win32_path */
 #endif
@@ -24,6 +27,10 @@ extern "C" {
 #include  <windows.h>
 #include  <gdiplus.h>
 
+#ifdef  USE_WIN32API
+#include  <malloc.h>	/* alloca */
+#endif
+
 using namespace Gdiplus ;
 
 #if  0
@@ -48,15 +55,27 @@ static const IID __uuid_inst =
 #define  BUILTIN_IMAGELIB	/* Necessary to use create_cardinals_from_sixel() */
 #include  "../../common/c_imagelib.c"
 
+static void
+help(void)
+{
+	/* Don't output to stdout where mlterm waits for image data. */
+	fprintf( stderr , "mlimgloader [window id] [width] [height] [file path] (-c)\n") ;
+	fprintf( stderr , "  window id: ignored.\n") ;
+	fprintf( stderr , "  -c       : output XA_CARDINAL format data to stdout.\n") ;
+}
+
 static u_int32_t *
 create_cardinals_from_file(
-	const char *  path ,
+	const char *  path ,	/* cygwin style path on cygwin/msys. win32 style path on win32. */
 	u_int  width ,
 	u_int  height
 	)
 {
+	/* MAX_PATH which is 260 (3+255+1+1) is defined in win32 alone. */
 	wchar_t  wpath[MAX_PATH] ;
-	size_t  wpath_len ;
+#if  defined(__CYGWIN__) || defined(__MSYS__)
+	char  winpath[MAX_PATH] ;
+#endif
 	Gdiplus::GdiplusStartupInput  startup ;
 	ULONG_PTR  token ;
 	HMODULE  module ;
@@ -64,6 +83,7 @@ create_cardinals_from_file(
 	IMoniker *  moniker ;
 	IStream *  stream ;
 	Gdiplus::Bitmap *  bitmap ;
+	int  hash ;
 	u_int32_t *  cardinal ;
 	u_int32_t *  p ;
 
@@ -72,12 +92,12 @@ create_cardinals_from_file(
 		return  cardinal ;
 	}
 
-	if( ((ssize_t)( wpath_len = mbstowcs( wpath , path , MAX_PATH))) <= 0 ||
-	    Gdiplus::GdiplusStartup( &token , &startup , NULL) != Gdiplus::Ok)
+	if( Gdiplus::GdiplusStartup( &token , &startup , NULL) != Gdiplus::Ok)
 	{
 		return  NULL ;
 	}
 
+	hash = hash_path( path) ;
 	stream = NULL ;
 
 	if( strstr( path , "://"))
@@ -90,6 +110,9 @@ create_cardinals_from_file(
 		    ( create_url_moniker = (func)GetProcAddress( module ,
 							"CreateURLMonikerEx")))
 		{
+			MultiByteToWideChar( CP_UTF8 , 0 , path , strlen(path) + 1 ,
+				wpath , MAX_PATH) ;
+
 			if( (*create_url_moniker)( NULL , wpath , &moniker ,
 					URL_MK_UNIFORM) == S_OK)
 			{
@@ -114,6 +137,88 @@ create_cardinals_from_file(
 			}
 		}
 	}
+#if  defined(__CYGWIN__) || defined(__MSYS__)
+	else if( strchr( path , '/'))	/* In case win32 style path is specified on cygwin/msys. */
+	{
+		/* cygwin style path => win32 style path. */
+		cygwin_conv_to_win32_path( path , winpath) ;
+		path = winpath ;
+	}
+#endif
+
+	if( strcmp( path + strlen(path) - 4 , ".gif") == 0)
+	{
+		/* Animation GIF */
+
+		char *  dir ;
+
+	#if  defined(__CYGWIN__) || defined(__MSYS__)
+		/* converted to win32 by cygwin_conv_to_win32_path */
+		if( ! strstr( path , "mlterm\\anim") &&
+		    ( dir = kik_get_user_rc_path( "mlterm/")))
+	#else
+		if( ! strstr( path , "mlterm/anim") &&
+		    ( dir = kik_get_user_rc_path( "mlterm\\")))
+	#endif
+		{
+			char *  new_path ;
+
+			if( ! ( new_path = (char*)alloca( strlen( dir) + 8 +
+						5 + DIGIT_STR_LEN(int) + 1)))
+			{
+				goto  end0 ;
+			}
+
+			sprintf( new_path , "%sanim%d.gif" , dir , hash) ;
+
+			if( stream)
+			{
+				FILE *  fp ;
+				BYTE  buf[10240] ;
+				ULONG  rd_len ;
+				HRESULT  res ;
+
+				if( ! ( fp = fopen( new_path , "wb")))
+				{
+					goto  end0 ;
+				}
+
+				do
+				{
+					res = stream->Read( buf , sizeof(buf) , &rd_len) ;
+					fwrite( buf , 1 , rd_len , fp) ;
+				}
+				while( res == Gdiplus::Ok) ;
+
+				fclose( fp) ;
+
+				stream->Release() ;
+				ctx->Release() ;
+				moniker->Release() ;
+				FreeLibrary( module) ;
+				stream = NULL ;
+
+				path = new_path ;
+			}
+
+			split_animation_gif( path , dir , hash) ;
+
+		#if  defined(__CYGWIN__) || defined(__MSYS__)
+			cygwin_conv_to_win32_path( new_path , winpath) ;
+			new_path = winpath ;
+		#endif
+			/* Replace path by the splitted file. */
+			path = new_path ;
+
+		end0:
+			free( dir) ;
+		}
+	}
+
+	if( ! stream)
+	{
+		MultiByteToWideChar( CP_UTF8 , 0 , path , strlen(path) + 1 , wpath , MAX_PATH) ;
+	}
 
 	cardinal = NULL ;
 
@@ -131,14 +236,15 @@ create_cardinals_from_file(
 		Image *  image ;
 		Graphics *  graphics ;
 
-		if( stream ?
-		    ! ( image = Image::FromStream( stream)) :
-		    ! ( image = Image::FromFile( wpath)) )
+		if( ( stream ?
+		      ! ( image = Image::FromStream( stream)) :
+		      ! ( image = Image::FromFile( wpath))) ||
+		    image->GetLastStatus() != Gdiplus::Ok)
 		{
 			goto  end1 ;
 		}
 
-		if( ! ( bitmap = new Bitmap( width , height)))
+		if( ! ( bitmap = new Bitmap( width , height , PixelFormat32bppARGB)))
 		{
 			delete  image ;
 
@@ -211,12 +317,18 @@ end1:
 
 /* --- global functions --- */
 
-int
-main(
-	int  argc ,
-	char **  argv
+int PASCAL
+WinMain(
+	HINSTANCE  hinst ,
+	HINSTANCE  hprev ,
+	char *  cmdline ,
+	int  cmdshow
 	)
 {
+	WCHAR **  w_argv ;
+	char **  argv ;
+	int  argc ;
+	int  count ;
 	u_char *  cardinal ;
 	u_int  width ;
 	u_int  height ;
@@ -226,11 +338,48 @@ main(
 	kik_set_msg_log_file_name( "mlterm/msg.log") ;
 #endif
 
+	w_argv = CommandLineToArgvW( GetCommandLineW() , &argc) ;
+	if( argc == 0 || ! ( argv = (char**)alloca( sizeof(*argv) * argc)))
+	{
+		GlobalFree( w_argv) ;
+
+		return  -1 ;
+	}
+
+	for( count = 0 ; count < argc ; count++)
+	{
+		int  len ;
+
+		len = WideCharToMultiByte( CP_UTF8 , 0 , w_argv[count] ,
+			wcslen( w_argv[count]) + 1 , NULL , 0 , NULL , NULL) ;
+		if( ( argv[count] = (char*)alloca( len)))
+		{
+			WideCharToMultiByte( CP_UTF8 , 0 , w_argv[count] ,
+				wcslen( w_argv[count]) + 1 , argv[count] , len , NULL , NULL) ;
+		}
+	}
+
+	GlobalFree( w_argv) ;
+
 	if( argc != 6 || strcmp( argv[5] , "-c") != 0)
 	{
+		help() ;
+
 		return  -1 ;
 	}
 
+	/* kik_str_alloca_dup() fails by illegal cast from void* to char*. */
+	char  new_path[strlen(argv[4]) + 1] ;
+
+	if( strstr( argv[4] , ".rgs"))
+	{
+		strcpy( new_path , argv[4]) ;
+		if( convert_regis_to_bmp( new_path))
+		{
+			argv[4] = new_path ;
+		}
+	}
+
 	width = atoi( argv[2]) ;
 	height = atoi( argv[3]) ;
 
@@ -241,19 +390,11 @@ main(
 
 	if( ! ( cardinal = (u_char*)create_cardinals_from_file( argv[4] , width , height)))
 	{
-	#if  defined(__CYGWIN__) || defined(__MSYS__)
-		char  winpath[MAX_PATH] ;
-		cygwin_conv_to_win32_path( argv[4] , winpath) ;
-
-		if( ! ( cardinal = (u_char*)create_cardinals_from_file( winpath , width , height)))
+	#ifdef  DEBUG
+		kik_debug_printf( KIK_DEBUG_TAG " Failed to load %s\n" , argv[4]) ;
 	#endif
-		{
-		#ifdef  DEBUG
-			kik_debug_printf( KIK_DEBUG_TAG " Failed to load %s\n" , argv[4]) ;
-		#endif
 
-			return  -1 ;
-		}
+		goto  error ;
 	}
 
 	width = ((u_int32_t*)cardinal)[0] ;
@@ -270,7 +411,7 @@ main(
 
 		if( ( n_wr = write( STDOUT_FILENO , cardinal , size)) < 0)
 		{
-			return  -1 ;
+			goto  error ;
 		}
 
 		cardinal += n_wr ;
@@ -282,4 +423,9 @@ main(
 #endif
 
 	return  0 ;
+
+error:
+	kik_error_printf( "Couldn't load %s\n" , argv[4]) ;
+
+	return  -1 ;
 }
