@@ -20,6 +20,7 @@
 #include  <ml_color.h>
 
 #include  "../x_window.h"
+#include  "../x_picture.h"
 #include  "x_virtual_kbd.h"
 
 
@@ -86,14 +87,17 @@
 #define  ENABLE_DOUBLE_BUFFER
 #endif
 
-/* Parameters of cursor_shape */
-#define  CURSOR_WIDTH   7
-#define  CURSOR_X_OFF   -3
-#define  CURSOR_HEIGHT  15
-#define  CURSOR_Y_OFF   -7
+/* Parameters of mouse cursor */
+#define  MAX_CURSOR_SHAPE_WIDTH  15
+#define  CURSOR_SHAPE_SIZE  (7*15*sizeof(u_int32_t))
 
 
-/* Note that this structure could be casted to Display */
+/*
+ * Note that this structure could be casted to Display.
+ *
+ * x, y, width and height are on the physical display.
+ * (rotate_display is not considered.)
+ */
 typedef struct
 {
 	int  fd ;	/* Same as Display */
@@ -119,7 +123,7 @@ typedef struct
 
 	} cursor ;
 
-	u_char  saved_image[sizeof(u_int32_t) * CURSOR_WIDTH * CURSOR_HEIGHT * 4] ;
+	u_char  saved_image[CURSOR_SHAPE_SIZE] ;
 	int  hidden_region_saved ;
 
 } Mouse ;
@@ -133,32 +137,60 @@ static Mouse  _mouse ;
 static x_display_t  _disp_mouse ;
 static x_display_t *  opened_disps[] = { &_disp , &_disp_mouse } ;
 static int  use_ansi_colors = 1 ;
+static int  rotate_display = 0 ;
 
 static struct termios  orig_tm ;
 
-static char *  cursor_shape =
+static const char  cursor_shape_normal[] =
+	"#######"
+	"#*****#"
+	"###*###"
+	"  #*#  "
+	"  #*#  "
+	"  #*#  "
+	"  #*#  "
+	"  #*#  "
+	"  #*#  "
+	"  #*#  "
+	"  #*#  "
+	"  #*#  "
+	"###*###"
+	"#*****#"
+	"#######" ;
+
+static const char  cursor_shape_rotate[] =
+	"###         ###"
+	"#*#         #*#"
+	"#*###########*#"
+	"#*************#"
+	"#*###########*#"
+	"#*#         #*#"
+	"###         ###" ;
+
+static struct cursor_shape
 {
-	"#######"
-	"#*****#"
-	"###*###"
-	"  #*#  "
-	"  #*#  "
-	"  #*#  "
-	"  #*#  "
-	"  #*#  "
-	"  #*#  "
-	"  #*#  "
-	"  #*#  "
-	"  #*#  "
-	"###*###"
-	"#*****#"
-	"#######"
+	const char *  shape ;
+	u_int  width ;
+	u_int  height ;
+	int  x_off ;
+	int  y_off ;
+
+} cursor_shape =
+{
+	cursor_shape_normal ,
+	 7 ,	/* width */
+	15 ,	/* height */
+	-3 ,	/* x_off */
+	-7	/* y_off */
 } ;
 
 
 /* --- static functions --- */
 
-/* _disp.roots[1] is ignored. */
+/*
+ * _disp.roots[1] is ignored.
+ * x and y are rotated values.
+ */
 static inline x_window_t *
 get_window(
 	int  x ,	/* X in display */
@@ -335,11 +367,13 @@ put_image_124bpp(
 			}
 			else
 			{
+			#ifndef  ENABLE_2_4_PPB
 				if( _disp.depth > 1)
 				{
 					goto  round_number_1 ;
 				}
 				else
+			#endif
 				{
 					goto  round_number_2 ;
 				}
@@ -449,9 +483,9 @@ put_image_124bpp(
 			#ifdef  ENABLE_DOUBLE_BUFFER
 				fb_pixel = _display.back_fb[
 						get_fb( x + size , y) - _display.fb +
-						_display.plane_len * plane] ;
+						_display.plane_offset[plane]] ;
 			#else
-				fb_pixel = get_fb( x + size , y)[_display.plane_len * plane] ;
+				fb_pixel = get_fb( x + size , y)[_display.plane_offset[plane]] ;
 			#endif
 
 				do
@@ -472,11 +506,14 @@ put_image_124bpp(
 
 		if( ++plane < _disp.depth)
 		{
-			fb += _display.plane_len ;
+			size_t  offset ;
+
+			offset = _display.plane_offset[plane] - _display.plane_offset[plane - 1] ;
+			fb += offset ;
 		#ifdef  ENABLE_DOUBLE_BUFFER
 			if( write_back_fb)
 			{
-				new_image += _display.plane_len ;
+				new_image += offset ;
 			}
 		#endif
 		}
@@ -487,39 +524,56 @@ put_image_124bpp(
 	}
 }
 
+static void
+rotate_mouse_cursor_shape(void)
+{
+	int  tmp ;
+
+	cursor_shape.shape = (cursor_shape.shape == cursor_shape_normal) ?
+				cursor_shape_rotate : cursor_shape_normal ;
+
+	tmp = cursor_shape.x_off ;
+	cursor_shape.x_off = cursor_shape.y_off ;
+	cursor_shape.y_off = tmp ;
+
+	tmp = cursor_shape.width ;
+	cursor_shape.width = cursor_shape.height ;
+	cursor_shape.height = tmp ;
+}
+
 static void
 update_mouse_cursor_state(void)
 {
-	if( -CURSOR_X_OFF > _mouse.x)
+	if( -cursor_shape.x_off > _mouse.x)
 	{
 		_mouse.cursor.x = 0 ;
-		_mouse.cursor.x_off = -CURSOR_X_OFF - _mouse.x ;
+		_mouse.cursor.x_off = -cursor_shape.x_off - _mouse.x ;
 	}
 	else
 	{
-		_mouse.cursor.x = _mouse.x + CURSOR_X_OFF ;
+		_mouse.cursor.x = _mouse.x + cursor_shape.x_off ;
 		_mouse.cursor.x_off = 0 ;
 	}
 
-	_mouse.cursor.width = CURSOR_WIDTH - _mouse.cursor.x_off ;
+	_mouse.cursor.width = cursor_shape.width - _mouse.cursor.x_off ;
 	if( _mouse.cursor.x + _mouse.cursor.width > _display.width)
 	{
 		_mouse.cursor.width -=
 			(_mouse.cursor.x + _mouse.cursor.width - _display.width) ;
 	}
 
-	if( -CURSOR_Y_OFF > _mouse.y)
+	if( -cursor_shape.y_off > _mouse.y)
 	{
 		_mouse.cursor.y = 0 ;
-		_mouse.cursor.y_off = -CURSOR_Y_OFF - _mouse.y ;
+		_mouse.cursor.y_off = -cursor_shape.y_off - _mouse.y ;
 	}
 	else
 	{
-		_mouse.cursor.y = _mouse.y + CURSOR_Y_OFF ;
+		_mouse.cursor.y = _mouse.y + cursor_shape.y_off ;
 		_mouse.cursor.y_off = 0 ;
 	}
 
-	_mouse.cursor.height = CURSOR_HEIGHT - _mouse.cursor.y_off ;
+	_mouse.cursor.height = cursor_shape.height - _mouse.cursor.y_off ;
 	if( _mouse.cursor.y + _mouse.cursor.height > _display.height)
 	{
 		_mouse.cursor.height -=
@@ -562,7 +616,7 @@ restore_hidden_region(void)
 		for( plane = 0 ; plane < num_of_planes ; plane++)
 		{
 			fb = get_fb(_mouse.cursor.x , _mouse.cursor.y) +
-				_display.plane_len * plane ;
+				_display.plane_offset[plane] ;
 
 			for( count = 0 ; count < _mouse.cursor.height ; count++)
 			{
@@ -575,16 +629,47 @@ restore_hidden_region(void)
 	else
 	{
 		x_window_t *  win ;
+		int  cursor_x ;
+		int  cursor_y ;
+		u_int  cursor_height ;
+		u_int  cursor_width ;
 
-		win = get_window( _mouse.x , _mouse.y) ;
+		if( rotate_display)
+		{
+			win = get_window( _mouse.y , _display.width - _mouse.x - 1) ;
+
+			if( rotate_display > 0)
+			{
+				cursor_x = _mouse.cursor.y ;
+				cursor_y = _display.width - _mouse.cursor.x - _mouse.cursor.width ;
+			}
+			else
+			{
+				cursor_x = _display.height - _mouse.cursor.y -
+						_mouse.cursor.height ;
+				cursor_y = _mouse.cursor.x ;
+			}
+
+			cursor_width = _mouse.cursor.height ;
+			cursor_height = _mouse.cursor.width ;
+		}
+		else
+		{
+			win = get_window( _mouse.x , _mouse.y) ;
+
+			cursor_x = _mouse.cursor.x ;
+			cursor_y = _mouse.cursor.y ;
+			cursor_width = _mouse.cursor.width ;
+			cursor_height = _mouse.cursor.height ;
+		}
 
 		if( win->window_exposed)
 		{
 			(*win->window_exposed)( win ,
-				_mouse.cursor.x - win->x - win->hmargin ,
-				_mouse.cursor.y - win->y - win->vmargin ,
-				_mouse.cursor.width ,
-				_mouse.cursor.height) ;
+				cursor_x - win->x - win->hmargin ,
+				cursor_y - win->y - win->vmargin ,
+				cursor_width ,
+				cursor_height) ;
 		}
 	}
 }
@@ -613,7 +698,7 @@ save_hidden_region(void)
 
 	for( plane = 0 ; plane < num_of_planes ; plane++)
 	{
-		fb = get_fb( _mouse.cursor.x , _mouse.cursor.y) + _display.plane_len * plane ;
+		fb = get_fb( _mouse.cursor.x , _mouse.cursor.y) + _display.plane_offset[plane] ;
 
 		for( count = 0 ; count < _mouse.cursor.height ; count++)
 		{
@@ -634,14 +719,29 @@ draw_mouse_cursor_line(
 	u_char *  fb ;
 	x_window_t *  win ;
 	char *  shape ;
-	u_char  image[CURSOR_WIDTH * sizeof(u_int32_t)] ;
+	u_char  image[MAX_CURSOR_SHAPE_WIDTH * sizeof(u_int32_t)] ;
 	int  x ;
 
 	fb = get_fb( _mouse.cursor.x , _mouse.cursor.y + y) ;
 
-	win = get_window( _mouse.x , _mouse.y) ;
-	shape = cursor_shape +
-		((_mouse.cursor.y_off + y) * CURSOR_WIDTH + _mouse.cursor.x_off) ;
+	if( rotate_display)
+	{
+		if( rotate_display > 0)
+		{
+			win = get_window( _mouse.y , _display.width - _mouse.x - 1) ;
+		}
+		else
+		{
+			win = get_window( _display.height - _mouse.y - 1 , _mouse.x) ;
+		}
+	}
+	else
+	{
+		win = get_window( _mouse.x , _mouse.y) ;
+	}
+
+	shape = cursor_shape.shape +
+		((_mouse.cursor.y_off + y) * cursor_shape.width + _mouse.cursor.x_off) ;
 
 	for( x = 0 ; x < _mouse.cursor.width ; x++)
 	{
@@ -685,10 +785,14 @@ draw_mouse_cursor_line(
 		{
 			switch( _display.bytes_per_pixel)
 			{
+				int  tmp ;
 			case  1:
+				tmp = rotate_display ;
+				rotate_display = 0 ;
 				image[x] = x_display_get_pixel(
 						_mouse.cursor.x + x ,
 						_mouse.cursor.y + y) ;
+				rotate_display = tmp ;
 				break ;
 
 			case  2:
@@ -788,6 +892,66 @@ expose_display(
 	}
 }
 
+static int
+check_visibility_of_im_window(void)
+{
+	static struct
+	{
+		int  saved ;
+		int  x ;
+		int  y ;
+		u_int  width ;
+		u_int  height ;
+
+	} im_region ;
+	int  redraw_im_win ;
+
+	redraw_im_win = 0 ;
+
+	if( _disp.num_of_roots == 2 && _disp.roots[1]->is_mapped)
+	{
+		if( im_region.saved)
+		{
+			if( im_region.x == _disp.roots[1]->x &&
+			    im_region.y == _disp.roots[1]->y &&
+			    im_region.width == ACTUAL_WIDTH(_disp.roots[1]) &&
+			    im_region.height == ACTUAL_HEIGHT(_disp.roots[1]))
+			{
+				return  0 ;
+			}
+
+			if( im_region.x < _disp.roots[1]->x ||
+			    im_region.y < _disp.roots[1]->y ||
+			    im_region.x + im_region.width >
+				_disp.roots[1]->x + ACTUAL_WIDTH(_disp.roots[1]) ||
+			    im_region.y + im_region.height >
+				_disp.roots[1]->y + ACTUAL_HEIGHT(_disp.roots[1]))
+			{
+				expose_display( im_region.x , im_region.y ,
+					im_region.width , im_region.height) ;
+				redraw_im_win = 1 ;
+			}
+		}
+
+		im_region.saved = 1 ;
+		im_region.x = _disp.roots[1]->x ;
+		im_region.y = _disp.roots[1]->y ;
+		im_region.width = ACTUAL_WIDTH(_disp.roots[1]) ;
+		im_region.height = ACTUAL_HEIGHT(_disp.roots[1]) ;
+	}
+	else
+	{
+		if( im_region.saved)
+		{
+			expose_display( im_region.x , im_region.y ,
+				im_region.width , im_region.height) ;
+			im_region.saved = 0 ;
+		}
+	}
+
+	return  redraw_im_win ;
+}
+
 static void
 receive_event_for_multi_roots(
 	XEvent *  xev
@@ -795,7 +959,7 @@ receive_event_for_multi_roots(
 {
 	int  redraw_im_win ;
 
-	if( ( redraw_im_win = x_display_check_visibility_of_im_window()))
+	if( ( redraw_im_win = check_visibility_of_im_window()))
 	{
 		/* Stop drawing input method window */
 		_disp.roots[1]->is_mapped = 0 ;
@@ -808,12 +972,13 @@ receive_event_for_multi_roots(
 		/* Restart drawing input method window */
 		_disp.roots[1]->is_mapped = 1 ;
 	}
-
-	if( x_display_check_visibility_of_im_window())
+	else if( ! check_visibility_of_im_window())
 	{
-		expose_window( _disp.roots[1] , _disp.roots[1]->x , _disp.roots[1]->y ,
-				ACTUAL_WIDTH(_disp.roots[1]) , ACTUAL_HEIGHT(_disp.roots[1])) ;
+		return ;
 	}
+
+	expose_window( _disp.roots[1] , _disp.roots[1]->x , _disp.roots[1]->y ,
+				ACTUAL_WIDTH(_disp.roots[1]) , ACTUAL_HEIGHT(_disp.roots[1])) ;
 }
 
 /*
@@ -829,7 +994,7 @@ check_virtual_kbd(
 	XKeyEvent  kev ;
 	int  ret ;
 
-	if( ! ( ret = x_is_virtual_kbd_event( &_disp , bev)))
+	if( rotate_display || ! ( ret = x_is_virtual_kbd_event( &_disp , bev)))
 	{
 		return  0 ;
 	}
@@ -919,6 +1084,20 @@ receive_stdin_key_event(void)
 			{ "[B" , XK_Down } ,
 			{ "[C" , XK_Right } ,
 			{ "[D" , XK_Left } ,
+		#if  defined(USE_GRF)
+			{ "[7~" , XK_End } ,
+			{ "[1~" , XK_Home } ,
+			{ "OP" , XK_F1 } ,
+			{ "OQ" , XK_F2 } ,
+			{ "OR" , XK_F3 } ,
+			{ "OS" , XK_F4 } ,
+			{ "[17~" , XK_F5 } ,
+			{ "[18~" , XK_F6 } ,
+			{ "[19~" , XK_F7 } ,
+			{ "[20~" , XK_F8 } ,
+			{ "[21~" , XK_F9 } ,
+			{ "[29~" , XK_F10 } ,
+		#else	/* USE_GRF */
 		#if  defined(__NetBSD__) || defined(__OpenBSD__)
 			{ "[8~" , XK_End } ,
 			{ "[7~" , XK_Home } ,
@@ -952,6 +1131,7 @@ receive_stdin_key_event(void)
 			{ "[21~" , XK_F10 } ,
 			{ "[23~" , XK_F11 } ,
 			{ "[24~" , XK_F12 } ,
+		#endif	/* USE_GRF */
 		} ;
 
 		size_t  count ;
@@ -1093,6 +1273,10 @@ cmap_new(
 	return  cmap ;
 }
 
+/*
+ * Note that x_display_wscons.c has its own cmap_init() because vinfo.depth != _disp.depth
+ * on NetBSD/luna68k etc.
+ */
 static int
 cmap_init(void)
 {
@@ -1147,6 +1331,14 @@ cmap_init(void)
 
 			return  0 ;
 		}
+
+		if( ! ( _display.color_cache = calloc( 1 , sizeof(*_display.color_cache))))
+		{
+			free( _display.cmap_orig) ;
+			free( _display.cmap) ;
+
+			return  0 ;
+		}
 	}
 
 	if( num_of_colors == 2)
@@ -1180,6 +1372,7 @@ cmap_init(void)
 		_display.cmap->blue[color] = BYTE_COLOR_TO_WORD(b) ;
 	}
 
+	/* Same processing as x_display_set_cmap(). */
 #ifndef  USE_GRF
 	ioctl( _display.fb_fd , FBIOPUTCMAP , _display.cmap) ;
 #else
@@ -1196,9 +1389,6 @@ cmap_init(void)
 	}
 #endif
 
-	_display.prev_pixel = 0xff000000 ;
-	_display.prev_closest_pixel = 0xff000000 ;
-
 	return  1 ;
 }
 
@@ -1218,6 +1408,7 @@ cmap_final(void)
 	}
 
 	free( _display.cmap) ;
+	free( _display.color_cache) ;
 }
 
 
@@ -1249,6 +1440,23 @@ x_display_open(
 			_display.height) ;
 	#endif
 
+		if( rotate_display)
+		{
+			u_int  tmp ;
+
+			if( _display.bytes_per_pixel < 2)
+			{
+				rotate_display = 0 ;
+				rotate_mouse_cursor_shape() ;
+			}
+			else
+			{
+				tmp = _disp.width ;
+				_disp.width = _disp.height ;
+				_disp.height = tmp ;
+			}
+		}
+
 		fcntl( STDIN_FILENO , F_SETFL ,
 			fcntl( STDIN_FILENO , F_GETFL , 0) | O_NONBLOCK) ;
 
@@ -1279,6 +1487,8 @@ x_display_close_all(void)
 {
 	if( DISP_IS_INITED)
 	{
+		x_picture_display_closed( _disp.display) ;
+
 		x_virtual_kbd_hide() ;
 
 		if( MOUSE_IS_INITED)
@@ -1534,8 +1744,10 @@ x_display_get_group_leader(
 int
 x_display_reset_cmap(void)
 {
-	_display.prev_pixel = 0xff000000 ;
-	_display.prev_closest_pixel = 0xff000000 ;
+	if( _display.color_cache)
+	{
+		memset( _display.color_cache , 0 , sizeof(*_display.color_cache)) ;
+	}
 
 	return  _display.cmap && cmap_init()
 	#ifdef  USE_GRF
@@ -1552,13 +1764,46 @@ x_display_set_use_ansi_colors(
 	use_ansi_colors = use ;
 }
 
+void
+x_display_enable_to_change_cmap(
+	int  flag
+	)
+{
+	if( flag)
+	{
+	#ifdef  USE_GRF
+		x68k_set_use_tvram_colors( 1) ;
+	#endif
+
+		if( ! use_ansi_colors)
+		{
+			use_ansi_colors = -1 ;
+		}
+	}
+	else
+	{
+	#ifdef  USE_GRF
+		x68k_set_use_tvram_colors( 0) ;
+	#endif
+
+		if( use_ansi_colors == -1)
+		{
+			use_ansi_colors = 0 ;
+		}
+	}
+}
+
 void
 x_display_set_cmap(
 	u_int32_t *  pixels ,
 	u_int  cmap_size
 	)
 {
-	if( ! use_ansi_colors && cmap_size <= 16 && _disp.depth == 4)
+	if(
+	#ifdef  USE_GRF
+	    ! x68k_set_tvram_cmap( pixels , cmap_size) &&
+	#endif
+	    use_ansi_colors == -1 && cmap_size <= 16 && _disp.depth == 4)
 	{
 		u_int  count ;
 		ml_color_t  color ;
@@ -1584,7 +1829,84 @@ x_display_set_cmap(
 			_display.cmap->blue[color] = pixels[count] & 0xff ;
 		}
 
+		/* Same processing as cmap_init(). */
+	#ifndef  USE_GRF
 		ioctl( _display.fb_fd , FBIOPUTCMAP , _display.cmap) ;
+	#else
+		for( count = 0 ; count < CMAP_SIZE(_display.cmap) ; count++)
+		{
+			((fb_reg_t*)_display.fb)->gpal[count] =
+				(_display.cmap->red[count] >> 3) << 6 |
+				(_display.cmap->green[count] >> 3) << 11 |
+				(_display.cmap->blue[count] >> 3) << 1 ;
+		}
+
+		gpal_init( ((fb_reg_t*)_display.fb)->gpal) ;
+	#endif
+
+		if( _display.color_cache)
+		{
+			memset( _display.color_cache , 0 , sizeof(*_display.color_cache)) ;
+		}
+
+		kik_msg_printf( "Palette changed.\n") ;
+	}
+}
+
+void
+x_display_rotate(
+	int  rotate	/* 1: clockwise, -1: counterclockwise */
+	)
+{
+	if( rotate == rotate_display)
+	{
+		return ;
+	}
+
+	if( rotate)
+	{
+		if( DISP_IS_INITED)
+		{
+			if( _display.bytes_per_pixel < 2)
+			{
+				/* rotate is available for 16 or more bpp */
+
+				return ;
+			}
+
+			/* virual kbd is unavailable on the rotated display. */
+			x_virtual_kbd_hide() ;
+		}
+	}
+
+	if( rotate_display + rotate != 0)
+	{
+		int  tmp ;
+
+		tmp = _disp.width ;
+		_disp.width = _disp.height ;
+		_disp.height = tmp ;
+
+		rotate_mouse_cursor_shape() ;
+
+		rotate_display = rotate ;
+
+		if( _disp.num_of_roots > 0)
+		{
+			x_window_resize_with_margin( _disp.roots[0] ,
+				_disp.width , _disp.height , NOTIFY_TO_MYSELF) ;
+		}
+	}
+	else
+	{
+		/* If rotate_display == -1 rotate == 1 or vice versa, don't swap. */
+
+		rotate_display = rotate ;
+
+		if( _disp.num_of_roots > 0)
+		{
+			x_window_update_all( _disp.roots[0]) ;
+		}
 	}
 }
 
@@ -1603,6 +1925,24 @@ x_display_get_pixel(
 		return  BG_MAGIC ;
 	}
 
+	if( rotate_display)
+	{
+		int  tmp ;
+
+		if( rotate_display > 0)
+		{
+			tmp = x ;
+			x = _disp.height - y - 1 ;
+			y = tmp ;
+		}
+		else
+		{
+			tmp = x ;
+			x = y ;
+			y = _disp.width - tmp - 1 ;
+		}
+	}
+
 	fb = get_fb( x , y) ;
 
 	switch( _display.bytes_per_pixel)
@@ -1636,15 +1976,80 @@ x_display_put_image(
 	{
 		put_image_124bpp( x , y , image , size , 1 , need_fb_pixel) ;
 	}
-	else
+	else if( ! rotate_display)
 	{
 		memcpy( get_fb( x , y) , image , size) ;
 	}
+	else
+	{
+		/* Display is rotated. */
+
+		u_char *  fb ;
+		int  tmp ;
+		int  line_length ;
+		size_t  count ;
+
+		tmp = x ;
+		if( rotate_display > 0)
+		{
+			x = _disp.height - y - 1 ;
+			y = tmp ;
+			line_length = _display.line_length ;
+		}
+		else
+		{
+			x = y ;
+			y = _disp.width - tmp - 1 ;
+			line_length = - _display.line_length ;
+		}
+
+		fb = get_fb( x , y) ;
+
+		if( _display.bytes_per_pixel == 2)
+		{
+			size /= 2 ;
+			for( count = 0 ; count < size ; count++)
+			{
+				*((u_int16_t*)fb) = ((u_int16_t*)image)[count] ;
+				fb += line_length ;
+			}
+		}
+		else /* if( _display.bytes_per_pixel == 4) */
+		{
+			size /= 4 ;
+			for( count = 0 ; count < size ; count++)
+			{
+				*((u_int32_t*)fb) = ((u_int32_t*)image)[count] ;
+				fb += line_length ;
+			}
+		}
+
+		if( rotate_display < 0)
+		{
+			y -= (size - 1) ;
+		}
+
+		if( /* MOUSE_IS_INITED && */ _mouse.cursor.is_drawn &&
+		    _mouse.cursor.x <= x &&
+		    x < _mouse.cursor.x + _mouse.cursor.width)
+		{
+			if( y <= _mouse.cursor.y + _mouse.cursor.height &&
+			    _mouse.cursor.y < y + size)
+			{
+				_mouse.hidden_region_saved = 0 ;
+				/* draw_mouse_cursor() */
+			}
+		}
+
+		return ;
+	}
 
 	if( /* MOUSE_IS_INITED && */ _mouse.cursor.is_drawn &&
 	    _mouse.cursor.y <= y &&
 	    y < _mouse.cursor.y + _mouse.cursor.height)
 	{
+		size /= _display.bytes_per_pixel ;
+
 		if( x <= _mouse.cursor.x + _mouse.cursor.width &&
 		    _mouse.cursor.x < x + size)
 		{
@@ -1654,7 +2059,10 @@ x_display_put_image(
 	}
 }
 
-/* Check if bytes_per_pixel == 1 or not by the caller. */
+/*
+ * For 8 or less bpp.
+ * Check if bytes_per_pixel == 1 or not by the caller.
+ */
 void
 x_display_fill_with(
 	int  x ,
@@ -1664,16 +2072,16 @@ x_display_fill_with(
 	u_int8_t  pixel
 	)
 {
-	u_char *  fb_orig ;
 	u_char *  fb ;
 	u_int  ppb ;
 	u_char *  buf ;
 	int  y_off ;
 
-	fb_orig = fb = get_fb( x , y) ;
+	fb = get_fb( x , y) ;
 
 	if( ( ppb = _display.pixels_per_byte) > 1)
 	{
+		u_char *  fb_orig ;
 		u_int  bpp ;
 		int  plane ;
 		u_char *  fb_end ;
@@ -1686,6 +2094,7 @@ x_display_fill_with(
 
 		bpp = 8 / ppb ;
 		plane = 0 ;
+		fb_orig = fb ;
 		fb_end = get_fb( x + width , y) ;
 
 	#ifndef ENABLE_DOUBLE_BUFFER
@@ -1829,12 +2238,14 @@ x_display_fill_with(
 				fb_end += _display.line_length ;
 			}
 
+		#ifndef  ENABLE_2_4_PPB
 			if( ++plane < _disp.depth)
 			{
-				fb = fb_orig + _display.plane_len * plane ;
+				fb = fb_orig + _display.plane_offset[plane] ;
 				fb_end = fb + (buf_end - buf) ;
 			}
 			else
+		#endif
 			{
 				break ;
 			}
@@ -1870,20 +2281,60 @@ x_display_copy_lines(
 	u_char *  dst ;
 	u_int  copy_len ;
 	u_int  count ;
+	int  num_of_planes ;
 	int  plane ;
 
 	/* XXX cheap implementation. */
 	restore_hidden_region() ;
 
+	if( rotate_display)
+	{
+		int  tmp ;
+
+		if( rotate_display > 0)
+		{
+			tmp = src_x ;
+			src_x = _disp.height - src_y - height ;
+			src_y = tmp ;
+
+			tmp = dst_x ;
+			dst_x = _disp.height - dst_y - height ;
+			dst_y = tmp ;
+		}
+		else
+		{
+			tmp = src_x ;
+			src_x = src_y ;
+			src_y = _disp.width - tmp - width ;
+
+			tmp = dst_x ;
+			dst_x = dst_y ;
+			dst_y = _disp.width - tmp - width ;
+		}
+
+		tmp = height ;
+		height = width ;
+		width = tmp ;
+	}
+
 	/* XXX could be different from FB_WIDTH_BYTES(display, dst_x, width) */
 	copy_len = FB_WIDTH_BYTES(&_display, src_x, width) ;
 
-	for( plane = 0 ; plane < _disp.depth ; plane++)
+	if( _display.pixels_per_byte == 8)
+	{
+		num_of_planes = _disp.depth ;
+	}
+	else
+	{
+		num_of_planes = 1 ;
+	}
+
+	for( plane = 0 ; plane < num_of_planes ; plane++)
 	{
 		if( src_y <= dst_y)
 		{
-			src = get_fb( src_x , src_y + height - 1) + _display.plane_len * plane ;
-			dst = get_fb( dst_x , dst_y + height - 1) + _display.plane_len * plane ;
+			src = get_fb( src_x , src_y + height - 1) + _display.plane_offset[plane] ;
+			dst = get_fb( dst_x , dst_y + height - 1) + _display.plane_offset[plane] ;
 
 		#ifdef  ENABLE_DOUBLE_BUFFER
 			if( _display.back_fb)
@@ -1942,8 +2393,8 @@ x_display_copy_lines(
 		}
 		else
 		{
-			src = get_fb( src_x , src_y) + _display.plane_len * plane ;
-			dst = get_fb( dst_x , dst_y) + _display.plane_len * plane ;
+			src = get_fb( src_x , src_y) + _display.plane_offset[plane] ;
+			dst = get_fb( dst_x , dst_y) + _display.plane_offset[plane] ;
 
 		#ifdef  ENABLE_DOUBLE_BUFFER
 			if( _display.back_fb)
@@ -1978,57 +2429,16 @@ x_display_copy_lines(
 }
 
 /* XXX for input method window */
-int
-x_display_check_visibility_of_im_window(void)
+void
+x_display_reset_input_method_window(void)
 {
-	static struct
-	{
-		int  saved ;
-		int  x ;
-		int  y ;
-		u_int  width ;
-		u_int  height ;
-
-	} im_region ;
-	int  redraw_im_win ;
-
-	redraw_im_win = 0 ;
-
+#if  0
 	if( _disp.num_of_roots == 2 && _disp.roots[1]->is_mapped)
+#endif
 	{
-		if( im_region.saved)
-		{
-			if( im_region.x == _disp.roots[1]->x &&
-			    im_region.y == _disp.roots[1]->y &&
-			    im_region.width == ACTUAL_WIDTH(_disp.roots[1]) &&
-			    im_region.height == ACTUAL_HEIGHT(_disp.roots[1]))
-			{
-				return  0 ;
-			}
-
-			expose_display( im_region.x , im_region.y ,
-				im_region.width , im_region.height) ;
-
-			redraw_im_win = 1 ;
-		}
-
-		im_region.saved = 1 ;
-		im_region.x = _disp.roots[1]->x ;
-		im_region.y = _disp.roots[1]->y ;
-		im_region.width = ACTUAL_WIDTH(_disp.roots[1]) ;
-		im_region.height = ACTUAL_HEIGHT(_disp.roots[1]) ;
+		check_visibility_of_im_window() ;
+		x_window_clear_margin_area( _disp.roots[1]) ;
 	}
-	else
-	{
-		if( im_region.saved)
-		{
-			expose_display( im_region.x , im_region.y ,
-				im_region.width , im_region.height) ;
-			im_region.saved = 0 ;
-		}
-	}
-
-	return  redraw_im_win ;
 }
 
 /* seek the closest color */
@@ -2040,7 +2450,8 @@ x_cmap_get_closest_color(
 	int  blue
 	)
 {
-	u_long  pixel ;
+	u_int  segment ;
+	u_int  offset ;
 	u_int  color ;
 	u_long  min = 0xffffff ;
 	u_long  diff ;
@@ -2051,17 +2462,42 @@ x_cmap_get_closest_color(
 		return  0 ;
 	}
 
-	/* 0xf8f8f8 ignores least significant 3bits */
-	if( (((pixel = (red << 16) | (green << 8) | blue) ^ _display.prev_pixel)
-	     & 0xfff8f8f8) == 0)
+#ifndef  COLOR_CACHE_MINIMUM
+	segment = 0 ;
+	/*
+	 * R        G        B
+	 * 11111111 11111111 11111111
+	 * ^^^^^    ^^^^^    ^^^^
+	 */
+	offset = ((red << 6) & 0x3e00) | ((green << 1) & 0x1f0) | ((blue >> 4) & 0xf) ;
+
+	if( _display.color_cache->flags[offset / 32] & (1 << (offset & 31)))
+#else
+	/*
+	 * R        G        B
+	 * 11111111 11111111 11111111
+	 * ^        ^        ^
+	 */
+	segment = ((red >> 5) & 0x4) | ((green >> 6) & 0x2) | ((blue >> 7) & 0x1) ;
+	/*
+	 * R        G        B
+	 * 11111111 11111111 11111111
+	 *  ^^^^     ^^^^     ^^^
+	 */
+	offset = ((red << 4) & 0x780) | (green & 0x78) | ((blue >> 4) & 0x7) ;
+
+	if( _display.color_cache->segments[offset] == (segment|0x80))
+#endif
 	{
-		*closest = _display.prev_closest_pixel ;
+		*closest = _display.color_cache->pixels[offset] ;
+	#ifdef  __DEBUG
+		kik_debug_printf( "CACHED PIXEL %x <= r%x g%x b%x segment %x offset %x\n" ,
+			*closest , red , green , blue , segment , offset) ;
+	#endif
 
 		return  1 ;
 	}
 
-	_display.prev_pixel = pixel ;
-
 	for( color = 0 ; color < CMAP_SIZE(_display.cmap) ; color++)
 	{
 	#ifdef  USE_GRF
@@ -2090,7 +2526,17 @@ x_cmap_get_closest_color(
 		}
 	}
 
-	_display.prev_closest_pixel = *closest ;
+#ifndef  COLOR_CACHE_MINIMUM
+	_display.color_cache->flags[offset / 32] |= (1 << (offset & 31)) ;
+#else
+	_display.color_cache->segments[offset] = (segment|0x80) ;
+#endif
+	_display.color_cache->pixels[offset] = *closest ;
+
+#ifdef  __DEBUG
+	kik_debug_printf( "NEW PIXEL %x <= r%x g%x b%x segment %x offset %x\n" ,
+		*closest , red , green , blue , segment , offset) ;
+#endif
 
 	return  1 ;
 }
