@@ -33,7 +33,7 @@ open_display(
 						dev : "/dev/ttyv0" ,
 					O_RDWR)) < 0)
 	{
-		kik_msg_printf( "Couldn't open %s.\n" , dev ? dev : "/dev/ttyv0") ;
+		kik_error_printf( "Couldn't open %s.\n" , dev ? dev : "/dev/ttyv0") ;
 
 		return  0 ;
 	}
@@ -47,25 +47,17 @@ open_display(
 	ioctl( _display.fb_fd , FBIO_ADPINFO , &vainfo) ;
 	ioctl( _display.fb_fd , FBIO_GETDISPSTART , &vstart) ;
 
-	if( ( _disp.depth = vinfo.vi_depth) < 8)
-	{
-	#if  0
-		/* 1/2/4 bpp is not supported. */
-		kik_msg_printf( "%d bpp is not supported.\n" , vinfo.vi_depth) ;
-
-		goto  error ;
-	#endif
-	}
-
 	if( ( _display.fb = mmap( NULL , (_display.smem_len = vainfo.va_window_size) ,
 				PROT_WRITE|PROT_READ , MAP_SHARED , _display.fb_fd , (off_t)0))
 				== MAP_FAILED)
 	{
-		kik_msg_printf( "Retry another mode of resolution and depth.\n") ;
+		kik_error_printf( "Retry another mode of resolution and depth.\n") ;
 
 		goto  error ;
 	}
 
+	_disp.depth = vinfo.vi_depth ;
+
 	if( ( _display.bytes_per_pixel = (_disp.depth + 7) / 8) == 3)
 	{
 		_display.bytes_per_pixel = 4 ;
@@ -73,7 +65,18 @@ open_display(
 
 	if( _disp.depth < 15)
 	{
-		if( _disp.depth < 8)
+		if( vainfo.va_mode == M_PC98_EGC640x400)
+		{
+			_display.pixels_per_byte = 8 ;
+			_disp.depth = 4 ;
+			_display.shift_0 = 7 ;
+			_display.mask = 1 ;
+			_display.plane_offset[0] = 0 ;		/* 0xA8000 */
+			_display.plane_offset[1] = 0x8000 ;	/* 0xB0000 */
+			_display.plane_offset[2] = 0x10000 ;	/* 0xB8000 */
+			_display.plane_offset[3] = 0x38000 ;	/* 0xE0000 */
+		}
+		else if( _disp.depth < 8)
 		{
 		#ifdef  ENABLE_2_4_PPB
 			_display.pixels_per_byte = 8 / _disp.depth ;
@@ -105,6 +108,8 @@ open_display(
 	if( _display.pixels_per_byte > 1 &&
 	    ! ( _display.back_fb = malloc( _display.smem_len)))
 	{
+		cmap_final() ;
+
 		goto  error ;
 	}
 #endif
@@ -351,8 +356,24 @@ receive_mouse_event(void)
 				xev.state = _display.key_state ;
 			}
 
-			xev.x = _mouse.x ;
-			xev.y = _mouse.y ;
+			if( rotate_display)
+			{
+				if( rotate_display > 0)
+				{
+					xev.x = _mouse.y ;
+					xev.y = _display.width - _mouse.x - 1 ;
+				}
+				else
+				{
+					xev.x = _display.height - _mouse.y - 1 ;
+					xev.y = _mouse.x ;
+				}
+			}
+			else
+			{
+				xev.x = _mouse.x ;
+				xev.y = _mouse.y ;
+			}
 
 		#ifdef  __DEBUG
 			kik_debug_printf( KIK_DEBUG_TAG
@@ -481,17 +502,6 @@ receive_key_event(void)
 				{
 					_display.key_state |= ShiftMask ;
 				}
-				else if( kcode == KEY_CAPSLOCK)
-				{
-					if( _display.key_state & ShiftMask)
-					{
-						_display.key_state &= ~ShiftMask ;
-					}
-					else
-					{
-						_display.key_state |= ShiftMask ;
-					}
-				}
 				else if( kcode == KEY_RIGHTCTRL ||
 					 kcode == KEY_LEFTCTRL)
 				{
