@@ -38,6 +38,7 @@ int  wskbd_repeat_1 = DEFAULT_KEY_REPEAT_1 ;
 int  wskbd_repeat_N = DEFAULT_KEY_REPEAT_N ;
 #endif
 static int  orig_console_mode = WSDISPLAYIO_MODE_EMUL ;	/* 0 */
+static int  wskbd_mode_switch = 0 ;
 
 
 /* --- static functions --- */
@@ -197,19 +198,45 @@ process_wskbd_event(
 		/* XXX */
 		ksym = XK_BackSpace ;
 	}
-	else if( _display.key_state & ShiftMask)
-	{
-		ksym = keymap.map[ev->value].group1[1] ;
-	}
 	else
 	{
-		ksym = keymap.map[ev->value].group1[0] ;
-	}
+		keysym_t *  group ;
 
-	if( KS_f1 <= ksym && ksym <= KS_f20)
-	{
-		/* KS_f1 => KS_F1 */
-		ksym += 0x40 ;
+		if( wskbd_mode_switch)
+		{
+			group = keymap.map[ev->value].group2 ;
+		}
+		else
+		{
+			group = keymap.map[ev->value].group1 ;
+		}
+
+		if( _display.key_state & ShiftMask)
+		{
+			ksym = group[1] ;
+		}
+		else
+		{
+			ksym = group[0] ;
+		}
+
+		if( KS_f1 <= ksym && ksym <= KS_f20)
+		{
+			/* KS_f1 => KS_F1 */
+			ksym += (KS_F1 - KS_f1) ;
+		}
+		else if( _display.lock_state & CLKED)
+		{
+			if( KS_a <= ksym && ksym <= KS_z)
+			{
+				ksym += (KS_A - KS_a) ;
+			}
+			else if( KS_agrave <= ksym && ksym <= KS_thorn &&
+				 ksym != KS_division)
+			{
+				ksym += (KS_Agrave - KS_agrave) ;
+			}
+		}
 	}
 
 	if( ev->type == WSCONS_EVENT_KEY_DOWN)
@@ -221,14 +248,7 @@ process_wskbd_event(
 		}
 		else if( ksym == KS_Caps_Lock)
 		{
-			if( _display.key_state & ShiftMask)
-			{
-				_display.key_state &= ~ShiftMask ;
-			}
-			else
-			{
-				_display.key_state |= ShiftMask ;
-			}
+			_display.lock_state ^= CLKED ;
 		}
 		else if( ksym == KS_Control_R ||
 			 ksym == KS_Control_L)
@@ -240,6 +260,10 @@ process_wskbd_event(
 		{
 			_display.key_state |= ModMask ;
 		}
+ 		else if( ksym == KS_Mode_switch)
+		{
+			wskbd_mode_switch = 1 ;
+		}
 		else if( ksym == KS_Num_Lock)
 		{
 			_display.lock_state ^= NLKED ;
@@ -280,6 +304,10 @@ process_wskbd_event(
 		{
 			_display.key_state &= ~ModMask ;
 		}
+ 		else if( ksym == KS_Mode_switch)
+		{
+			wskbd_mode_switch = 0 ;
+		}
 		else if( ev->value == prev_key_event.value)
 		{
 			prev_key_event.value = 0 ;
@@ -322,7 +350,7 @@ open_display(
 	if( ( _display.fb_fd = open( ( dev = getenv("FRAMEBUFFER")) ? dev : DEFAULT_FBDEV ,
 					O_RDWR)) < 0)
 	{
-		kik_msg_printf( "Couldn't open %s.\n" , dev ? dev : DEFAULT_FBDEV) ;
+		kik_error_printf( "Couldn't open %s.\n" , dev ? dev : DEFAULT_FBDEV) ;
 
 		return  0 ;
 	}
@@ -331,9 +359,7 @@ open_display(
 
 	ioctl( STDIN_FILENO , WSDISPLAYIO_GMODE , &orig_console_mode) ;
 
-#ifdef  __NetBSD__
-	mode = WSDISPLAYIO_MODE_MAPPED ;
-#else
+#ifdef  __OpenBSD__
 	{
 		struct wsdisplay_gfx_mode  gfx_mode ;
 
@@ -343,13 +369,12 @@ open_display(
 
 		if( ioctl( _display.fb_fd , WSDISPLAYIO_SETGFXMODE , &gfx_mode) == -1)
 		{
-			/* Always failed on OpenBSD/luna88k. */
-			kik_msg_printf( "Failed to set screen resolution (gfx mode).\n") ;
+			kik_error_printf( "Couldn't set screen resolution (gfx mode).\n") ;
 		}
 	}
+#endif
 
 	mode = WSDISPLAYIO_MODE_DUMBFB ;
-#endif
 
 	if( ioctl( STDIN_FILENO , WSDISPLAYIO_SMODE , &mode) == -1)
 	{
@@ -376,28 +401,37 @@ open_display(
 
 	_display.width = _disp.width = vinfo.width ;
 	_display.height = _disp.height = vinfo.height ;
+	_disp.depth = vinfo.depth ;
 
-	if( ( _disp.depth = vinfo.depth) < 8)
+#ifdef  WSDISPLAY_TYPE_LUNA
+	if( wstype == WSDISPLAY_TYPE_LUNA)
+	{
+		/* always 8 or less bpp */
+
+		if( _disp.depth > 8)
+		{
+			goto  error ;
+		}
+		else if( depth == 1 || depth == 4 || depth == 8)
+		{
+			_disp.depth = depth ;
+		}
+
+		_display.pixels_per_byte = 8 ;
+		_display.shift_0 = 7 ;
+		_display.mask = 1 ;
+	}
+	else
+#endif
+	if( _disp.depth < 8)
 	{
 	#ifdef  ENABLE_2_4_PPB
 		_display.pixels_per_byte = 8 / _disp.depth ;
 	#else
-	#ifdef  WSDISPLAY_TYPE_LUNA
-		if( wstype == WSDISPLAY_TYPE_LUNA)
-		{
-			if( ( depth == 1 || depth == 4))
-			{
-				_disp.depth = depth ;
-			}
-		}
-		else
-	#endif
-		{
-			/* XXX Forcibly set 1 bpp */
-			_disp.depth = 1 ;
-		}
-	#endif
+		/* XXX Forcibly set 1 bpp */
+		_disp.depth = 1 ;
 		_display.pixels_per_byte = 8 ;
+	#endif
 
 		_display.shift_0 = FB_SHIFT_0(_display.pixels_per_byte,_disp.depth) ;
 		_display.mask = FB_MASK(_display.pixels_per_byte) ;
@@ -431,10 +465,16 @@ open_display(
 	}
 
 #ifdef  WSDISPLAY_TYPE_LUNA
-	if( wstype == WSDISPLAY_TYPE_LUNA && _disp.depth == 4)
+	if( wstype == WSDISPLAY_TYPE_LUNA && (_disp.depth == 4 || _disp.depth == 8))
 	{
-		_display.smem_len = 0x40000 * 4 ;
-		_display.plane_len = 0x40000 ;
+		u_int  plane ;
+
+		_display.smem_len = 0x40000 * _disp.depth ;
+
+		for( plane = 0 ; plane < _disp.depth ; plane++)
+		{
+			_display.plane_offset[plane] = 0x40000 * plane ;
+		}
 	}
 	else
 #endif
@@ -446,7 +486,7 @@ open_display(
 				PROT_WRITE|PROT_READ , MAP_SHARED , _display.fb_fd , (off_t)0))
 				== MAP_FAILED)
 	{
-		kik_msg_printf( "Retry another mode of resolution and depth.\n") ;
+		kik_error_printf( "Retry another mode of resolution and depth.\n") ;
 
 		goto  error ;
 	}
@@ -482,6 +522,15 @@ open_display(
 
 					goto  error ;
 				}
+
+				if( ! ( _display.color_cache =
+						calloc( 1 , sizeof(*_display.color_cache))))
+				{
+					free( _display.cmap_orig) ;
+					free( _display.cmap) ;
+
+					goto  error ;
+				}
 			}
 
 			for( color = 0 ; color < num_of_colors ; color ++)
@@ -492,9 +541,6 @@ open_display(
 			}
 
 			ioctl( _display.fb_fd , FBIOPUTCMAP , _display.cmap) ;
-
-			_display.prev_pixel = 0xff000000 ;
-			_display.prev_closest_pixel = 0xff000000 ;
 		}
 		else if( ! cmap_init())
 		{
@@ -783,8 +829,24 @@ receive_mouse_event(void)
 			}
 
 			xev.time = ev.time.tv_sec * 1000 + ev.time.tv_nsec / 1000000 ;
-			xev.x = _mouse.x ;
-			xev.y = _mouse.y ;
+			if( rotate_display)
+			{
+				if( rotate_display > 0)
+				{
+					xev.x = _mouse.y ;
+					xev.y = _display.width - _mouse.x - 1 ;
+				}
+				else
+				{
+					xev.x = _display.height - _mouse.y - 1 ;
+					xev.y = _mouse.x ;
+				}
+			}
+			else
+			{
+				xev.x = _mouse.x ;
+				xev.y = _mouse.y ;
+			}
 			xev.state = _display.key_state ;
 
 		#ifdef  __DEBUG
@@ -871,8 +933,24 @@ receive_mouse_event(void)
 			update_mouse_cursor_state() ;
 
 			xev.type = MotionNotify ;
-			xev.x = _mouse.x ;
-			xev.y = _mouse.y ;
+			if( rotate_display)
+			{
+				if( rotate_display > 0)
+				{
+					xev.x = _mouse.y ;
+					xev.y = _display.width - _mouse.x - 1 ;
+				}
+				else
+				{
+					xev.x = _display.height - _mouse.y - 1 ;
+					xev.y = _mouse.x ;
+				}
+			}
+			else
+			{
+				xev.x = _mouse.x ;
+				xev.y = _mouse.y ;
+			}
 			xev.time = ev.time.tv_sec * 1000 + ev.time.tv_nsec / 1000000 ;
 			xev.state = _mouse.button_state | _display.key_state ;
 
