@@ -677,55 +677,6 @@ clear_margin_area(
 	return  1 ;
 }
 
-static int
-window_move(
-	x_window_t *  win ,
-	int  x ,
-	int  y ,
-	int  expose	/* 1=normal, 0=no */
-	)
-{
-	int  prev_x ;
-	int  prev_y ;
-
-	prev_x = win->x ;
-	prev_y = win->y ;
-
-	win->x = x ;
-	win->y = y ;
-
-	if( expose)
-	{
-		/*
-		 * x_display_check_visibility_of_im_window() must be called after
-		 * win->x and win->y is set because window_exposed event which is
-		 * called in expose_window() can call x_window_move() recursively.
-		 */
-		x_display_check_visibility_of_im_window() ;
-	}
-
-	clear_margin_area( win) ;
-
-	if( expose && win->window_exposed)
-	{
-		(*win->window_exposed)( win , 0 , 0 , win->width , win->height) ;
-	}
-#if  0
-	else
-	{
-		x_window_clear_all( win) ;
-	}
-#endif
-
-	/* XXX */
-	if( win->parent)
-	{
-		clear_margin_area( win->parent) ;
-	}
-
-	return  1 ;
-}
-
 static int
 fix_rl_boundary(
 	x_window_t *  win ,
@@ -1264,18 +1215,52 @@ x_window_move(
 	int  y
 	)
 {
-	return  window_move( win , x , y , 1) ;
-}
+	int  prev_x ;
+	int  prev_y ;
 
-/* XXX for x_im_candidate_screen */
-int
-x_window_move_no_expose(
-	x_window_t *  win ,
-	int  x ,
-	int  y
-	)
-{
-	return  window_move( win , x , y , 0) ;
+	prev_x = win->x ;
+	prev_y = win->y ;
+
+	win->x = x ;
+	win->y = y ;
+
+	/*
+	 * XXX
+	 * Check if win is input method window or not, because x_window_move()
+	 * can fall into the following infinite loop on framebuffer.
+	 * 1) x_im_stat_screen::draw_screen() ->
+	 *    x_window_move() ->
+	 *    x_im_stat_screen::window_exposed() ->
+	 *    x_im_stat_screen::draw_screen()
+	 * 2) x_im_candidate_screen::draw_screen() ->
+	 *    x_im_candidate_screen::resize() ->
+	 *    x_window_move() ->
+	 *    x_im_candidate_screen::window_exposed() ->
+	 *    x_im_candidate_screen::draw_screen()
+	 */
+	if( ( win->disp->num_of_roots == 1 || win != win->disp->roots[1]))
+	{
+		clear_margin_area( win) ;
+
+		if( win->window_exposed)
+		{
+			(*win->window_exposed)( win , 0 , 0 , win->width , win->height) ;
+		}
+	#if  0
+		else
+		{
+			x_window_clear_all( win) ;
+		}
+	#endif
+
+		/* XXX */
+		if( win->parent)
+		{
+			clear_margin_area( win->parent) ;
+		}
+	}
+
+	return  1 ;
 }
 
 int
@@ -1641,7 +1626,7 @@ x_window_get_str(
 				case  XK_KP_Left:
 					ch = '4' ;
 					break ;
-				case  XK_KP_5:
+				case  XK_KP_Begin:
 					ch = '5' ;
 					break ;
 				case  XK_KP_Right:
@@ -2038,13 +2023,23 @@ x_window_utf_selection_request(
 }
 
 int
-x_window_send_selection(
+x_window_send_picture_selection(
+	x_window_t *  win ,
+	Pixmap  pixmap ,
+	u_int  width ,
+	u_int  height
+	)
+{
+	return  0 ;
+}
+
+int
+x_window_send_text_selection(
 	x_window_t *  win ,
 	XSelectionRequestEvent *  req_ev ,
 	u_char *  sel_data ,
 	size_t  sel_len ,
-	Atom  sel_type ,
-	int sel_format
+	Atom  sel_type
 	)
 {
 	return  1 ;
