@@ -227,7 +227,8 @@ parse_fc_font_name(
 					}
 				}
 
-				if( *p != '0') /* In case of "DevLys 010" font family. */
+				if( *p != '0' || /* In case of "DevLys 010" font family. */
+				    *(p + 1) == '\0')	/* "MS Gothic 0" => "MS Gothic" + "0" */
 				{
 					char *  end ;
 					double  size ;
@@ -238,7 +239,10 @@ parse_fc_font_name(
 						/* p has no more parameters. */
 
 						*orig_p = '\0' ;
-						*font_size = size ;
+						if( size > 0)
+						{
+							*font_size = size ;
+						}
 
 						break ;
 					}
@@ -268,16 +272,17 @@ get_fc_col_width(
 	u_int  letter_space
 	)
 {
-	if( percent > 0)
+	if( percent == 0)
 	{
-		return  DIVIDE_ROUNDING(fontsize_d * font->cols * percent , 100 * 2) +
-			letter_space ;
+		if( letter_space == 0 || font->is_var_col_width)
+		{
+			return  0 ;
+		}
+
+		percent = 100 ;
 	}
-	else if( letter_space == 0 || font->is_var_col_width)
-	{
-		return  0 ;
-	}
-	else if( strcmp( fc_size_type , FC_SIZE) == 0)
+
+	if( strcmp( fc_size_type , FC_SIZE) == 0)
 	{
 		double  dpi ;
 
@@ -296,11 +301,13 @@ get_fc_col_width(
 			dpi = (widthpix * 254) / (widthmm * 10) ;
 		}
 
-		return  DIVIDE_ROUNDINGUP(dpi * fontsize_d * font->cols , 72 * 2) + letter_space ;
+		return  DIVIDE_ROUNDINGUP(dpi * fontsize_d * font->cols * percent , 72 * 100 * 2) +
+			letter_space ;
 	}
 	else
 	{
-		return  DIVIDE_ROUNDINGUP(fontsize_d * font->cols , 2) + letter_space ;
+		return  DIVIDE_ROUNDINGUP(fontsize_d * font->cols * percent , 100 * 2) +
+			letter_space ;
 	}
 }
 
@@ -335,14 +342,15 @@ fc_pattern_create(
 	{
 		FcPatternAddInteger( pattern , FC_SLANT , slant) ;
 	}
-	FcPatternAddInteger( pattern , FC_SPACING , ch_width > 0 ? FC_MONO : FC_PROPORTIONAL) ;
 #ifdef  USE_TYPE_XFT
 	if( ch_width > 0)
 	{
+		FcPatternAddInteger( pattern , FC_SPACING , FC_CHARCELL) ;
 		/* XXX FC_CHAR_WIDTH doesn't make effect in cairo ... */
 		FcPatternAddInteger( pattern , FC_CHAR_WIDTH , ch_width) ;
 	}
 #endif
+
 	if( aa_opt)
 	{
 		FcPatternAddBool( pattern , FC_ANTIALIAS , aa_opt == 1 ? True : False) ;
@@ -354,6 +362,7 @@ fc_pattern_create(
 #ifdef  USE_TYPE_XFT
 	if( encoding)
 	{
+		/* no meaning on xft2 */
 		FcPatternAddString( pattern , XFT_ENCODING , encoding) ;
 	}
 #endif
@@ -390,6 +399,12 @@ xft_font_open(
 		return  NULL ;
 	}
 
+	if( IS_ISCII(FONT_CS(font->id)))
+	{
+		/* no meaning on xft2 */
+		FcPatternAddString( pattern , XFT_ENCODING , "apple-roman") ;
+	}
+
 	match = XftFontMatch( font->display , DefaultScreen( font->display) , pattern ,
 				&result) ;
 	FcPatternDestroy( pattern) ;
@@ -408,7 +423,36 @@ xft_font_open(
 
 		return  NULL ;
 	}
-	
+
+#if  1
+	if( IS_ISCII(FONT_CS(font->id)))
+	{
+		FT_Face  face ;
+		int  count ;
+
+		face = XftLockFace( xfont) ;
+
+		for( count = 0 ; count < face->num_charmaps ; count++)
+		{
+		#ifdef  DEBUG
+			kik_debug_printf( KIK_DEBUG_TAG " ISCII font encoding %c%c%c%c\n" ,
+				((face->charmaps[count]->encoding) >> 24) & 0xff ,
+				((face->charmaps[count]->encoding) >> 16) & 0xff ,
+				((face->charmaps[count]->encoding) >> 8) & 0xff ,
+				(face->charmaps[count]->encoding & 0xff)) ;
+		#endif
+
+			if( face->charmaps[count]->encoding == FT_ENCODING_APPLE_ROMAN)
+			{
+				FT_Set_Charmap( face , face->charmaps[count]) ;
+				break ;
+			}
+		}
+
+		XftUnlockFace( xfont) ;
+	}
+#endif
+
 	return  xfont ;
 }
 
@@ -507,8 +551,7 @@ cairo_font_open(
 	 */
 	pixel_size2 = DIVIDE_ROUNDINGUP(pixel_size,2.0) * 2 ;
 
-	cairo_matrix_init_identity( &font_matrix) ;
-	cairo_matrix_scale( &font_matrix , pixel_size2 , pixel_size2) ;
+	cairo_matrix_init_scale( &font_matrix , pixel_size2 , pixel_size2) ;
 	cairo_get_matrix( cairo , &ctm) ;
 
 	xfont = cairo_scaled_font_create( font_face , &font_matrix , &ctm , options) ;
@@ -526,6 +569,34 @@ cairo_font_open(
 		return  NULL ;
 	}
 
+#if  1
+	if( IS_ISCII(FONT_CS(font->id)))
+	{
+		FT_Face  face ;
+		int  count ;
+
+		face = cairo_ft_scaled_font_lock_face( xfont) ;
+
+		for( count = 0 ; count < face->num_charmaps ; count++)
+		{
+		#ifdef  DEBUG
+			kik_debug_printf( KIK_DEBUG_TAG " ISCII font encoding %c%c%c%c\n" ,
+				((face->charmaps[count]->encoding) >> 24) & 0xff ,
+				((face->charmaps[count]->encoding) >> 16) & 0xff ,
+				((face->charmaps[count]->encoding) >> 8) & 0xff ,
+				(face->charmaps[count]->encoding & 0xff)) ;
+		#endif
+
+			if( face->charmaps[count]->encoding == FT_ENCODING_APPLE_ROMAN)
+			{
+				FT_Set_Charmap( face , face->charmaps[count]) ;
+			}
+		}
+
+		cairo_ft_scaled_font_unlock_face( xfont) ;
+	}
+#endif
+
 	return  xfont ;
 }
 
@@ -566,6 +637,7 @@ ft_font_open(
 
 u_int  xft_calculate_char_width( x_font_t *  font , u_int32_t  ch) ;
 u_int  cairo_calculate_char_width( x_font_t *  font , u_int32_t  ch) ;
+int  xft_unset_font( x_font_t *  font) ;
 
 static int
 fc_set_font(
@@ -787,7 +859,7 @@ fc_set_font(
 
 font_found:
 
-	font->is_proportional = font->is_var_col_width ;
+	font->is_proportional = 0 ;
 
 	if( use_xft)
 	{
@@ -816,25 +888,51 @@ font_found:
 		if( ch_width == 0)
 		{
 			/*
-			 * Proportional (font->is_var_col_width is true)
-			 * letter_space is ignored in variable column width mode.
+			 * font->is_var_col_width is true or letter_space == 0.
+			 * (see get_fc_col_width())
 			 */
+
 			font->width = xft_calculate_char_width( font , 'W') ;
+
+			if( font->width != xft_calculate_char_width( font , 'l'))
+			{
+				/* Regard it as proportional. */
+
+				if( font->is_var_col_width)
+				{
+					font->is_proportional = 1 ;
+				}
+				else
+				{
+					u_int  new_width ;
+
+					new_width = xft_calculate_char_width( font , 'M') ;
+					if( font->is_vertical)
+					{
+						new_width *= 2 ;
+					}
+
+					xft_unset_font( font) ;
+
+					/* reloading it as mono spacing. */
+					return  fc_set_font( font , fontname , fontsize ,
+							new_width , letter_space , aa_opt ,
+							use_xft) ;
+				}
+			}
 		}
 		else
 		{
+			/* Always mono space */
 			font->width = ch_width ;
-
-			if( font->is_vertical && font->cols == 1)
-			{
-				font->x_off += ch_width / 4 ;	/* Centering */
-			}
-			else
-			{
-				font->x_off += letter_space * font->cols / 2 ;
-			}
 		}
 
+		font->x_off += (letter_space * font->cols / 2) ;
+
+		if( font->is_vertical && font->cols == 1)
+		{
+			font->x_off += (font->width / 4) ;	/* Centering */
+		}
 	#endif	/* USE_TYPE_XFT */
 	}
 	else
@@ -873,45 +971,63 @@ font_found:
 				font->width *= 2 ;
 				font->x_off = font->width / 4 ;	/* Centering */
 			}
-		}
-
-		/*
-		 * Set letter_space here because cairo_font_open() ignores it.
-		 * (FC_CHAR_WIDTH doesn't make effect in cairo.)
-		 * Then, column width for vertical mode should be set again here.
-		 */
-
-		/* letter_space is ignored in variable column width mode. */
-		if( ! font->is_var_col_width && letter_space > 0)
-		{
-			font->is_proportional = 1 ;
-			font->width += (letter_space * font->cols) ;
-			font->x_off += (letter_space * font->cols / 2) ;  /* Centering */
-		}
-
-		if( col_width > 0 /* is not usascii */ && ! font->is_proportional &&
-		    ch_width != font->width)
-		{
-			kik_msg_printf( "Font(id %x) width(%d) is not matched with "
-				"standard width(%d).\n" , font->id , font->width , ch_width) ;
-
-			/*
-			 * XXX
-			 * Surrounded by #if 0 ... #endif,
-			 * because the case like ch_width = 12 and
-			 * extents.max_x_advance = 12.28(dealt as 13 though should be dealt as 12)
-			 * happens.
-			 */
-		#if  0
-			font->is_proportional = 1 ;
-
-			if( font->width < ch_width)
+			else if( font->width != cairo_calculate_char_width( font , 'l'))
 			{
-				font->x_off = (ch_width - font->width) / 2 ;
-			}
-		#endif
+				if( ! font->is_var_col_width)
+				{
+					font->width = cairo_calculate_char_width( font , 'M') ;
+				}
 
-			font->width = ch_width ;
+				/* Regard it as proportional. */
+				font->is_proportional = 1 ;
+			}
+		}
+
+		if( ! font->is_var_col_width)
+		{
+			/*
+			 * Set letter_space here because cairo_font_open() ignores it.
+			 * (FC_CHAR_WIDTH doesn't make effect in cairo.)
+			 * Note that letter_space is ignored in variable column width mode.
+			 */
+			if( letter_space > 0)
+			{
+				font->is_proportional = 1 ;
+
+				if( font->is_vertical)
+				{
+					letter_space *= 2 ;
+				}
+				else
+				{
+					letter_space *= font->cols ;
+				}
+
+				font->width += letter_space ;
+				font->x_off += (letter_space / 2) ; /* Centering */
+			}
+
+			if( ch_width > 0 && ch_width != font->width)
+			{
+				kik_msg_printf( "Font(id %x) width(%d) is not matched with "
+					"standard width(%d).\n" ,
+					font->id , font->width , ch_width) ;
+
+				/*
+				 * XXX
+				 * Note that ch_width = 12 and extents.max_x_advance = 12.28
+				 * (dealt as 13 though should be dealt as 12) may happen.
+				 */
+
+				font->is_proportional = 1 ;
+
+				if( font->width < ch_width)
+				{
+					font->x_off += (ch_width - font->width) / 2 ;
+				}
+
+				font->width = ch_width ;
+			}
 		}
 	#endif	/* USE_TYPE_CAIRO */
 	}
