@@ -17,7 +17,7 @@
 #include  <kiklib/kik_dialog.h>
 
 #include  "../x_window.h"
-
+#include  "../x_picture.h"
 #include  "x_gdiobj_pool.h"
 
 
@@ -196,6 +196,8 @@ x_display_close_all(void)
 		return  0 ;
 	}
 
+	x_picture_display_closed( _disp.display) ;
+
 	x_gc_delete( _disp.gc) ;
 
 	for( count = 0 ; count < _disp.num_of_roots ; count ++)
