@@ -11,6 +11,7 @@
 
 #include  <kiklib/kik_util.h>	/* DIGIT_STR_LEN */
 #include  <kiklib/kik_debug.h>
+#include  <kiklib/kik_mem.h>
 #if  defined(__CYGWIN__) || defined(__MSYS__)
 #include  <kiklib/kik_path.h>	/* cygwin_conv_to_win32_path */
 #endif
@@ -18,6 +19,9 @@
 
 /* --- static functions --- */
 
+#define  CARD_HEAD_SIZE  0
+#include  "../../common/c_sixel.c"
+
 static void
 value_table_refresh(
 	u_char *  value_table ,		/* 256 bytes */
@@ -105,7 +109,7 @@ adjust_pixmap(
 
 static int
 load_file(
-	char *  path ,
+	char *  path ,	/* must be UTF-8 */
 	u_int *  width ,
 	u_int *  height ,
 	x_picture_modifier_t *  pic_mod ,
@@ -114,6 +118,8 @@ load_file(
 	)
 {
 	char *  cmd_line ;
+	WCHAR *  w_cmd_line ;
+	int  num ;
 	SECURITY_ATTRIBUTES  sa ;
 	HANDLE  output_write ;
 	HANDLE  output_read ;
@@ -126,6 +132,13 @@ load_file(
 	HDC  hdc ;
 	BYTE *  image ;
 
+	if( strstr( path , ".six") && *width == 0 && *height == 0 &&
+	    /* XXX fopen() in load_sixel_from_file() on win32api doesn't support UTF-8. */
+	    ( image = (u_int32_t*)load_sixel_from_file( path , width , height)))
+	{
+		goto  loaded ;
+	}
+
 #define  CMD_LINE_FMT  "mlimgloader.exe 0 %u %u \"%s\" -c"
 
 	if( ! ( cmd_line = alloca( sizeof( CMD_LINE_FMT) + DIGIT_STR_LEN(int) * 2 +
@@ -136,6 +149,16 @@ load_file(
 
 	sprintf( cmd_line , CMD_LINE_FMT , *width , *height , path) ;
 
+	/* Assume that path is UTF-8 */
+	if( ( num = MultiByteToWideChar( CP_UTF8 , 0 , cmd_line , strlen(cmd_line) + 1 ,
+				NULL , 0)) == 0 ||
+	    ! ( w_cmd_line = alloca( sizeof(WCHAR) * num)))
+	{
+		return  0 ;
+	}
+
+	MultiByteToWideChar( CP_UTF8 , 0 , cmd_line , strlen(cmd_line) + 1 , w_cmd_line , num) ;
+
 	/* Set up the security attributes struct. */
 	sa.nLength = sizeof(SECURITY_ATTRIBUTES);
 	sa.lpSecurityDescriptor = NULL;
@@ -158,7 +181,7 @@ load_file(
 	si.hStdInput = GetStdHandle(STD_INPUT_HANDLE) ;
 	si.hStdError = GetStdHandle(STD_ERROR_HANDLE) ;
 
-	if( ! CreateProcess( NULL , cmd_line ,
+	if( ! CreateProcessW( NULL , w_cmd_line ,
 		NULL , NULL , TRUE , CREATE_NO_WINDOW , NULL , NULL , &si , &pi))
 	{
 	#if  defined(__CYGWIN__) || defined(__MSYS__)
@@ -170,22 +193,40 @@ load_file(
 		char *  new_cmd_line ;
 
 		cygwin_conv_to_win32_path( BINDIR , bindir) ;
-		if( ! ( new_cmd_line = alloca( strlen(bindir) + 1 + strlen(cmd_line) + 1)) ||
-		    sprintf( new_cmd_line , "%s\\%s" , bindir , cmd_line) < 0 ||
-		    ! CreateProcess( NULL , new_cmd_line ,
-			NULL , NULL , TRUE , CREATE_NO_WINDOW , NULL , NULL , &si , &pi))
-	#endif
+		if( ! ( new_cmd_line = alloca( strlen(bindir) + 1 + strlen(cmd_line) + 1)))
 		{
-		#ifdef  DEBUG
-			kik_debug_printf( KIK_DEBUG_TAG " CreateProcess() failed.\n") ;
-		#endif
+			sprintf( new_cmd_line , "%s\\%s" , bindir , cmd_line) ;
 
-			CloseHandle( output_write) ;
+			num = MultiByteToWideChar( CP_UTF8 , 0 ,
+					new_cmd_line , strlen(new_cmd_line) + 1 ,
+					NULL , 0) ;
+			if( ( w_cmd_line = alloca( sizeof(WCHAR) * num)))
+			{
+				MultiByteToWideChar( CP_UTF8 , 0 ,
+					new_cmd_line , strlen(new_cmd_line) + 1 ,
+					w_cmd_line , num) ;
 
-			goto  error ;
+				if( CreateProcessW( NULL , w_cmd_line ,
+					NULL , NULL , TRUE , CREATE_NO_WINDOW ,
+					NULL , NULL , &si , &pi))
+				{
+					goto  executed ;
+				}
+			}
 		}
+	#endif
+
+	#ifdef  DEBUG
+		kik_debug_printf( KIK_DEBUG_TAG
+			" CreateProcess() failed.\n") ;
+	#endif
+
+		CloseHandle( output_write) ;
+
+		goto  error ;
 	}
 
+executed:
 	CloseHandle( output_write) ;
 	CloseHandle( pi.hProcess) ;
 	CloseHandle( pi.hThread) ;
@@ -231,6 +272,7 @@ load_file(
 
 	CloseHandle( output_read) ;
 
+loaded:
 	adjust_pixmap( image , *width , *height , pic_mod) ;
 
 	if( hbmp_mask)
