@@ -579,6 +579,29 @@ text_out(
 		/* TextOutW is supported in windows 9x. */
 		return  TextOutW( gc , x , y , (WCHAR*)str , len / 2) ;
 	}
+	else if( cs == ISCII_BENGALI || cs == ISCII_ASSAMESE)
+	{
+		u_int  count ;
+		u_int16_t *  str2 ;
+
+		str2 = alloca( len * 2) ;
+
+		/* char code -> glyph index in BN-TTDurga and AS-TTDurga fonts. */
+		for( count = 0 ; count < len ; count++)
+		{
+			if( str[count] <= 0x7e)
+			{
+				str2[count] = str[count] - 0x1d ;
+			}
+			else
+			{
+				str2[count] = str[count] - 0x1e ;
+			}
+		}
+
+		/* ExtTextOutA doesn't work correctly. */
+		return  ExtTextOutW( gc , x , y , ETO_GLYPH_INDEX , NULL , str2 , len , NULL) ;
+	}
 	else
 	{
 		return  TextOutA( gc , x , y , str , len) ;
@@ -2207,7 +2230,7 @@ x_window_receive_event(
 
 			bev.time = GetMessageTime() ;
 
-			bev.state = 0 ;
+			bev.state = get_key_state() ;
 		#if  0
 			if( event->wparam & MK_LBUTTON)
 			{
@@ -2294,6 +2317,23 @@ x_window_receive_event(
 				}
 			}
 
+			/* XXX grandchild windows aren't regarded for now. */
+			for( count = 0 ; count < win->num_of_children ; count++)
+			{
+				if( bev.x <= win->children[count]->x &&
+				    bev.y <= win->children[count]->y &&
+				    win->children[count]->x + ACTUAL_WIDTH(win->children[count])
+				          <= bev.x + 1 &&
+				    win->children[count]->y + ACTUAL_HEIGHT(win->children[count])
+				          <= bev.y + 1)
+				{
+					win = win->children[count] ;
+					bev.x -= win->x ;
+					bev.y -= win->y ;
+					break ;
+				}
+			}
+
 			if( event->msg == WM_MOUSEWHEEL || event->msg == WM_LBUTTONDOWN ||
 				event->msg == WM_RBUTTONDOWN || event->msg == WM_MBUTTONDOWN)
 			{
@@ -2353,7 +2393,7 @@ x_window_receive_event(
 			mev.x = LOWORD(event->lparam) - win->hmargin ;
 			mev.y = HIWORD(event->lparam) - win->vmargin ;
 
-			mev.state = 0 ;
+			mev.state = get_key_state() ;
 
 			if( event->wparam & MK_LBUTTON)
 			{
@@ -3096,13 +3136,60 @@ x_window_utf_selection_request(
 }
 
 int
-x_window_send_selection(
+x_window_send_picture_selection(
+	x_window_t *  win ,
+	Pixmap  pixmap ,
+	u_int  width ,
+	u_int  height
+	)
+{
+	HBITMAP  hbmp ;
+	HGDIOBJ old ;
+	HDC  hdc ;
+
+	if( MessageBox( win->my_window , "Set this picture to the clipboard." ,
+		"" , MB_OKCANCEL) != IDOK)
+	{
+		return  0 ;
+	}
+
+	hbmp = CreateCompatibleBitmap( pixmap , width , height) ;
+	hdc = CreateCompatibleDC( pixmap) ;
+	old = SelectObject( hdc , hbmp) ;
+	BitBlt( hdc , 0 , 0 , width , height , pixmap , 0 , 0 , SRCCOPY) ;
+	SelectObject( hdc , old) ;
+	DeleteDC( hdc) ;
+
+	if( ( ! win->is_sel_owner &&
+	      ! x_display_own_selection( win->disp , win)) ||
+	    OpenClipboard( win->my_window) == FALSE)
+	{
+		return  0 ;
+	}
+
+	/*
+	 * If win->is_sel_owner is already 1, win->is_sel_owner++ prevents
+	 * WM_DESTROYCLIPBOARD by EmtpyClipboard() from calling
+	 * x_display_clear_selection().
+	 */
+	win->is_sel_owner ++ ;
+
+	EmptyClipboard() ;
+	SetClipboardData( CF_BITMAP , hbmp) ;
+	CloseClipboard() ;
+
+	DeleteObject( hbmp) ;
+
+	return  1 ;
+}
+
+int
+x_window_send_text_selection(
 	x_window_t *  win ,
 	XSelectionRequestEvent *  req_ev ,
 	u_char *  sel_data ,
 	size_t  sel_len ,
-	Atom  sel_type ,
-	int sel_format
+	Atom  sel_type
 	)
 {
 	HGLOBAL  hmem ;
