@@ -8,8 +8,10 @@
 
 #ifndef  NO_IMAGE
 #include  "x_picture.h"
-#endif
 
+#define  INLINEPIC_ID(glyph)  (((glyph) >> PICTURE_POS_BITS) & (MAX_INLINE_PICTURES - 1))
+#define  INLINEPIC_POS(glyph) ((glyph) & ((1 << PICTURE_POS_BITS) - 1))
+#endif
 
 #if  0
 #define  PERF_DEBUG
@@ -18,35 +20,86 @@
 
 /* --- static functions --- */
 
-static int
-adjust_bd_ul_color(
-	x_color_manager_t *  color_man ,
-	ml_color_t  fg_color ,
-	ml_color_t  bg_color ,
-	int  is_bold ,
-	int  is_underlined
+static void
+draw_line(
+	x_window_t *  window ,
+	x_color_t *  color ,
+	int  is_vertical ,
+	int  flag ,		/* 0 == underline 1 == double underline, 2 == crossed out */
+	int  x ,
+	int  y ,
+	u_int  width ,
+	u_int  height ,
+	u_int  ascent ,
+	u_int  top_margin
 	)
 {
-	if( is_bold)
+	u_int  w ;
+	u_int  h ;
+	int  x2 ;
+	int  y2 ;
+
+	if( is_vertical)
 	{
-		/* If bg_color == ML_FG_COLOR, it seems to be reversed. */
-		if( ( fg_color == ML_FG_COLOR || bg_color == ML_FG_COLOR) &&
-		    x_color_manager_adjust_bd_color( color_man))
+		w = 1 ;
+		h = height ;
+
+		if( flag == 1)	/* double */
 		{
-			return  1 ;
+			x2 = x + 2 ;
+			y2 = y ;
+		}
+		else
+		{
+			w += ((ascent - top_margin) / 16) ;
+
+			if( flag == 2)	/* crossed out */
+			{
+				x += ((width - 1) / 2) ;
+			}
+		}
+	}
+	else
+	{
+		w = width ;
+		h = 1 ;
+
+		if( flag == 1)	/* double */
+		{
+			x2 = x ;
+
+			if( ascent + 2 >= height)
+			{
+				y2 = y + height - 1 ;
+				y = y2 - 2 ;
+			}
+			else
+			{
+				y += ascent ;
+				y2 = y + 2 ;
+			}
+		}
+		else
+		{
+			h += ((ascent - top_margin) / 16) ;
+
+			if( flag == 2)	/* crossed out */
+			{
+				y += ((height + 1) / 2) ;
+			}
+			else
+			{
+				y += ascent ;
+			}
 		}
 	}
 
-	if( is_underlined)
-	{
-		if( ( fg_color == ML_FG_COLOR || bg_color == ML_FG_COLOR) &&
-		    x_color_manager_adjust_ul_color( color_man))
-		{
-			return  2 ;
-		}
-	}
+	x_window_fill_with( window , color , x , y , w , h) ;
 
-	return  0 ;
+	if( flag == 1)	/* double */
+	{
+		x_window_fill_with( window , color , x2 , y2 , w , h) ;
+	}
 }
 
 #ifndef  NO_IMAGE
@@ -129,7 +182,7 @@ draw_picture(
 			}
 		}
 
-		if( need_clear)
+		if( need_clear > 0)
 		{
 			x_window_clear( window , dst_x , dst_y , dst_width , line_height) ;
 		}
@@ -172,6 +225,12 @@ draw_picture(
 			src_height = line_height ;
 		}
 
+		if( strstr( cur_pic->file_path , "mlterm/animx") && cur_pic->next_frame >= 0)
+		{
+			/* Don't clear if cur_pic is 2nd or later GIF Animation frame. */
+			need_clear = -1 ;
+		}
+
 		if( (src_width = w) < ch_width && ! need_clear)
 		{
 			x_window_clear( window , dst_x , dst_y , ch_width , line_height) ;
@@ -328,7 +387,8 @@ get_state(
 	if( comb_chars && ml_char_cs( comb_chars) == PICTURE_CHARSET)
 	{
 		*draw_alone = 0 ;	/* forcibly set 0 regardless of xfont. */
-		*pic_glyph = ml_char_code( comb_chars) ;
+		*pic_glyph = ml_char_code( comb_chars) |
+		             (ml_char_picture_id( comb_chars) << PICTURE_POS_BITS) ;
 		*drcs_glyph = NULL ;
 
 		return  4 ;
@@ -402,7 +462,8 @@ fc_draw_combining_chars(
 			c = ch_code ;
 			x_window_draw_decsp_string( window , xfont , xcolor , x , y , &c , 1) ;
 		}
-		else if( ch_cs == US_ASCII || ch_cs == ISO8859_1_R || IS_ISCII(ch_cs))
+		/* ISCII characters never have combined ones. */
+		else if( ch_cs == US_ASCII || ch_cs == ISO8859_1_R /* || IS_ISCII(ch_cs) */)
 		{
 			u_char  c ;
 
@@ -458,7 +519,8 @@ fc_draw_str(
 	ml_font_t  font ;
 	ml_color_t  fg_color ;
 	ml_color_t  bg_color ;
-	int  is_underlined ;
+	int  underline_style ;
+	int  is_crossed_out ;
 	ml_char_t *  comb_chars ;
 	u_int  comb_size ;
 	int  draw_alone ;
@@ -472,7 +534,8 @@ fc_draw_str(
 	ml_font_t  next_font ;
 	ml_color_t  next_fg_color ;
 	ml_color_t  next_bg_color ;
-	int  next_is_underlined ;
+	int  next_underline_style ;
+	int  next_is_crossed_out ;
 	ml_char_t *  next_comb_chars ;
 	u_int  next_comb_size ;
 	u_int  next_ch_width ;
@@ -533,7 +596,8 @@ fc_draw_str(
 	fg_color = ml_char_fg_color( &chars[count]) ;
 	bg_color = ml_char_bg_color( &chars[count]) ;
 
-	is_underlined = ml_char_is_underlined( &chars[count]) ;
+	underline_style = ml_char_underline_style( &chars[count]) ;
+	is_crossed_out = ml_char_is_crossed_out( &chars[count]) ;
 
 	if( ! ( str8 = str32 = pic_glyphs = drcs_glyphs =
 		alloca( K_MAX(sizeof(*str8),K_MAX(sizeof(*str32),
@@ -600,7 +664,8 @@ fc_draw_str(
 			ch_cs = FONT_CS(next_font) ;
 			next_fg_color = ml_char_fg_color( &chars[count]) ;
 			next_bg_color = ml_char_bg_color( &chars[count]) ;
-			next_is_underlined = ml_char_is_underlined( &chars[count]) ;
+			next_underline_style = ml_char_underline_style( &chars[count]) ;
+			next_is_crossed_out = ml_char_is_crossed_out( &chars[count]) ;
 			next_ch_width = x_calculate_char_width( next_xfont ,
 						ch_code , ch_cs , &next_draw_alone) ;
 			next_comb_chars = ml_get_combining_chars( &chars[count] ,
@@ -630,12 +695,14 @@ fc_draw_str(
 			else if( next_xfont != xfont
 				|| next_fg_color != fg_color
 				|| next_bg_color != bg_color
-				|| next_is_underlined != is_underlined
+				|| next_underline_style != underline_style
+				|| next_is_crossed_out != is_crossed_out
 				/*
-				 * Eevn if both is_underline and next_is_underline are 1,
+				 * Eevn if both is_underline and next_is_underline are 1
 				 * underline is drawn one by one in vertical mode.
+				 * (is_crossed_out is the same.)
 				 */
-				|| (is_underlined && xfont->is_vertical)
+				|| ((underline_style || is_crossed_out) && xfont->is_vertical)
 				|| state != next_state
 				|| draw_alone
 				|| next_draw_alone
@@ -656,7 +723,6 @@ fc_draw_str(
 			 * status is changed.
 			 */
 
-			int  color_adjusted ;	/* 0 => None , 1 => BD , 2 => UL */
 			x_color_t *  fg_xcolor ;
 			x_color_t *  bg_xcolor ;
 
@@ -674,8 +740,6 @@ fc_draw_str(
 			}
 		#endif
 
-			color_adjusted = adjust_bd_ul_color( color_man , fg_color , bg_color ,
-							font & FONT_BOLD , is_underlined) ;
 			fg_xcolor = x_get_xcolor( color_man , fg_color) ;
 			bg_xcolor = x_get_xcolor( color_man , bg_color) ;
 
@@ -684,8 +748,11 @@ fc_draw_str(
 			 */
 			if( bg_color == ML_BG_COLOR)
 			{
-				x_window_clear( window ,
-					x , y , current_width - x , height) ;
+				if( updated_width)
+				{
+					x_window_clear( window ,
+						x , y , current_width - x , height) ;
+				}
 			}
 			else
 			{
@@ -696,7 +763,11 @@ fc_draw_str(
 			/*
 			 * drawing string
 			 */
-			if( state == 0)
+			if( ! ml_char_is_visible( &chars[count - 1]))
+			{
+				/* don't draw it */
+			}
+			else if( state == 0)
 			{
 				x_window_ft_draw_string8( window , xfont , fg_xcolor ,
 					x , y + ascent , str8 , str_len) ;
@@ -714,7 +785,9 @@ fc_draw_str(
 			else /* if( state == 3) */
 			{
 				draw_drcs( window , drcs_glyphs , str_len ,
-					x , y , ch_width , height , fg_xcolor) ;
+					x , y + top_margin ,
+					ch_width , height - top_margin - bottom_margin ,
+					fg_xcolor) ;
 			}
 
 			if( comb_chars)
@@ -737,32 +810,19 @@ fc_draw_str(
 					, y + ascent) ;
 			}
 
-			if( ! hide_underline && color_adjusted != 2 && is_underlined)
+			if( ! hide_underline && underline_style)
 			{
-				if( xfont->is_vertical)
-				{
-					x_window_fill_with( window , fg_xcolor ,
-						x , y , (ch_width >> 4) + 1 , height) ;
-				}
-				else
-				{
-					x_window_fill_with( window , fg_xcolor ,
-						x , y + ascent ,
-						current_width - x ,
-						((ascent - bottom_margin) >> 4) + 1 ) ;
-				}
+				draw_line( window , fg_xcolor , xfont->is_vertical ,
+					underline_style == UNDERLINE_DOUBLE ? 1 : 0 ,
+					x , y , current_width - x , height , ascent ,
+					top_margin) ;
 			}
 
-			if( color_adjusted)
+			if( is_crossed_out)
 			{
-				if( color_adjusted == 1)
-				{
-					x_color_manager_adjust_bd_color( color_man) ;
-				}
-				else /* if( color_adjusted == 2) */
-				{
-					x_color_manager_adjust_ul_color( color_man) ;
-				}
+				draw_line( window , fg_xcolor , xfont->is_vertical , 2 ,
+					x , y , current_width - x , height , ascent ,
+					top_margin) ;
 			}
 
 		end_draw:
@@ -777,7 +837,8 @@ fc_draw_str(
 			break ;
 		}
 
-		is_underlined = next_is_underlined ;
+		underline_style = next_underline_style ;
+		is_crossed_out = next_is_crossed_out ;
 		xfont = next_xfont ;
 		font = next_font ;
 		fg_color = next_fg_color ;
@@ -912,7 +973,8 @@ xcore_draw_str(
 	ml_font_t  font ;
 	ml_color_t  fg_color ;
 	ml_color_t  bg_color ;
-	int  is_underlined ;
+	int  underline_style ;
+	int  is_crossed_out ;
 	int  draw_alone ;
 	u_int32_t  pic_glyph ;
 	u_int32_t *  pic_glyphs ;
@@ -927,7 +989,8 @@ xcore_draw_str(
 	ml_font_t  next_font ;
 	ml_color_t  next_fg_color ;
 	ml_color_t  next_bg_color ;
-	int  next_is_underlined ;
+	int  next_underline_style ;
+	int  next_is_crossed_out ;
 	int  next_draw_alone ;
 #ifdef  PERF_DEBUG
 	int  draw_count = 0 ;
@@ -982,7 +1045,8 @@ xcore_draw_str(
 
 	fg_color = ml_char_fg_color( &chars[count]) ;
 	bg_color = ml_char_bg_color( &chars[count]) ;
-	is_underlined = ml_char_is_underlined( &chars[count]) ;
+	underline_style = ml_char_underline_style( &chars[count]) ;
+	is_crossed_out = ml_char_is_crossed_out( &chars[count]) ;
 
 	if( ! ( str2b = str = pic_glyphs = drcs_glyphs =
 		/* '* 2' is for UTF16 surrogate pair. */
@@ -1048,7 +1112,8 @@ xcore_draw_str(
 			ch_cs = FONT_CS(next_font) ;
 			next_fg_color = ml_char_fg_color( &chars[count]) ;
 			next_bg_color = ml_char_bg_color( &chars[count]) ;
-			next_is_underlined = ml_char_is_underlined( &chars[count]) ;
+			next_underline_style = ml_char_underline_style( &chars[count]) ;
+			next_is_crossed_out = ml_char_is_crossed_out( &chars[count]) ;
 			next_ch_width = x_calculate_char_width( next_xfont ,
 						ch_code , ch_cs , &next_draw_alone) ;
 			next_comb_chars = ml_get_combining_chars( &chars[count] ,
@@ -1077,12 +1142,14 @@ xcore_draw_str(
 			else if( next_xfont != xfont
 				|| next_fg_color != fg_color
 				|| next_bg_color != bg_color
-				|| next_is_underlined != is_underlined
+				|| next_underline_style != underline_style
+				|| next_is_crossed_out != is_crossed_out
 				/*
 				 * Eevn if both is_underline and next_is_underline are 1,
 				 * underline is drawn one by one in vertical mode.
+				 * (is_crossed_out is the same.)
 				 */
-				|| (is_underlined && xfont->is_vertical)
+				|| ((underline_style || is_crossed_out) && xfont->is_vertical)
 				|| next_state != state
 				|| draw_alone
 				|| next_draw_alone
@@ -1103,7 +1170,6 @@ xcore_draw_str(
 			 * status is changed.
 			 */
 
-			int  color_adjusted ;	/* 0 => None , 1 => BD , 2 => UL */
 			x_color_t *  fg_xcolor ;
 			x_color_t *  bg_xcolor ;
 
@@ -1121,9 +1187,6 @@ xcore_draw_str(
 			}
 		#endif
 
-			color_adjusted = adjust_bd_ul_color( color_man , fg_color , bg_color ,
-							font & FONT_BOLD , is_underlined) ;
-
 			fg_xcolor = x_get_xcolor( color_man , fg_color) ;
 
 		#ifdef  USE_FRAMEBUFFER
@@ -1161,7 +1224,11 @@ xcore_draw_str(
 						x , y , current_width - x , height) ;
 				}
 
-				if( state == 2)
+				if( ! ml_char_is_visible( &chars[count - 1]))
+				{
+					/* don't draw it */
+				}
+				else if( state == 2)
 				{
 					x_window_draw_string16( window , xfont , fg_xcolor ,
 						x , y + ascent , str2b , str_len) ;
@@ -1179,7 +1246,9 @@ xcore_draw_str(
 				else /* if( state == 3) */
 				{
 					draw_drcs( window , drcs_glyphs , str_len ,
-						x , y , ch_width , height , fg_xcolor) ;
+						x , y + top_margin ,
+						ch_width , height - top_margin - bottom_margin ,
+						fg_xcolor) ;
 				}
 			}
 			else
@@ -1224,32 +1293,19 @@ xcore_draw_str(
 					, y + ascent) ;
 			}
 
-			if( ! hide_underline && color_adjusted != 2 && is_underlined)
+			if( ! hide_underline && underline_style)
 			{
-				if( xfont->is_vertical)
-				{
-					x_window_fill_with( window , fg_xcolor ,
-						x , y , (ch_width >> 4) + 1 , height) ;
-				}
-				else
-				{
-					x_window_fill_with( window , fg_xcolor ,
-						x , y + ascent ,
-						current_width - x ,
-						((ascent - bottom_margin) >> 4) + 1) ;
-				}
+				draw_line( window , fg_xcolor , xfont->is_vertical ,
+					underline_style == UNDERLINE_DOUBLE ? 1 : 0 ,
+					x , y , current_width - x , height , ascent ,
+					top_margin) ;
 			}
 
-			if( color_adjusted)
+			if( is_crossed_out)
 			{
-				if( color_adjusted == 1)
-				{
-					x_color_manager_adjust_bd_color( color_man) ;
-				}
-				else /* if( color_adjusted == 2) */
-				{
-					x_color_manager_adjust_ul_color( color_man) ;
-				}
+				draw_line( window , fg_xcolor , xfont->is_vertical , 2 ,
+					x , y , current_width - x , height , ascent ,
+					top_margin) ;
 			}
 
 		end_draw:
@@ -1268,7 +1324,8 @@ xcore_draw_str(
 		font = next_font ;
 		fg_color = next_fg_color ;
 		bg_color = next_bg_color ;
-		is_underlined = next_is_underlined ;
+		underline_style = next_underline_style ;
+		is_crossed_out = next_is_crossed_out ;
 		state = next_state ;
 		draw_alone = next_draw_alone ;
 		comb_chars = next_comb_chars ;
@@ -1370,7 +1427,10 @@ x_draw_str_to_eol(
 #if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
 	case  TYPE_XFT:
 	case  TYPE_CAIRO:
-		if( ! fc_draw_str( window , font_man , color_man , &updated_width ,
+		x_window_clear( window , x , y , window->width - x , height) ;
+
+		if( ! fc_draw_str( window , font_man , color_man ,
+			NULL /* NULL disables x_window_clear() in fc_draw_str() */ ,
 			chars , num_of_chars , x , y , height , ascent ,
 			top_margin , bottom_margin , hide_underline))
 		{
@@ -1389,15 +1449,15 @@ x_draw_str_to_eol(
 			return	0 ;
 		}
 
+		if( updated_width < window->width)
+		{
+			x_window_clear( window , updated_width , y ,
+				window->width - updated_width , height) ;
+		}
+
 		break ;
 #endif
 	}
 
-	if( updated_width < window->width)
-	{
-		x_window_clear( window , updated_width , y ,
-			window->width - updated_width , height) ;
-	}
-
 	return	1 ;
 }
