@@ -103,7 +103,7 @@ receive_next_event(void)
 		tval.tv_sec = 0 ;
 
 	#ifdef  USE_LIBSSH2
-		if( ( ret = ml_pty_ssh_poll( &read_fds)) > 0)
+		if( ml_pty_ssh_poll( &read_fds) > 0)
 		{
 			num_of_displays = 0 ;	/* Don't check FD_ISSET(x_display_fd) */
 
@@ -111,28 +111,34 @@ receive_next_event(void)
 		}
 	#endif
 
-		displays = x_get_opened_displays( &num_of_displays) ;
-
 		maxfd = 0 ;
 		FD_ZERO( &read_fds) ;
 
 	#ifdef  USE_LIBSSH2
 		for( count = 0 ; count < num_of_xssh_fds ; count ++)
 		{
-			FD_SET( xssh_fds[count] , &read_fds) ;
-
-			if( xssh_fds[count] > maxfd)
+			if( xssh_fds[count] >= 0)
 			{
-				maxfd = xssh_fds[count] ;
+				FD_SET( xssh_fds[count] , &read_fds) ;
+
+				if( xssh_fds[count] > maxfd)
+				{
+					maxfd = xssh_fds[count] ;
+				}
 			}
 		}
 	#endif
 
+		displays = x_get_opened_displays( &num_of_displays) ;
+
 		for( count = 0 ; count < num_of_displays ; count ++)
 		{
 		#ifdef  X_PROTOCOL
-			/* XFlush() is necessary after using Xlib functions. */
-			XFlush( displays[count]->display) ;
+			/*
+			 * Need to read pending events and to flush events in
+			 * output buffer on X11 before waiting in select().
+			 */
+			x_display_sync( displays[count]) ;
 		#endif
 
 			xfd = x_display_fd( displays[count]) ;
@@ -236,7 +242,7 @@ receive_next_event(void)
 	for( count = num_of_xssh_fds ; count > 0 ; count--)
 	{
 		ml_pty_ssh_send_recv_x11( count - 1 ,
-			FD_ISSET( xssh_fds[count - 1] , &read_fds)) ;
+			xssh_fds[count - 1] >= 0 && FD_ISSET( xssh_fds[count - 1] , &read_fds)) ;
 	}
 #endif
 
