@@ -5,6 +5,7 @@
 #include  "x_font_config.h"
 
 #include  <string.h>		/* memset */
+
 #include  <kiklib/kik_mem.h>	/* malloc */
 #include  <kiklib/kik_str.h>	/* strdup */
 #include  <kiklib/kik_util.h>	/* DIGIT_STR_LEN */
@@ -12,9 +13,10 @@
 #include  <kiklib/kik_file.h>
 
 #include  <ml_char.h>
+#include  <ml_char_encoding.h>	/* ml_parse_unicode_area */
 
 
-#define  DEFAULT_FONT (MAX_CHARSET + 1)
+#define  DEFAULT_FONT  0x1ff	/* MAX_CHARSET */
 
 #if  0
 #define  __DEBUG
@@ -255,11 +257,11 @@ parse_key(
 
 	if( key_len >= 3 && strncmp( key , "U+" , 2) == 0)
 	{
-		int  min ;
-		int  max ;
+		u_int  min ;
+		u_int  max ;
 
-		if( sscanf( key + 2 , "%x-%x" , &min , &max) == 2 &&
-		    ( font = ml_char_add_unicode_area_font( min , max)) != UNKNOWN_CS)
+		if( ml_parse_unicode_area( key , &min , &max) &&
+		    ( font = ml_char_get_unicode_area_font( min , max)) != UNKNOWN_CS)
 		{
 			goto  check_style ;
 		}
@@ -296,21 +298,11 @@ parse_key(
 
 	font = NORMAL_FONT_OF(cs) ;
 
-	if( font & FONT_FULLWIDTH)
+	if( ! ( font & FONT_FULLWIDTH) &&
+	    ( strstr( key , "_BIWIDTH") || /* compat with 3.2.2 or before. */
+	      strstr( key , "_FULLWIDTH")))
 	{
-		if( strstr( key , "_NARROW") || /* compat with 3.2.2 or before. */
-		    strstr( key , "_HALFWIDTH"))
-		{
-			font &= ~FONT_FULLWIDTH ;
-		}
-	}
-	else
-	{
-		if( strstr( key , "_BIWIDTH") || /* compat with 3.2.2 or before. */
-		    strstr( key , "_FULLWIDTH"))
-		{
-			font |= FONT_FULLWIDTH ;
-		}
+		font |= FONT_FULLWIDTH ;
 	}
 
 check_style:
@@ -511,8 +503,9 @@ read_conf(
 static int
 read_all_conf(
 	x_font_config_t *  font_config ,
-	const char *  changed_font_file	/* This function is called after a font file is changed,
-					 * specify it. Otherwise specify NULL. */
+	const char *  changed_font_file	/* If this function is called after a font file is
+	                                 * changed, specify it here to avoid re-read font files.
+					 * Otherwise specify NULL. */
 	)
 {
 	char *  font_rcfile ;
@@ -2058,12 +2051,8 @@ x_get_all_config_font_names(
 		/*
 		 * XXX
 		 * Ignore DEFAULT_FONT setting because it doesn't have encoding name.
-		 * Also ignore XXXX_HALFWIDTH and XXX_FULLWIDTH except ISO10646_UCS4_1_FULLWIDTH.
 		 */
-		if( array[count]->key != DEFAULT_FONT &&
-		    (FONT_CS(array[count]->key) == ISO10646_UCS4_1 ||
-		     (IS_FULLWIDTH_CS(FONT_CS(array[count]->key)) ==
-		      ((array[count]->key & FONT_FULLWIDTH) == FONT_FULLWIDTH))) )
+		if( FONT_CS(array[count]->key) != DEFAULT_FONT)
 		{
 			strcpy( p , array[count]->value) ;
 			p += strlen( p) ;
@@ -2076,12 +2065,8 @@ x_get_all_config_font_names(
 		/*
 		 * XXX
 		 * Ignore DEFAULT_FONT setting because it doesn't have encoding name.
-		 * Also ignore XXXX_NARROW and XXX_FULLWIDTH except ISO10646_UCS4_FULLWIDTH.
 		 */
-		if( d_array[count]->key != DEFAULT_FONT &&
-		    ( FONT_CS(d_array[count]->key) == ISO10646_UCS4_1 ||
-		      (IS_FULLWIDTH_CS(FONT_CS(d_array[count]->key)) ==
-		        ((d_array[count]->key & FONT_FULLWIDTH) == FONT_FULLWIDTH))) )
+		if( FONT_CS(d_array[count]->key) != DEFAULT_FONT)
 		{
 			sprintf( p , d_array[count]->value , font_size) ;
 			p += strlen( p) ;
