@@ -108,7 +108,8 @@ x_font_manager_new(
 	int  use_multi_col_char ,
 	u_int  step_in_changing_font_size ,
 	u_int  letter_space ,
-	int  use_bold_font
+	int  use_bold_font ,
+	int  use_italic_font
 	)
 {
 	x_font_manager_t *  font_man ;
@@ -176,6 +177,7 @@ x_font_manager_new(
 	}
 
 	font_man->use_bold_font = use_bold_font ;
+	font_man->use_italic_font = use_italic_font ;
 
 	return  font_man ;
 }
@@ -208,6 +210,11 @@ x_get_font(
 		font &= ~FONT_BOLD ;
 	}
 
+	if( ! font_man->use_italic_font)
+	{
+		font &= ~FONT_ITALIC ;
+	}
+
 	if( ( xfont = x_font_cache_get_xfont( font_man->font_cache , font)))
 	{
 		return  xfont ;
@@ -500,6 +507,22 @@ x_set_use_bold_font(
 	return  1 ;
 }
 
+int
+x_set_use_italic_font(
+	x_font_manager_t *  font_man ,
+	int  use_italic_font
+	)
+{
+	if( font_man->use_italic_font == use_italic_font)
+	{
+		return  0 ;
+	}
+
+	font_man->use_italic_font = use_italic_font ;
+
+	return  1 ;
+}
+
 XFontSet
 x_get_fontset(
 	x_font_manager_t *  font_man
