@@ -197,38 +197,23 @@ resize(
 	u_int  height
 	)
 {
-	int  x ;
-	int  y ;
-
 	if( x_window_resize( &cand_screen->window , width , height , 0))
 	{
+		int  x ;
+		int  y ;
+
 		x = cand_screen->x ;
 		y = cand_screen->y ;
 		adjust_window_position_by_size( cand_screen , &x , &y) ;
 
 		if( x != cand_screen->window.x || y != cand_screen->window.y)
 		{
-		#ifdef  USE_FRAMEBUFFER
-			/*
-			 * In frmebuffer,
-			 * x_im_candidate_screen::draw_screen() ->
-			 * x_im_candidate_screen::resize() ->
-			 * x_window_move() ->
-			 * x_im_candidate_screen::window_exposed() ->
-			 * x_im_candidate_screen::draw_screen()
-			 */
-			x_window_move_no_expose( &cand_screen->window , x , y) ;
-		#else
 			x_window_move( &cand_screen->window , x , y) ;
-		#endif
-		}
-	#ifdef  USE_FRAMEBUFFER
-		else
-		{
-			/* resized but position is not changed. */
-			x_window_clear_margin_area( &cand_screen->window) ;	/* XXX */
 		}
 
+	#ifdef  USE_FRAMEBUFFER
+		/* resized but position is not changed. */
+		x_display_reset_input_method_window() ;
 		x_window_draw_rect_frame( &cand_screen->window , -MARGIN , -MARGIN ,
 					  cand_screen->window.width + MARGIN - 1 ,
 					  cand_screen->window.height + MARGIN - 1) ;
@@ -242,7 +227,8 @@ static void
 draw_screen_vertical(
 	x_im_candidate_screen_t *  cand_screen ,
 	u_int  top ,
-	u_int  last
+	u_int  last ,
+	int  do_resize
 	)
 {
 	x_font_t *  xfont ;
@@ -286,30 +272,38 @@ draw_screen_vertical(
 
 	xfont = x_get_usascii_font( cand_screen->font_man) ;
 
-	/* width of window */
-	win_width = xfont->width * (num_of_digits + 1) ;
-	win_width += max_candidate_width( cand_screen->font_man ,
-					  &cand_screen->candidates[top] ,
-					  last - top + 1) ;
-	if( win_width < ( MAX_NUM_OF_DIGITS * 2 + 1) * xfont->width)
+	if( do_resize)
 	{
-		win_width = ( MAX_NUM_OF_DIGITS * 2 + 1) * xfont->width ;
-	}
+		/* width of window */
+		win_width = xfont->width * (num_of_digits + 1) ;
+		win_width += max_candidate_width( cand_screen->font_man ,
+						  &cand_screen->candidates[top] ,
+						  last - top + 1) ;
+		if( win_width < ( MAX_NUM_OF_DIGITS * 2 + 1) * xfont->width)
+		{
+			win_width = ( MAX_NUM_OF_DIGITS * 2 + 1) * xfont->width ;
+		}
 
-	/* height of window */
-	if( cand_screen->num_of_candidates > cand_screen->num_per_window)
-	{
-		win_height = (xfont->height + LINE_SPACE) *
-			     (cand_screen->num_per_window + 1) ;
+		/* height of window */
+		if( cand_screen->num_of_candidates > cand_screen->num_per_window)
+		{
+			win_height = (xfont->height + LINE_SPACE) *
+				     (cand_screen->num_per_window + 1) ;
+		}
+		else
+		{
+			win_height = (xfont->height + LINE_SPACE) *
+				     cand_screen->num_of_candidates ;
+		}
+
+		resize( cand_screen , win_width , win_height) ;
 	}
 	else
 	{
-		win_height = (xfont->height + LINE_SPACE) *
-			     cand_screen->num_of_candidates ;
+		win_width = cand_screen->window.width ;
+		win_height = cand_screen->window.height ;
 	}
 
-	resize( cand_screen , win_width , win_height) ;
-
 	/*
 	 * digits and candidates
 	 */
@@ -363,7 +357,7 @@ draw_screen_vertical(
 
 			ml_char_set( &ch , digit[j] , US_ASCII , 0 , 0 ,
 				     ML_FG_COLOR , ML_BG_COLOR ,
-				     0 , 0 , 0) ;
+				     0 , 0 , 0 , 0 , 0) ;
 
 			x_draw_str( &cand_screen->window ,
 				    cand_screen->font_man ,
@@ -454,7 +448,7 @@ draw_screen_vertical(
 
 			ml_char_set( &ch , navi[i] , US_ASCII , 0 , 0 ,
 				     ML_FG_COLOR , ML_BG_COLOR ,
-				     0 , 0 , 0) ;
+				     0 , 0 , 0 , 0 , 0) ;
 
 			x_draw_str( &cand_screen->window ,
 				    cand_screen->font_man ,
@@ -473,7 +467,8 @@ static void
 draw_screen_horizontal(
 	x_im_candidate_screen_t *  cand_screen ,
 	u_int  top ,
-	u_int  last
+	u_int  last ,
+	int  do_resize
 	)
 {
 	x_font_t *  xfont ;
@@ -495,22 +490,30 @@ draw_screen_horizontal(
 
 	xfont = x_get_usascii_font( cand_screen->font_man) ;
 
-	/* width of window */
-	win_width = 0 ;
-	for( i = 1 ; i <= last - top + 1 ; i++)
+	if( do_resize)
 	{
-		u_int  num_of_digits ;
+		/* width of window */
+		win_width = 0 ;
+		for( i = 1 ; i <= last - top + 1 ; i++)
+		{
+			u_int  num_of_digits ;
 
-		NUM_OF_DIGITS( num_of_digits , i) ;
-		win_width += xfont->width * (num_of_digits + 2) ;
+			NUM_OF_DIGITS( num_of_digits , i) ;
+			win_width += xfont->width * (num_of_digits + 2) ;
+		}
+		win_width += total_candidate_width( cand_screen->font_man ,
+						    cand_screen->candidates ,
+						    top , last) ;
+		/* height of window */
+		win_height = xfont->height + LINE_SPACE ;
+
+		resize( cand_screen , win_width , win_height) ;
+	}
+	else
+	{
+		win_width = cand_screen->window.width ;
+		win_height = cand_screen->window.height ;
 	}
-	win_width += total_candidate_width( cand_screen->font_man ,
-					    cand_screen->candidates ,
-					    top , last) ;
-	/* height of window */
-	win_height = xfont->height + LINE_SPACE ;
-
-	resize( cand_screen , win_width , win_height) ;
 
 	for( i = top ; i <= last; i++)
 	{
@@ -543,7 +546,7 @@ draw_screen_horizontal(
 			ml_char_init( &ch) ;
 
 			ml_char_set( &ch , digit[j] , US_ASCII , 0 , 0 ,
-				     ML_FG_COLOR , ML_BG_COLOR , 0 , 0 , 0) ;
+				     ML_FG_COLOR , ML_BG_COLOR , 0 , 0 , 0 , 0 , 0) ;
 
 			x_draw_str( &cand_screen->window ,
 				    cand_screen->font_man ,
@@ -591,7 +594,8 @@ draw_screen_horizontal(
 
 static void
 draw_screen(
-	x_im_candidate_screen_t *  cand_screen
+	x_im_candidate_screen_t *  cand_screen ,
+	int  do_resize
 	)
 {
 	u_int  top ;
@@ -604,11 +608,11 @@ draw_screen(
 
 	if( cand_screen->is_vertical_direction)
 	{
-		draw_screen_vertical( cand_screen , top , last) ;
+		draw_screen_vertical( cand_screen , top , last , do_resize) ;
 	}
 	else
 	{
-		draw_screen_horizontal( cand_screen , top , last) ;
+		draw_screen_horizontal( cand_screen , top , last , do_resize) ;
 	}
 }
 
@@ -618,8 +622,6 @@ adjust_window_x_position(
 	int *  x
 	)
 {
-	u_int  dh ;
-	u_int  dw ;
 	u_int  top ;
 	u_int  last ;
 	u_int  num_of_digits ;
@@ -650,9 +652,6 @@ adjust_window_x_position(
 		num_of_digits = 1 ;
 	}
 
-	dh = cand_screen->window.disp->height ;
-	dw = cand_screen->window.disp->width ;
-
 	if( num_of_digits)
 	{
 		*x -= (x_get_usascii_font( cand_screen->font_man)->width * (num_of_digits + 1) +
@@ -699,10 +698,7 @@ hide(
 	x_im_candidate_screen_t *  cand_screen
 	)
 {
-	if( ! cand_screen->window.is_focused)
-	{
-		x_window_unmap( &cand_screen->window) ;
-	}
+	x_window_unmap( &cand_screen->window) ;
 
 	return  1 ;
 }
@@ -722,14 +718,14 @@ set_spot(
 
 	if( cand_screen->window.x != x || cand_screen->window.y != y)
 	{
-	#ifdef  USE_FRAMEBUFFER
-		x_window_move_no_expose( &cand_screen->window , x , y) ;
-	#else
 		x_window_move( &cand_screen->window , x , y) ;
-	#endif
-	}
 
-	return  1 ;
+		return  1 ;
+	}
+	else
+	{
+		return  0 ;
+	}
 }
 
 static int
@@ -863,7 +859,7 @@ set_candidate(
 
 			if( ml_char_combine( p - 1 , mkf_char_to_int(&ch) ,
 					ch.cs , is_fullwidth , is_comb ,
-					ML_FG_COLOR , ML_BG_COLOR , 0 , 0 , 0))
+					ML_FG_COLOR , ML_BG_COLOR , 0 , 0 , 0 , 0 , 0))
 			{
 				continue ;
 			}
@@ -874,7 +870,7 @@ set_candidate(
 		}
 
 		ml_char_set( p , mkf_char_to_int(&ch) , ch.cs , is_fullwidth ,
-			is_comb , ML_FG_COLOR , ML_BG_COLOR , 0 , 0 , 0) ;
+			is_comb , ML_FG_COLOR , ML_BG_COLOR , 0 , 0 , 0 , 0 , 0) ;
 
 		p++ ;
 		cand_screen->candidates[index].filled_len++ ;
@@ -925,7 +921,7 @@ select_candidate(
 
 	cand_screen->index = index ;
 
-	draw_screen( cand_screen) ;
+	draw_screen( cand_screen , 1) ;
 
 	return  1 ;
 }
@@ -962,7 +958,7 @@ window_exposed(
 	u_int  height
 	)
 {
-	draw_screen( (x_im_candidate_screen_t *) win) ;
+	draw_screen( (x_im_candidate_screen_t *) win , 0) ;
 
 	/* draw border (margin area has been already cleared in x_window.c) */
 	x_window_draw_rect_frame( win , -MARGIN , -MARGIN ,
