@@ -13,76 +13,159 @@
 #define  LINE_SPACE	2
 
 
-/* --- static variables --- */
-
-
 /* --- static functions --- */
 
-static int  set_spot( x_im_status_screen_t *  stat_screen , int  x , int  y) ;
+static void
+adjust_window_position_by_size(
+	x_im_status_screen_t *  stat_screen ,
+	int *  x ,
+	int *  y
+	)
+{
+	if( *y + ACTUAL_HEIGHT(&stat_screen->window) >= stat_screen->window.disp->height)
+	{
+		*y -= ACTUAL_HEIGHT(&stat_screen->window) ;
+		if( ! stat_screen->is_vertical)
+		{
+			*y -= stat_screen->line_height ;
+		}
+	}
+
+	if( *x + ACTUAL_WIDTH(&stat_screen->window) >= stat_screen->window.disp->width)
+	{
+		if( stat_screen->is_vertical)
+		{
+			/* x_im_stat_screen doesn't know column width. */
+			*x -= (ACTUAL_WIDTH(&stat_screen->window) + stat_screen->line_height) ;
+		}
+		else
+		{
+			*x = stat_screen->window.disp->width - ACTUAL_WIDTH(&stat_screen->window) ;
+		}
+	}
+}
+
+#ifdef  USE_FRAMEBUFFER
+static void
+reset_screen(
+	x_window_t *  win
+	)
+{
+	x_display_reset_input_method_window() ;
+	x_window_draw_rect_frame( win , -MARGIN , -MARGIN ,
+				  win->width + MARGIN - 1 ,
+				  win->height + MARGIN - 1) ;
+}
+#endif
 
 static void
 draw_screen(
-	x_im_status_screen_t *  stat_screen
+	x_im_status_screen_t *  stat_screen ,
+	int  do_resize
 	)
 {
+#define  MAX_ROWS ((sizeof(stat_screen->head_indexes) / sizeof(stat_screen->head_indexes[0])) - 1)
 	x_font_t *  xfont ;
-	u_int  width = 0 ;
-	int  i ;
+	u_int  line_height ;
+	int *  heads ;
+	u_int  i ;
+
+	xfont = x_get_usascii_font( stat_screen->font_man) ;
+	line_height = xfont->height + LINE_SPACE ;
+	heads = stat_screen->head_indexes ;
 
 	/*
 	 * resize window
 	 */
 
 	/* width of window */
-	for( i = 0 ; i < stat_screen->filled_len ; i++)
+
+	if( do_resize)
 	{
-		x_font_t *  xfont ;
+		u_int  max_width ;
+		u_int  width ;
+		u_int  rows ;
 
-		xfont = x_get_font( stat_screen->font_man ,
-				    ml_char_font( &stat_screen->chars[i])) ;
+		max_width = stat_screen->window.disp->width / 2 ;
+		width = 0 ;
+		heads[0] = 0 ;
+		rows = 1 ;
 
-		width += x_calculate_char_width(
-					xfont ,
+		for( i = 0 ; i < stat_screen->filled_len ; i++)
+		{
+			u_int  ch_width ;
+
+			ch_width = x_calculate_char_width(
+					x_get_font( stat_screen->font_man ,
+						ml_char_font( &stat_screen->chars[i])) ,
 					ml_char_code( &stat_screen->chars[i]) ,
 					ml_char_cs( &stat_screen->chars[i]) , NULL) ;
-	}
 
-	xfont = x_get_usascii_font( stat_screen->font_man) ;
+			if( width + ch_width > max_width)
+			{
+				if( rows == 1)
+				{
+					max_width = width ;
+				}
 
-	if( x_window_resize( &stat_screen->window , width ,
-				xfont->height + LINE_SPACE, 0))
-	{
-		/* Reset window position */
+				heads[rows++] = i ;
 
-	#ifdef  USE_FRAMEBUFFER
-		if( set_spot( stat_screen , stat_screen->x , stat_screen->y))
-		{
-			/*
-			 * set_spot() -> x_window_move() -> window_exposed() ->
-			 * draw_screen() and x_window_draw_rect_frame().
-			 */
-			return ;
+				if( rows == MAX_ROWS)
+				{
+					break ;
+				}
+
+				width = ch_width ;
+			}
+			else
+			{
+				width += ch_width ;
+			}
 		}
 
-		x_window_draw_rect_frame( &stat_screen->window , -MARGIN , -MARGIN ,
-					  stat_screen->window.width + MARGIN - 1 ,
-					  stat_screen->window.height + MARGIN - 1) ;
-	#else
-		set_spot( stat_screen , stat_screen->x , stat_screen->y) ;
-	#endif
+		if( rows > 1)
+		{
+			width = max_width ;
+		}
+
+		/* for following 'heads[i + 1] - heads[i]' */
+		heads[rows] = stat_screen->filled_len ;
+
+		if( x_window_resize( &stat_screen->window , width ,
+					line_height * rows , 0))
+		{
+			int  x ;
+			int  y ;
+
+			x = stat_screen->x ;
+			y = stat_screen->y ;
+			adjust_window_position_by_size( stat_screen , &x , &y) ;
+
+			if( stat_screen->window.x != x || stat_screen->window.y != y)
+			{
+				x_window_move( &stat_screen->window , x , y) ;
+			}
+
+		#ifdef  USE_FRAMEBUFFER
+			reset_screen( &stat_screen->window) ;
+		#endif
+		}
 	}
 
-	x_draw_str_to_eol( &stat_screen->window ,
-			   stat_screen->font_man ,
-			   stat_screen->color_man ,
-			   stat_screen->chars ,
-			   stat_screen->filled_len ,
-			   0 , 0 ,
-			   xfont->height + LINE_SPACE ,
-			   xfont->ascent + LINE_SPACE / 2 ,
-			   LINE_SPACE / 2 ,
-			   LINE_SPACE / 2 + LINE_SPACE % 2 ,
-			   1 /* no need to draw underline */) ;
+	for( i = 0 ; heads[i] < stat_screen->filled_len ; i++)
+	{
+		x_draw_str_to_eol( &stat_screen->window ,
+				   stat_screen->font_man ,
+				   stat_screen->color_man ,
+				   stat_screen->chars + heads[i] ,
+				   heads[i + 1] - heads[i] ,
+				   0 , line_height * i ,
+				   line_height ,
+				   xfont->ascent + LINE_SPACE / 2 ,
+				   LINE_SPACE / 2 ,
+				   LINE_SPACE / 2 + LINE_SPACE % 2 ,
+				   1 /* no need to draw underline */) ;
+	}
 }
 
 
@@ -122,10 +205,7 @@ hide(
 	x_im_status_screen_t *  stat_screen
 	)
 {
-	if( ! stat_screen->window.is_focused)
-	{
-		x_window_unmap( &stat_screen->window) ;
-	}
+	x_window_unmap( &stat_screen->window) ;
 
 	return  1 ;
 }
@@ -140,31 +220,15 @@ set_spot(
 	stat_screen->x = x ;
 	stat_screen->y = y ;
 
-	if( y + ACTUAL_HEIGHT(&stat_screen->window) >= stat_screen->window.disp->height)
-	{
-		y -= ACTUAL_HEIGHT(&stat_screen->window) ;
-		if( ! stat_screen->is_vertical)
-		{
-			y -= stat_screen->line_height ;
-		}
-	}
-
-	if( x + ACTUAL_WIDTH(&stat_screen->window) >= stat_screen->window.disp->width)
-	{
-		if( stat_screen->is_vertical)
-		{
-			/* x_im_stat_screen doesn't know column width. */
-			x -= (ACTUAL_WIDTH(&stat_screen->window) + stat_screen->line_height) ;
-		}
-		else
-		{
-			x = stat_screen->window.disp->width - ACTUAL_WIDTH(&stat_screen->window) ;
-		}
-	}
+	adjust_window_position_by_size( stat_screen , &x , &y) ;
 
 	if( stat_screen->window.x != x || stat_screen->window.y != y)
 	{
 		x_window_move( &stat_screen->window , x , y) ;
+	#ifdef  USE_FRAMEBUFFER
+		reset_screen( &stat_screen->window) ;
+		draw_screen( stat_screen , 0) ;
+	#endif
 
 		return  1 ;
 	}
@@ -251,7 +315,7 @@ set(
 		{
 			if( ml_char_combine( p - 1 , mkf_char_to_int(&ch) ,
 				ch.cs , is_fullwidth , is_comb , ML_FG_COLOR ,
-				ML_BG_COLOR , 0 , 0 , 0))
+				ML_BG_COLOR , 0 , 0 , 0 , 0 , 0))
 			{
 				continue;
 			}
@@ -267,13 +331,13 @@ set(
 		}
 
 		ml_char_set( p , mkf_char_to_int(&ch) , ch.cs , is_fullwidth ,
-			is_comb , ML_FG_COLOR , ML_BG_COLOR , 0 , 0 , 0) ;
+			is_comb , ML_FG_COLOR , ML_BG_COLOR , 0 , 0 , 0 , 0 , 0) ;
 
 		p++ ;
 		stat_screen->filled_len++ ;
 	}
 
-	draw_screen( stat_screen) ;
+	draw_screen( stat_screen , 1) ;
 
 	return  1 ;
 }
@@ -310,7 +374,7 @@ window_exposed(
 	u_int  height
 	)
 {
-	draw_screen( (x_im_status_screen_t *) win) ;
+	draw_screen( (x_im_status_screen_t *) win , 0) ;
 
 	/* draw border (margin area has been already cleared in x_window.c) */
 	x_window_draw_rect_frame( win , -MARGIN , -MARGIN ,
