@@ -9,6 +9,8 @@
 #include  <kiklib/kik_str.h>	/* kik_str_to_uint */
 #include  <kiklib/kik_locale.h>	/* kik_get_lang */
 
+#include  "ml_term_manager.h"
+
 
 /* --- global functions --- */
 
@@ -198,18 +200,28 @@ x_prepare_for_main_config(
 		"step in changing font size in GUI configurator [1]") ;
 	kik_conf_add_opt( conf , '\0' , "bdfont" , 1 , "use_bold_font" ,
 		"use bold fonts [true]") ;
+	kik_conf_add_opt( conf , '\0' , "itfont" , 1 , "use_italic_font" ,
+		"use italic fonts [true]") ;
 	kik_conf_add_opt( conf , '\0' , "iconpath" , 0 , "icon_path" ,
 		"path to an imagefile to be use as an window icon") ;
 #if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_FRIBIDI)
 	kik_conf_add_opt( conf , '\0' , "bimode" , 0 , "bidi_mode" ,
 		"bidi mode [normal]") ;
+	kik_conf_add_opt( conf , '\0' , "bisep" , 0 , "bidi_separators" ,
+		"Separator characters to render bidi text") ;
 #endif
 	kik_conf_add_opt( conf , '\0' , "parent" , 0 , "parent_window" ,
 		"parent window") ;
 	kik_conf_add_opt( conf , '\0' , "bd" , 0 , "bd_color" ,
 		"Color to use to display bold characters (equivalent to colorBD)") ;
+	kik_conf_add_opt( conf , '\0' , "it" , 0 , "it_color" ,
+		"Color to use to display italic characters") ;
 	kik_conf_add_opt( conf , '\0' , "ul" , 0 , "ul_color" ,
 		"Color to use to display underlined characters (equivalent to colorUL)") ;
+	kik_conf_add_opt( conf , '\0' , "bl" , 0 , "bl_color" ,
+		"Color to use to display blinking characters (equivalent to colorBL)") ;
+	kik_conf_add_opt( conf , '\0' , "co" , 0 , "co_color" ,
+		"Color to use to display crossed-out characters") ;
 	kik_conf_add_opt( conf , '\0' , "noul" , 1 , "hide_underline" ,
 		"Don't draw underline [false]") ;
 #if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
@@ -245,8 +257,6 @@ x_prepare_for_main_config(
 #endif
 	kik_conf_add_opt( conf , '\0' , "csp" , 0 , "letter_space" ,
 		"extra space between letters in pixels [0]") ;
-	kik_conf_add_opt( conf , '\0' , "ucsprop" , 1 , "use_unicode_property" ,
-		"use unicode property for characters [false]") ;
 	kik_conf_add_opt( conf , '\0' , "osc52" , 1 , "allow_osc52" ,
 		"allow access to clipboard by OSC 52 sequence [false]") ;
 	kik_conf_add_opt( conf , '\0' , "blink" , 1 , "blink_cursor" ,
@@ -271,18 +281,28 @@ x_prepare_for_main_config(
 		"force unicode or decsp font for box-drawing characters. [noconv]") ;
 	kik_conf_add_opt( conf , '\0' , "urgent" , 1 , "use_urgent_bell" ,
 		"draw the user's attention when making a bell sound. [false]") ;
-	kik_conf_add_opt( conf , '\0' , "locale" , 0 , "locale" , "set locale.") ;
+	kik_conf_add_opt( conf , '\0' , "locale" , 0 , "locale" , "set locale") ;
 	kik_conf_add_opt( conf , '\0' , "ucsnoconv" , 0 , "unicode_noconv_areas" ,
-		"use unicode fonts partially regardless of -n option.") ;
+		"use unicode fonts partially regardless of -n option") ;
+	kik_conf_add_opt( conf , '\0' , "fullwidth" , 0 , "unicode_full_width_areas" ,
+		"force full width regardless of EastAsianWidth.txt") ;
 	kik_conf_add_opt( conf , '\0' , "ade" , 0 , "auto_detect_encodings" ,
-		"encodings detected automatically.") ;
+		"encodings detected automatically") ;
 	kik_conf_add_opt( conf , '\0' , "auto" , 1 , "use_auto_detect" ,
-		"detect character encoding automatically.") ;
+		"detect character encoding automatically") ;
 	kik_conf_add_opt( conf , '\0' , "ldd" , 1 , "leftward_double_drawing" ,
-		"embold glyphs by drawing doubly at 1 pixel leftward instead of rightward.") ;
+		"embold glyphs by drawing doubly at 1 pixel leftward instead of rightward") ;
+	kik_conf_add_opt( conf , '\0' , "working-directory" , 0 , "working_directory" ,
+		"working directory") ;
+	kik_conf_add_opt( conf , '\0' , "seqfmt" , 0 , "vt_seq_format" ,
+		"format of logging vt100 sequence. [raw]") ;
 #ifdef  USE_GRF
 	kik_conf_add_opt( conf , '\0' , "multivram" , 1 , "separate_wall_picture" ,
-		"draw wall picture on another vram. (available on 4bpp) [false]") ;
+		"draw wall picture on another vram. (available on 4bpp) [true]") ;
+#endif
+#ifdef  USE_FRAMEBUFFER
+	kik_conf_add_opt( conf , '\0' , "rotate" , 0 , "rotate_display" ,
+		"rotate display. [none]") ;
 #endif
 #ifdef  USE_IM_CURSOR_COLOR
 	kik_conf_add_opt( conf , '\0' , "imcolor" , 0 , "im_cursor_color" ,
@@ -429,6 +449,11 @@ x_main_config_init(
 		}
 	}
 
+	if( ( value = kik_conf_get_value( conf , "vt_seq_format")))
+	{
+		ml_set_use_ttyrec_format( strcmp( value , "ttyrec") == 0) ;
+	}
+
 	if( ( value = kik_conf_get_value( conf , "logging_msg")) &&
 	    strcmp( value , "false") == 0)
 	{
@@ -527,16 +552,38 @@ x_main_config_init(
 		main_config->cursor_bg_color = strdup( value) ;
 	}
 
+	main_config->alt_color_mode = 0 ;
+
 	if( ( value = kik_conf_get_value( conf , "bd_color")))
 	{
 		main_config->bd_color = strdup( value) ;
+		main_config->alt_color_mode |= ALT_COLOR_BOLD ;
+	}
+
+	if( ( value = kik_conf_get_value( conf , "it_color")))
+	{
+		main_config->it_color = strdup( value) ;
+		main_config->alt_color_mode |= ALT_COLOR_ITALIC ;
 	}
 
 	if( ( value = kik_conf_get_value( conf , "ul_color")))
 	{
 		main_config->ul_color = strdup( value) ;
+		main_config->alt_color_mode |= ALT_COLOR_UNDERLINE ;
 	}
-	
+
+	if( ( value = kik_conf_get_value( conf , "bl_color")))
+	{
+		main_config->ul_color = strdup( value) ;
+		main_config->alt_color_mode |= ALT_COLOR_BLINKING ;
+	}
+
+	if( ( value = kik_conf_get_value( conf , "co_color")))
+	{
+		main_config->ul_color = strdup( value) ;
+		main_config->alt_color_mode |= ALT_COLOR_CROSSED_OUT ;
+	}
+
 	if( ( value = kik_conf_get_value( conf , "sb_fg_color")))
 	{
 		main_config->sb_fg_color = strdup( value) ;
@@ -755,6 +802,11 @@ x_main_config_init(
 		ml_set_unicode_noconv_areas( value) ;
 	}
 
+	if( ( value = kik_conf_get_value( conf , "unicode_full_width_areas")))
+	{
+		ml_set_full_width_areas( value) ;
+	}
+
 	if( ( value = kik_conf_get_value( conf , "only_use_unicode_font")))
 	{
 		if( strcmp( value , "true") == 0)
@@ -775,23 +827,6 @@ x_main_config_init(
 		}
 	}
 
-	if( ( value = kik_conf_get_value( conf , "use_unicode_property")))
-	{
-		if( strcmp( value , "true") == 0)
-		{
-			if( main_config->unicode_policy == ONLY_USE_UNICODE_FONT)
-			{
-				kik_msg_printf(
-					"only_use_unicode_font and use_unicode_property "
-					"cannot be used at the same time.\n") ;
-			}
-			else
-			{
-				main_config->unicode_policy |= USE_UNICODE_PROPERTY ;
-			}
-		}
-	}
-
 	if( ( value = kik_conf_get_value( conf , "box_drawing_font")))
 	{
 		if( strcmp( value , "decsp") == 0)
@@ -969,16 +1004,29 @@ x_main_config_init(
 		main_config->encoding = ML_ISO8859_1 ;
 	}
 
-#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_FRIBIDI)
-	main_config->use_bidi = 1 ;
+#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_IND)
+	if( ( value = kik_conf_get_value( conf , "use_ind")))
+	{
+		if( strcmp( value , "true") == 0)
+		{
+			main_config->use_ind = 1 ;
+		}
+	}
+#endif
 
+#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_FRIBIDI)
 	if( ( value = kik_conf_get_value( conf , "use_bidi")))
 	{
-		if( strcmp( value , "false") == 0)
+		if( strcmp( value , "true") == 0)
 		{
-			main_config->use_bidi = 0 ;
+			main_config->use_bidi = 1 ;
+			main_config->use_ind = 0 ;
 		}
 	}
+	else if( ! main_config->use_ind)
+	{
+		main_config->use_bidi = 1 ;
+	}
 #endif
 
 	main_config->bidi_mode = BIDI_NORMAL_MODE ;
@@ -986,17 +1034,29 @@ x_main_config_init(
 #if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_FRIBIDI)
 	if( ( value = kik_conf_get_value( conf , "bidi_mode")))
 	{
-		main_config->bidi_mode = ml_get_bidi_mode( value) ;
-	}
-#endif
-
-#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_IND)
-	if( ( value = kik_conf_get_value( conf , "use_ind")))
-	{
-		if( strcmp( value , "true") == 0)
+	#if  1
+		/* Compat with 3.3.6 or before. */
+		if( strcmp( value , "cmd_l") == 0)
 		{
-			main_config->use_ind = 1 ;
+			main_config->bidi_mode = BIDI_ALWAYS_LEFT ;
+			main_config->bidi_separators = strdup( " ") ;
 		}
+		else if( strcmp( value , "cmd_r") == 0)
+		{
+			main_config->bidi_mode = BIDI_ALWAYS_RIGHT ;
+			main_config->bidi_separators = strdup( " ") ;
+		}
+		else
+	#endif
+		{
+			main_config->bidi_mode = ml_get_bidi_mode( value) ;
+		}
+	}
+
+	if( ( value = kik_conf_get_value( conf , "bidi_separators")))
+	{
+		free( main_config->bidi_separators) ;
+		main_config->bidi_separators = strdup( value) ;
 	}
 #endif
 
@@ -1279,6 +1339,16 @@ x_main_config_init(
 		}
 	}
 
+	main_config->use_italic_font = 1 ;
+
+	if( ( value = kik_conf_get_value( conf , "use_italic_font")))
+	{
+		if( strcmp( value , "false") == 0)
+		{
+			main_config->use_italic_font = 0 ;
+		}
+	}
+
 	if( ( value = kik_conf_get_value( conf , "hide_underline")))
 	{
 		if( strcmp( value , "true") == 0)
@@ -1358,15 +1428,26 @@ x_main_config_init(
 		x_set_use_leftward_double_drawing( strcmp( value , "true") == 0) ;
 	}
 
+	if( ( value = kik_conf_get_value( conf , "working_directory")))
+	{
+		main_config->work_dir = strdup( value) ;
+	}
+
 #ifdef  USE_GRF
 	if( ( value = kik_conf_get_value( conf , "separate_wall_picture")))
 	{
-		if( strcmp( value , "true") == 0)
-		{
-			extern int  separate_wall_picture ;
+		extern int  separate_wall_picture ;
 
-			separate_wall_picture = 1 ;
-		}
+		separate_wall_picture = (strcmp( value , "true") == 0 ? 1 : 0) ;
+	}
+#endif
+
+#ifdef  USE_FRAMEBUFFER
+	if( ( value = kik_conf_get_value( conf , "rotate_display")))
+	{
+		x_display_rotate(
+			strcmp( value , "right") == 0 ? 1 :
+			( strcmp( value , "left") == 0 ? -1 : 0)) ;
 	}
 #endif
 
@@ -1410,22 +1491,28 @@ x_main_config_final(
 	free( main_config->title) ;
 	free( main_config->icon_name) ;
 	free( main_config->term_type) ;
+	free( main_config->scrollbar_view_name) ;
+	free( main_config->pic_file_path) ;
 	free( main_config->shortcut_strs[0]) ;
 	free( main_config->shortcut_strs[1]) ;
 	free( main_config->shortcut_strs[2]) ;
 	free( main_config->shortcut_strs[3]) ;
-	free( main_config->pic_file_path) ;
-	free( main_config->scrollbar_view_name) ;
 	free( main_config->fg_color) ;
 	free( main_config->bg_color) ;
 	free( main_config->cursor_fg_color) ;
 	free( main_config->cursor_bg_color) ;
+	free( main_config->bd_color) ;
+	free( main_config->it_color) ;
+	free( main_config->ul_color) ;
+	free( main_config->bl_color) ;
+	free( main_config->co_color) ;
 	free( main_config->sb_fg_color) ;
 	free( main_config->sb_bg_color) ;
 	free( main_config->mod_meta_key) ;
 	free( main_config->icon_path) ;
 	free( main_config->input_method) ;
 	free( main_config->init_str) ;
+	free( main_config->bidi_separators) ;
 
 #if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
 	if( main_config->server_list)
@@ -1442,6 +1529,7 @@ x_main_config_final(
 	free( main_config->private_key) ;
 #endif
 
+	free( main_config->work_dir) ;
 	free( main_config->cmd_argv) ;
 
 	return  1 ;
