@@ -6,16 +6,23 @@
 
 #include  "x_picture.h"
 
+#include  <unistd.h>	/* unlink */
+#include  <stdlib.h>	/* system */
+#include  <fcntl.h>
+#include  <sys/stat.h>
 
 #include  <kiklib/kik_debug.h>
 #include  <kiklib/kik_str.h>	/* strdup */
 #include  <kiklib/kik_mem.h>	/* malloc */
 #include  <kiklib/kik_util.h>	/* K_MIN */
+#include  <kiklib/kik_conf_io.h>	/* kik_get_user_rc_path */
 
 /*
  * XXX
  * Don't link libpthread to mlterm for now.
  * Xlib doesn't work on threading without XInitThreads().
+ * Threading is not supported for 8 or less bpp framebuffer imaging
+ * because of x_display_enable_to_change_cmap() and x_display_set_cmap().
  */
 #if ! defined(__CYGWIN__) || ! defined(USE_WIN32GUI)
 #undef  HAVE_PTHREAD
@@ -65,6 +72,8 @@ static x_icon_picture_t **  icon_pics ;
 static u_int  num_of_icon_pics ;
 static x_inline_picture_t *  inline_pics ;
 static u_int  num_of_inline_pics ;
+static u_int  num_of_anims ;
+static int  need_cleanup ;
 
 
 /* --- static functions --- */
@@ -254,23 +263,142 @@ delete_icon_picture(
 	return  1 ;
 }
 
-static void
+static int
+hash_path(
+	char *  path
+	)
+{
+	int  hash ;
+
+	hash = 0 ;
+	while( *path)
+	{
+		hash += *(path++) ;
+	}
+
+	return  hash & 65535 /* 0xffff */ ;
+}
+
+static inline size_t
+get_anim_file_path_len(
+	char *  dir
+	)
+{
+	return  strlen( dir) + 10 + 5 + DIGIT_STR_LEN(int) + 1 ;
+}
+
+static int
+anim_file_exists(
+	char *  file_path ,
+	char *  dir ,
+	int  hash ,
+	int  count
+	)
+{
+	struct stat  st ;
+
+	if( count > 0)
+	{
+		sprintf( file_path , "%sanim%d-%d.gif" , dir , hash , count) ;
+		if( stat( file_path , &st) == 0)
+		{
+			return  1 ;
+		}
+
+		sprintf( file_path , "%sanimx%d-%d.gif" , dir , hash , count) ;
+	}
+	else
+	{
+		sprintf( file_path , "%sanim%d.gif" , dir , hash) ;
+	}
+
+	return  stat( file_path , &st) == 0 ;
+}
+
+/*
+ * XXX
+ * This function should be called synchronously because both load_file_async()
+ * and cleanup_inline_pictures() can call this asynchronously.
+ */
+static int
 delete_inline_picture(
 	x_inline_picture_t *  pic	/* pic->pixmap mustn't be NULL. */
 	)
 {
-	/* pic->disp can be NULL by x_picture_display_closed(). */
+	if( pic->pixmap == DUMMY_PIXMAP)
+	{
+		if( strstr( pic->file_path , "mlterm/anim"))
+		{
+			/* GIF Animation frame */
+			unlink( pic->file_path) ;
+		}
+		else if( pic->disp)
+		{
+			/* loading async */
+			return  0 ;
+		}
+	}
+
+	/* pic->disp can be NULL by x_picture_display_closed() and load_file(). */
 	if( pic->disp)
 	{
-		x_delete_image( pic->disp->display , pic->pixmap) ;
-		x_delete_mask( pic->disp->display , pic->mask) ;
+		if( pic->pixmap != DUMMY_PIXMAP)
+		{
+			x_delete_image( pic->disp->display , pic->pixmap) ;
+			x_delete_mask( pic->disp->display , pic->mask) ;
+		}
+
+		pic->disp = NULL ;
+	}
+
+	if( pic->file_path)
+	{
+		if( strcasecmp( pic->file_path + strlen( pic->file_path) - 4 , ".gif") == 0 &&
+		    /* If check_anim was processed, next_frame == -2. */
+		    pic->next_frame == -1)
+		{
+			char *  dir ;
+			char *  file_path ;
+
+			if( ( dir = kik_get_user_rc_path( "mlterm/")) &&
+			    ( file_path = alloca( get_anim_file_path_len( dir))))
+			{
+				int  hash ;
+				int  count ;
+
+				hash = hash_path( pic->file_path) ;
+
+				for( count = 0 ; ; count++)
+				{
+					if( ! anim_file_exists( file_path , dir , hash , count))
+					{
+						break ;
+					}
+
+					unlink( file_path) ;
+				}
+			}
+
+			free( dir) ;
+		}
 
 		free( pic->file_path) ;
-		pic->disp = NULL ;
+		pic->file_path = NULL ;
 	}
 
 	/* pixmap == None means that the inline picture is empty. */
 	pic->pixmap = None ;
+
+	/*
+	 * Don't next_frame = -1 here because x_animate_inline_pictures() refers it
+	 * even if load_file() fails.
+	 */
+	if( pic->next_frame >= 0)
+	{
+		num_of_anims -- ;
+	}
+
+	return  1 ;
 }
 
 static void
@@ -282,8 +410,7 @@ pty_closed(
 
 	for( count = 0 ; count < num_of_inline_pics ; count++)
 	{
-		if( inline_pics[count].term == term &&
-		    PIXMAP_IS_ACTIVE(inline_pics[count]))
+		if( inline_pics[count].term == term && inline_pics[count].pixmap)
 		{
 		#ifdef  DEBUG
 			kik_debug_printf( KIK_DEBUG_TAG " delete inline picture %d (%d)\n" ,
@@ -303,7 +430,6 @@ cleanup_inline_pictures(
 	)
 {
 #define THRESHOLD  48
-	static int  need_cleanup ;
 	int  count ;
 	int  empty_idx ;
 	u_int8_t *  flags ;
@@ -349,14 +475,19 @@ cleanup_inline_pictures(
 			{
 				for( count = 0 ; count < line->num_of_filled_chars ; count++)
 				{
-					ml_char_t *  comb ;
-					u_int  size ;
+					ml_char_t *  ch ;
 
-					if( ( comb = ml_get_combining_chars( line->chars + count ,
-								&size)) &&
-					    ml_char_cs( comb) == PICTURE_CHARSET)
+					if( ( ch = ml_get_picture_char( line->chars + count)))
 					{
-						flags[INLINEPIC_ID(ml_char_code( comb))] = 1 ;
+						int  idx ;
+
+						idx = ml_char_picture_id( ch) ;
+						do
+						{
+							flags[idx] = 1 ;
+							idx = inline_pics[idx].next_frame ;
+						}
+						while( idx >= 0 && flags[idx] == 0) ;
 					}
 				}
 			}
@@ -385,17 +516,16 @@ cleanup_inline_pictures(
 		{
 			/* do nothing */
 		}
-		else if( inline_pics[count].pixmap != DUMMY_PIXMAP &&
-		         ! flags[count] && inline_pics[count].term == term)
+		else if( ! flags[count] && inline_pics[count].term == term)
 		{
 			/*
 			 * Don't cleanup inline pictures refered twice or more times
-			 * until num_of_inline_pics reaches 32 or more.
+			 * until num_of_inline_pics reaches THRESHOLD or more.
 			 */
-			if( inline_pics[count].ref_count >= 2 &&
+			if( inline_pics[count].weighting >= 2 &&
 			    num_of_inline_pics < THRESHOLD + 8)
 			{
-				inline_pics[count].ref_count /= 2 ;
+				inline_pics[count].weighting /= 2 ;
 
 				continue ;
 			}
@@ -408,7 +538,10 @@ cleanup_inline_pictures(
 					count , num_of_inline_pics) ;
 			#endif
 
-				delete_inline_picture( inline_pics + count) ;
+				if( ! delete_inline_picture( inline_pics + count))
+				{
+					continue ;
+				}
 
 				if( count == num_of_inline_pics - 1)
 				{
@@ -474,17 +607,24 @@ load_file(
 	height = inline_pics[idx].height ;
 
 	if( x_imagelib_load_file( inline_pics[idx].disp , inline_pics[idx].file_path ,
-		NULL , &pixmap , &mask , &width , &height))
+			NULL , &pixmap , &mask , &width , &height))
 	{
+		if( strstr( inline_pics[idx].file_path , "mlterm/anim"))
+		{
+			/* GIF Animation frame */
+			unlink( inline_pics[idx].file_path) ;
+		}
+
 		/* XXX pthread_mutex_lock( &mutex) is necessary. */
-		inline_pics[idx].pixmap = pixmap ;
 		inline_pics[idx].mask = mask ;
 		inline_pics[idx].width = width ;
 		inline_pics[idx].height = height ;
+		inline_pics[idx].pixmap = pixmap ;
 
 	#ifdef  DEBUG
-		kik_debug_printf( KIK_DEBUG_TAG " new inline picture (%s %d %p %p) is created.\n" ,
-			inline_pics[idx].file_path , idx ,
+		kik_debug_printf( KIK_DEBUG_TAG
+			" new inline picture (%s %d %d %d %p %p) is created.\n" ,
+			inline_pics[idx].file_path , idx , width , height , 
 			inline_pics[idx].pixmap , inline_pics[idx].mask) ;
 	#endif
 
@@ -492,9 +632,8 @@ load_file(
 	}
 	else
 	{
-		free( inline_pics[idx].file_path) ;
-		inline_pics[idx].file_path = NULL ;
-		inline_pics[idx].pixmap = None ;	/* Mark as empty */
+		inline_pics[idx].disp = NULL ;
+		delete_inline_picture( inline_pics + idx) ;
 
 		return  0 ;
 	}
@@ -537,6 +676,79 @@ load_file_async(
 
 #endif
 
+static int
+ensure_inline_picture(
+	x_display_t *  disp ,
+	char *  file_path ,
+	u_int *  width ,	/* can be 0 */
+	u_int *  height ,	/* can be 0 */
+	u_int  col_width ,
+	u_int  line_height ,
+	ml_term_t *  term
+	)
+{
+	int  idx ;
+
+	if( ( idx = cleanup_inline_pictures( term)) == -1)
+	{
+		void *  p ;
+
+		/* XXX pthread_mutex_lock( &mutex) is necessary. */
+		if( num_of_inline_pics >= MAX_INLINE_PICTURES ||
+		    ! ( p = realloc( inline_pics ,
+				(num_of_inline_pics + 1) * sizeof(*inline_pics))))
+		{
+			return  -1 ;
+		}
+
+		inline_pics = p ;
+		idx = num_of_inline_pics ++ ;
+	}
+
+	inline_pics[idx].pixmap = None ;	/* mark as empty */
+	inline_pics[idx].file_path = strdup( file_path) ;
+	inline_pics[idx].width = *width ;
+	inline_pics[idx].height = *height ;
+	inline_pics[idx].disp = disp ;
+	inline_pics[idx].term = term ;
+	inline_pics[idx].col_width = col_width ;
+	inline_pics[idx].line_height = line_height ;
+	inline_pics[idx].next_frame = -1 ;
+	/* Don't delete before being inserted to ml_term_t after loading async. */
+	inline_pics[idx].weighting = 2 ;
+
+	return  idx ;
+}
+
+static int
+next_frame_pos(
+	x_inline_picture_t *  prev ,
+	x_inline_picture_t *  next ,
+	int  pos
+	)
+{
+	u_int  cur_rows ;
+	u_int  next_rows ;
+	int  row ;
+	int  col ;
+
+	cur_rows = ( prev->height + prev->line_height - 1) / prev->line_height ;
+	next_rows = ( next->height + next->line_height - 1) / next->line_height ;
+
+	row = pos % cur_rows ;
+	col = pos / cur_rows ;
+
+	if( row < next_rows &&
+	    col < ( next->width + next->col_width - 1) / next->col_width)
+	{
+		return  MAKE_INLINEPIC_POS( col , row , next_rows) ;
+	}
+	else
+	{
+		return  -1 ;
+	}
+}
+
 
 /* --- global functions --- */
 
@@ -597,8 +809,6 @@ x_picture_display_closed(
 			 * Don't set x_inline_picture_t::pixmap = None here because
 			 * this inline picture can still exist in ml_term_t.
 			 */
-
-			free( inline_pics[count].file_path) ;
 			inline_pics[count].disp = NULL ;
 		}
 	}
@@ -844,12 +1054,12 @@ x_load_inline_picture(
 	int  idx ;
 	inline_pic_args_t *  args ;
 
-	/* XXX Don't reuse ~/.mlterm/[pty name].six */
-	if( ! strstr( file_path , "mlterm/"))
+	/* XXX Don't reuse ~/.mlterm/[pty name].six, [pty name].rgs and anim-*.gif */
+	if( ! strstr( file_path , "mlterm/") || strstr( file_path , "mlterm/macro"))
 	{
 		for( idx = 0 ; idx < num_of_inline_pics ; idx++)
 		{
-			if( inline_pics[idx].pixmap &&
+			if( PIXMAP_IS_ACTIVE(inline_pics[idx]) &&
 			    disp == inline_pics[idx].disp &&
 			    strcmp( file_path , inline_pics[idx].file_path) == 0 &&
 			    term == inline_pics[idx].term &&
@@ -861,48 +1071,27 @@ x_load_inline_picture(
 					file_path) ;
 			#endif
 
-				inline_pics[idx].ref_count ++ ;
+				inline_pics[idx].weighting ++ ;
 
-				if( inline_pics[idx].pixmap == DUMMY_PIXMAP)
+				if( strcasecmp( file_path + strlen(file_path) - 4 ,
+					".gif") == 0 &&
+				    /* If check_anim was processed, next_frame == -2. */
+				    inline_pics[idx].next_frame == -1)
 				{
-					return  -1 ;
+					goto  check_anim ;
 				}
 				else
 				{
-					goto  success ;
+					goto  end ;
 				}
 			}
 		}
 	}
 
-	if( ( idx = cleanup_inline_pictures( term)) == -1)
+	if( ( idx = ensure_inline_picture( disp , file_path , width , height ,
+			col_width , line_height , term)) == -1 ||
+	    ! ( args = malloc( sizeof(inline_pic_args_t))))
 	{
-		void *  p ;
-
-		/* XXX pthread_mutex_lock( &mutex) is necessary. */
-		if( ! ( p = realloc( inline_pics ,
-				(num_of_inline_pics + 1) * sizeof(*inline_pics))))
-		{
-			return  -1 ;
-		}
-
-		inline_pics = p ;
-		idx = num_of_inline_pics ++ ;
-	}
-
-	inline_pics[idx].file_path = strdup( file_path) ;
-	inline_pics[idx].width = *width ;
-	inline_pics[idx].height = *height ;
-	inline_pics[idx].disp = disp ;
-	inline_pics[idx].term = term ;
-	inline_pics[idx].col_width = col_width ;
-	inline_pics[idx].line_height = line_height ;
-	inline_pics[idx].ref_count = 1 ;
-
-	if( ! ( args = malloc( sizeof(inline_pic_args_t))))
-	{
-		inline_pics[idx].pixmap = None ;	/* mark as empty */
-
 		return  -1 ;
 	}
 
@@ -953,7 +1142,7 @@ x_load_inline_picture(
 		if( WaitForSingleObject( args->ev , 750) != WAIT_TIMEOUT &&
 		    PIXMAP_IS_ACTIVE(inline_pics[idx]))
 		{
-			goto  success ;
+			goto  check_anim ;
 		}
 	#else
 		if( fds[1] != -1)
@@ -972,7 +1161,7 @@ x_load_inline_picture(
 
 			if( ret != 0 && PIXMAP_IS_ACTIVE(inline_pics[idx]))
 			{
-				goto  success ;
+				goto  check_anim ;
 			}
 		}
 	#endif
@@ -987,13 +1176,67 @@ x_load_inline_picture(
 
 		if( ret)
 		{
-			goto  success ;
+			goto  check_anim ;
 		}
 	}
 
 	return  -1 ;
 
-success:
+check_anim:
+	if( strcasecmp( file_path + strlen(file_path) - 4 , ".gif") == 0)
+	{
+		/* Animation GIF */
+
+		char *  dir ;
+
+		/* mark checked */
+		inline_pics[idx].next_frame = -2 ;
+
+		if( ( dir = kik_get_user_rc_path( "mlterm/")) &&
+		    ( file_path = alloca( get_anim_file_path_len( dir))))
+		{
+			int  hash ;
+			int  count ;
+			int  i ;
+			int  prev_i ;
+
+			hash = hash_path( inline_pics[idx].file_path) ;
+
+			if( anim_file_exists( file_path , dir , hash , 0))
+			{
+				/* The first frame has been already loaded. */
+				unlink( file_path) ;
+			}
+
+			prev_i = idx ;
+			for( count = 1 ; ; count++)
+			{
+				if( ! anim_file_exists( file_path , dir , hash , count))
+				{
+					break ;
+				}
+
+				/*
+				 * Don't clean up because the 1st frame has not been set
+				 * to ml_term_t yet here.
+				 */
+				need_cleanup = 0 ;
+
+				if( ( i = ensure_inline_picture( disp , file_path ,
+						width , height , col_width ,
+						line_height , term)) >= 0 &&
+				    x_add_frame_to_animation( prev_i , i))
+				{
+					inline_pics[i].pixmap = DUMMY_PIXMAP ;
+					prev_i = i ;
+				}
+			}
+		}
+
+		free( dir) ;
+	}
+
+end:
 	*width = inline_pics[idx].width ;
 	*height = inline_pics[idx].height ;
 
@@ -1015,4 +1258,125 @@ x_get_inline_picture(
 	}
 }
 
+int
+x_add_frame_to_animation(
+	int  prev_idx ,
+	int  next_idx
+	)
+{
+	x_inline_picture_t *  prev_pic ;
+	x_inline_picture_t *  next_pic ;
+
+	if( ( prev_pic = x_get_inline_picture( prev_idx)) &&
+	    ( next_pic = x_get_inline_picture( next_idx)) &&
+	    /* Animation is stopped after adding next_idx which equals to prev_pic->next_frame */
+	    prev_pic->next_frame != next_idx &&
+	    /* Don't add a picture which has been already added to an animation. */
+	    next_pic->next_frame < 0)
+	{
+		if( prev_pic->next_frame < 0)
+		{
+			num_of_anims += 2 ;
+			prev_pic->next_frame = next_idx ;
+			next_pic->next_frame = prev_idx ;
+		}
+		else
+		{
+			num_of_anims ++ ;
+			next_pic->next_frame = prev_pic->next_frame ;
+			prev_pic->next_frame = next_idx ;
+		}
+
+		return  1 ;
+	}
+	else
+	{
+		return  0 ;
+	}
+}
+
+int
+x_animate_inline_pictures(
+	ml_term_t *  term
+	)
+{
+	int  wait ;
+	int  row ;
+	ml_line_t *  line ;
+
+	if( ! num_of_anims)
+	{
+		return  0 ;
+	}
+
+	wait = 0 ;
+
+	for( row = 0 ; row < ml_term_get_rows( term) ; row++)
+	{
+		if( ( line = ml_term_get_line_in_screen( term , row)))
+		{
+			int  char_index ;
+
+			for( char_index = 0 ; char_index < line->num_of_filled_chars ;
+			     char_index++)
+			{
+				ml_char_t *  ch ;
+
+				if( ( ch = ml_get_picture_char( line->chars + char_index)))
+				{
+					int32_t  pos ;
+					int  idx ;
+					int  next ;
+
+					pos = ml_char_code( ch) ;
+					idx = ml_char_picture_id( ch) ;
+					if( ( next = inline_pics[idx].next_frame) < 0)
+					{
+						continue ;
+					}
+
+				retry:
+					if( inline_pics[next].pixmap == DUMMY_PIXMAP)
+					{
+						inline_pic_args_t  args ;
+
+						args.idx = next ;
+						if( ! load_file( &args))
+						{
+							if( inline_pics[next].next_frame == idx)
+							{
+								inline_pics[idx].next_frame = -1 ;
+								continue ;
+							}
+
+							next = inline_pics[idx].next_frame =
+								inline_pics[next].next_frame ;
+							goto  retry ;
+						}
+
+						/* shorten waiting time. */
+						wait = 2 ;
+					}
+
+					if( ( pos = next_frame_pos( inline_pics + idx ,
+							inline_pics + next , pos)) >= 0)
+					{
+						ml_char_set_code( ch , pos) ;
+						ml_char_set_picture_id( ch , next) ;
+						ml_line_set_modified( line ,
+							char_index , char_index) ;
+
+						if( wait == 0)
+						{
+							wait = 1 ;
+						}
+					}
+				}
+			}
+		}
+	}
+
+	return  wait ;
+}
+
 #endif	/* NO_IMAGE */
