@@ -65,15 +65,14 @@ typedef struct x_inline_picture
 	u_int8_t  col_width ;
 	u_int8_t  line_height ;
 
-	u_int16_t  ref_count ;
+	int16_t  next_frame ;
+	u_int16_t  weighting ;
 
 } x_inline_picture_t ;
 
 
-#define  INLINEPIC_ID_SHIFT   14
-#define  INLINEPIC_ID(glyph)  ((glyph) >> 14 & 0x3ff)
-#define  INLINEPIC_POS(glyph) ((glyph) & 0x3fff)
-
+#define  MAX_INLINE_PICTURES  (1 << PICTURE_ID_BITS)
+#define  MAKE_INLINEPIC_POS(col , row , num_of_rows)  ((col) * (num_of_rows) + (row))
 
 #ifdef  NO_IMAGE
 
@@ -110,6 +109,10 @@ int  x_load_inline_picture( x_display_t *  disp , char *  file_path ,
 
 x_inline_picture_t *  x_get_inline_picture( int  idx) ;
 
+int  x_add_frame_to_animation( int  prev_idx , int  next_idx) ;
+
+int  x_animate_inline_pictures( ml_term_t *  term) ;
+
 #endif
 
 #define  x_picture_modifier_is_normal(pic_mod) (x_picture_modifiers_equal((pic_mod), NULL))
