@@ -37,9 +37,6 @@ typedef struct x_system_event_listener
 	
 	void  (*pty_closed)( void * , x_screen_ptr_t) ;
 
-	ml_term_t *  (*get_pty)( void * , char *) ;
-	char *  (*pty_list)( void *) ;
-
 	int  (*mlclient)( void * , x_screen_ptr_t , char * , FILE *) ;
 
 	void  (*font_config_updated)(void) ;
@@ -134,6 +131,8 @@ typedef struct  x_screen
 
 	x_icon_picture_t *  icon ;
 
+	int16_t  prev_inline_pic ;
+
 	char  prev_mouse_report_seq[5] ;
 
 	u_int8_t  fade_ratio ;
@@ -143,9 +142,11 @@ typedef struct  x_screen
 	int8_t  use_extended_scroll_shortcut ;
 	int8_t  borderless ;
 	int8_t  font_or_color_config_updated ;	/* 0x1 = font updated, 0x2 = color updated */
-	int8_t  cursor_blink_wait ;
+	int8_t  blink_wait ;
+	int8_t  blink_cursor ;
 	int8_t  hide_underline ;
 	int8_t  processing_vtseq ;
+	int8_t  anim_wait ;
 
 } x_screen_t ;
 
@@ -154,6 +155,8 @@ void  x_exit_backscroll_by_pty( int  flag) ;
 
 void  x_allow_change_shortcut( int  flag) ;
 
+void  x_set_mod_meta_prefix( char *  prefix) ;
+
 #ifdef  USE_IM_CURSOR_COLOR
 void  x_set_im_cursor_color( char *  color) ;
 #endif
@@ -178,10 +181,10 @@ int  x_screen_attached( x_screen_t *  screen) ;
 
 ml_term_t *  x_screen_detach( x_screen_t *  screen) ;
 
-int  x_set_system_listener( x_screen_t *  screen ,
+void  x_set_system_listener( x_screen_t *  screen ,
 	x_system_event_listener_t *  system_listener) ;
 
-int  x_set_screen_scroll_listener( x_screen_t *  screen ,
+void  x_set_screen_scroll_listener( x_screen_t *  screen ,
 	x_screen_scroll_event_listener_t *  screen_scroll_listener) ;
 
 	
