@@ -130,15 +130,18 @@ create_term_intern(void)
 	
 	if( ( term = ml_create_term( main_config.cols , main_config.rows ,
 			main_config.tab_size , main_config.num_of_log_lines ,
-			main_config.encoding , main_config.is_auto_encoding , 
+			main_config.encoding , main_config.is_auto_encoding ,
+			main_config.use_auto_detect , main_config.logging_vt_seq ,
 			main_config.unicode_policy , main_config.col_size_of_width_a ,
 			main_config.use_char_combining , main_config.use_multi_col_char ,
-			main_config.use_bidi , main_config.bidi_mode , main_config.use_ind ,
+			main_config.use_bidi , main_config.bidi_mode ,
+			main_config.bidi_separators , main_config.use_ind ,
 			x_termcap_get_bool_field(
 				x_termcap_get_entry( &termcap , main_config.term_type) , ML_BCE) ,
 			main_config.use_dynamic_comb , main_config.bs_mode ,
 			main_config.vertical_mode , main_config.use_local_echo ,
-			main_config.title , main_config.icon_name)) == NULL)
+			main_config.title , main_config.icon_name ,
+			main_config.alt_color_mode)) == NULL)
 	{
 		return  NULL ;
 	}
@@ -148,16 +151,6 @@ create_term_intern(void)
 		ml_term_set_icon_path( term , main_config.icon_path) ;
 	}
 
-	if( main_config.logging_vt_seq)
-	{
-		ml_term_set_logging_vt_seq( term , 1) ;
-	}
-
-	if( main_config.use_auto_detect)
-	{
-		ml_term_set_use_auto_detect( term , 1) ;
-	}
-
 	if( main_config.unlimit_log_size)
 	{
 		ml_term_unlimit_log_size( term) ;
@@ -383,7 +376,8 @@ open_pty_intern(
 		cmd_argv[1] = NULL ;
 	}
 
-	ret = ml_term_open_pty( term , cmd_path , cmd_argv , env , uri ? uri : display , pass ,
+	ret = ml_term_open_pty( term , cmd_path , cmd_argv , env , uri ? uri : display ,
+			main_config.work_dir , pass ,
 		#ifdef  USE_LIBSSH2
 			main_config.public_key , main_config.private_key
 		#else
@@ -412,6 +406,74 @@ open_pty_intern(
 	return  ret ;
 }
 
+
+#ifndef  NO_IMAGE
+
+static ml_char_t *
+get_picture_data(
+	void *  p ,
+	char *  file_path ,
+	int *  num_of_cols ,	/* can be 0 */
+	int *  num_of_rows	/* can be 0 */
+	)
+{
+	ml_char_t *  data ;
+
+	if( num_of_screens > 0)
+	{
+		ml_term_t *  orig_term ;
+
+		orig_term = screens[0]->term ;
+		screens[0]->term = p ;		/* XXX */
+		data = (*screens[0]->xterm_listener.get_picture_data)(
+				screens[0]->xterm_listener.self ,
+				file_path , num_of_cols , num_of_rows) ;
+		screens[0]->term = orig_term ;
+	}
+	else
+	{
+		data = NULL ;
+	}
+
+	return  data ;
+}
+
+static ml_term_t *
+detach_screen(
+	x_screen_t *  screen
+	)
+{
+	ml_term_t *  term ;
+
+	if( ( term = x_screen_detach( screen)))
+	{
+		ml_xterm_event_listener_t *  listener ;
+
+		if( ! ( listener = ml_term_get_user_data( term , term)))
+		{
+			if( ! ( listener = calloc( 1 , sizeof(ml_xterm_event_listener_t))))
+			{
+				return  term ;
+			}
+
+			listener->self = term ;
+			listener->get_picture_data = get_picture_data ;
+
+			ml_term_set_user_data( term , term , listener) ;
+		}
+
+		/* XXX */
+		term->parser->xterm_listener = listener ;
+	}
+
+	return  term ;
+}
+
+#define  x_screen_detach( screen)  detach_screen( screen)
+
+#endif
+
+
 static int
 close_screen_intern(
 	x_screen_t *  screen
@@ -514,8 +576,8 @@ open_screen_intern(
 		main_config.type_engine , main_config.font_present ,
 		main_config.font_size , usascii_font_cs ,
 		main_config.use_multi_col_char ,
-		main_config.step_in_changing_font_size ,
-		main_config.letter_space , main_config.use_bold_font)) == NULL)
+		main_config.step_in_changing_font_size , main_config.letter_space ,
+		main_config.use_bold_font , main_config.use_italic_font)) == NULL)
 	{
 		char *  name ;
 
@@ -535,7 +597,9 @@ open_screen_intern(
 	if( ( color_man = x_color_manager_new( disp ,
 				main_config.fg_color , main_config.bg_color ,
 				main_config.cursor_fg_color , main_config.cursor_bg_color ,
-				main_config.bd_color , main_config.ul_color)) == NULL)
+				main_config.bd_color , main_config.it_color ,
+				main_config.ul_color , main_config.bl_color ,
+				main_config.co_color)) == NULL)
 	{
 		goto  error ;
 	}
@@ -564,13 +628,9 @@ open_screen_intern(
 
 	/* Override config event listener. */
 	screen->config_listener.saved = config_saved ;
-	
-	if( ! x_set_system_listener( screen , &system_listener))
-	{
-		goto  error ;
-	}
 
-#ifndef  __ANDROID__
+	x_set_system_listener( screen , &system_listener) ;
+
 	if( main_config.use_scrollbar &&
 	    ( sb_screen = x_sb_screen_new( screen ,
 				main_config.scrollbar_view_name ,
@@ -580,7 +640,6 @@ open_screen_intern(
 		root = &sb_screen->window ;
 	}
 	else
-#endif
 	{
 		root = &screen->window ;
 	}
@@ -658,7 +717,7 @@ open_screen_intern(
 		if( main_config.init_str)
 		{
 			ml_term_write( term , main_config.init_str ,
-				strlen( main_config.init_str) , 0) ;
+				strlen( main_config.init_str)) ;
 		}
 	}
 
@@ -1012,23 +1071,6 @@ close_screen(
 	}
 }
 
-static ml_term_t *
-get_pty(
-	void *  p ,
-	char *  dev
-	)
-{
-	return  ml_get_term( dev) ;
-}
-
-static char *
-pty_list(
-	void *  p
-	)
-{
-	return  ml_get_pty_list() ;
-}
-
 static int
 mlclient(
 	void *  self ,
@@ -1123,7 +1165,7 @@ mlclient(
 		
 		x_prepare_for_main_config(conf) ;
 
-		if( ! kik_conf_parse_args( conf , &argc , &argv))
+		if( ! kik_conf_parse_args( conf , &argc , &argv , 1))
 		{
 			kik_conf_delete( conf) ;
 
@@ -1283,8 +1325,6 @@ x_screen_manager_init(
 	system_listener.prev_pty = prev_pty ;
 	system_listener.close_pty = close_pty ;
 	system_listener.pty_closed = pty_closed ;
-	system_listener.get_pty = get_pty ;
-	system_listener.pty_list = pty_list ;
 	system_listener.mlclient = mlclient ;
 	system_listener.font_config_updated = font_config_updated ;
 	system_listener.color_config_updated = color_config_updated ;
