@@ -102,13 +102,14 @@ read_conf(
 		 */
 		if( ! x_shortcut_parse( shortcut , key , value))
 		{
+		#ifdef  ENABLE_BACKWARD_COMPAT
 			/*
 			 * XXX
 			 * Backward compatibility with 2.4.0 or before.
 			 * [operation]=[shortcut key]
 			 */
-			 
 			x_shortcut_parse( shortcut , value , key) ;
+		#endif
 		}
 	}
 
@@ -413,70 +414,9 @@ x_shortcut_parse(
 		char *  p ;
 		x_str_key_t *  str_map ;
 
-		if( ( str = malloc( strlen( oper))) == NULL)
-		{
-			return  0 ;
-		}
-
-		p = str ;
-
-		oper ++ ;
-
-		while( *oper != '"' && *oper != '\0')
-		{
-			u_int  digit ;
-
-			if( sscanf( oper , "\\x%2x" , &digit) == 1)
-			{
-				*p = (char)digit ;
-
-				oper += 4 ;
-			}
-			else
-			{
-				if( *oper == '\\')
-				{
-					oper ++ ;
-
-					if( *oper == '\0')
-					{
-						break ;
-					}
-					else if( *oper == 'n')
-					{
-						*p = '\n' ;
-					}
-					else if( *oper == 'r')
-					{
-						*p = '\r' ;
-					}
-					else if( *oper == 't')
-					{
-						*p = '\t' ;
-					}
-					else if( *oper == 'e')
-					{
-						*p = '\033' ;
-					}
-					else
-					{
-						*p = *oper ;
-					}
-				}
-				else
-				{
-					*p = *oper ;
-				}
-
-				oper ++ ;
-			}
-
-			p ++ ;
-		}
-
-		*p = '\0' ;
-
-		if( ! ( str_map = realloc( shortcut->str_map ,
+		if( ! ( str = kik_str_unescape( ++oper)) ||
+		    ! ( p = strrchr( str , '\"')) ||
+		    ! ( str_map = realloc( shortcut->str_map ,
 				   sizeof( x_str_key_t) * (shortcut->str_map_size + 1))))
 		{
 			free( str) ;
@@ -484,6 +424,7 @@ x_shortcut_parse(
 			return  0 ;
 		}
 
+		*p = '\0' ;
 		str_map[shortcut->str_map_size].ksym = ksym ;
 		str_map[shortcut->str_map_size].state = state ;
 		str_map[shortcut->str_map_size].str = str ;
