@@ -32,6 +32,7 @@
 #include <kiklib/kik_unistd.h>
 #include <kiklib/kik_str.h>	/* strdup */
 #include <kiklib/kik_util.h>	/* DIGIT_STR_LEN */
+#include <kiklib/kik_mem.h>
 
 #include "x_display.h"		/* x_display_get_visual_info */
 
@@ -451,8 +452,8 @@ load_file(
 			return  NULL ;
 		}
 
-		/* XXX Don't cache ~/.mlterm/[pty name].six. */
-		if( ! strstr( path , "mlterm/"))
+		/* XXX Don't cache ~/.mlterm/[pty name].six, [pty name].rgs and anim-*.gif */
+		if( ! strstr( path , "mlterm/") || strstr( path , "mlterm/macro"))
 		{
 		#ifdef  __DEBUG
 			kik_warn_printf(KIK_DEBUG_TAG " adding a pixbuf to cache(%s)\n" , path) ;
